@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
  */
 
 #include <string>
-#include "config.h"
-#include "menudef.h"
 void SetVal(const std::string& secname, const std::string& preval, const std::string& val);
 
 #include <SDL_video.h>
@@ -36,7 +34,6 @@ void Mount_Img_HDD(char drive, std::string realpath);
 void DOSBox_SetMenu(void);
 void DOSBox_NoMenu(void);
 void DOSBox_RefreshMenu(void);
-void ToggleMenu(bool pressed);
 void DOSBox_CheckOS(int &id, int &major, int &minor);
 void MountDrive(char drive, const char drive2[DOS_PATHLENGTH]);
 void MountDrive_2(char drive, const char drive2[DOS_PATHLENGTH], std::string drive_type);
@@ -422,6 +419,7 @@ class DOSBoxMenu {
     public:
         bool                            item_exists(const item_handle_t i);
         bool                            item_exists(const std::string &name);
+        std::vector<item>               get_master_list(void);
         item&                           get_item(const item_handle_t i);
         item&                           get_item(const std::string &name);
         item_handle_t                   get_item_id_by_name(const std::string &name);
@@ -448,9 +446,9 @@ class DOSBoxMenu {
     public:
         HMENU                           getWinMenu(void) const;
         bool                            mainMenuWM_COMMAND(unsigned int id);
+#endif
     public:
         static constexpr unsigned int   winMenuMinimumID = 0x1000;
-#endif
 #if DOSBOXMENU_TYPE == DOSBOXMENU_NSMENU /* Mac OS X NSMenu / NSMenuItem handle */
     protected:
         void*                           nsMenu = NULL;
@@ -521,4 +519,3 @@ extern DOSBoxMenu mainMenu;
 void DOSBox_SetMenu(DOSBoxMenu &altMenu);
 
 #endif /* MENU_DOSBOXMENU_H */
-
