@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,15 +22,9 @@
 // set to 1 for debug messages and debugging log:
 #define PARALLEL_DEBUG 0
 
-#ifndef DOSBOX_DOSBOX_H
-#include "dosbox.h"
-#endif
-#ifndef DOSBOX_INOUT_H
 #include "inout.h"
-#endif
 
-#include "control.h"
-#include "dos_inc.h"
+#include "programs.h"
 
 class device_LPT : public DOS_Device {
 public:
@@ -97,8 +91,6 @@ public:
 };
 
 extern CParallel* parallelPortObjects[];
-
-const uint16_t parallel_baseaddr[3] = {0x378,0x278,0x3bc};
+extern uint16_t parallel_baseaddr[9];
 
 #endif
-
