@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,22 +20,7 @@
 #ifndef DOSBOX_PROGRAMS_H
 #define DOSBOX_PROGRAMS_H
 
-#ifndef DOSBOX_DOSBOX_H
-#include "dosbox.h"
-#endif
-#ifndef DOSBOX_DOS_INC_H
 #include "dos_inc.h"
-#endif
-
-#ifndef CH_LIST
-#define CH_LIST
-#include <list>
-#endif
-
-#ifndef CH_STRING
-#define CH_STRING
-#include <string>
-#endif
 
 class CommandLine {
 public:
@@ -48,15 +33,16 @@ public:
 	};
 public:
 	CommandLine(int argc,char const * const argv[],enum opt_style opt=CommandLine::either);
-	CommandLine(char const * const name,char const * const cmdline,enum opt_style opt=CommandLine::either);
+	CommandLine(char const * const name,char const * const cmdline,enum opt_style opt=CommandLine::either,bool squote=false);
 	const char * GetFileName(){ return file_name.c_str();}
 
 	bool FindExist(char const * const name,bool remove=false);
-	bool FindHex(char const * const name,int & value,bool remove=false);
+	bool FindHex(char const * const name,unsigned int & value,bool remove=false);
 	bool FindInt(char const * const name,int & value,bool remove=false);
 	bool FindString(char const * const name,std::string & value,bool remove=false);
 	bool FindCommand(unsigned int which,std::string & value);
 	bool FindStringBegin(char const * const begin,std::string & value, bool remove=false);
+	bool FindStringFullBegin(char const * const begin,std::string & value, bool squote, bool remove=false);
 	bool FindStringRemain(char const * const name,std::string & value);
 	bool FindStringRemainBegin(char const * const name,std::string & value);
 	bool GetStringRemain(std::string & value);
@@ -100,8 +86,8 @@ class Program {
 public:
 	Program();                                          //! Constructor
 	virtual ~Program(){                                 //! Default destructor
-		if (cmd != NULL) delete cmd;
-		if (psp != NULL) delete psp;
+		delete cmd;
+		delete psp;
 	}
 
     /*! \brief      Exit status of the program
@@ -124,6 +110,6 @@ public:
 };
 
 typedef void (PROGRAMS_Main)(Program * * make);
-void PROGRAMS_MakeFile(char const * const name,PROGRAMS_Main * SDL_main);
+void PROGRAMS_MakeFile(char const * const name,PROGRAMS_Main * SDL_main,const char *dir="");
 
 #endif
