@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,21 +20,8 @@
 #ifndef DOSBOX_SERIALPORT_H
 #define DOSBOX_SERIALPORT_H
 
-#ifndef DOSBOX_DOSBOX_H
-#include "dosbox.h"
-#endif
-#ifndef DOSBOX_INOUT_H
 #include "inout.h"
-#endif
-#ifndef DOSBOX_TIMER_H
-#include "timer.h"
-#endif
-#ifndef DOSBOX_DOS_INC_H
-#include "dos_inc.h"
-#endif
-#ifndef DOSBOX_PROGRAMS_H
 #include "programs.h"
-#endif
 
 // set this to 1 for serial debugging in release mode
 #define SERIAL_DBG_FORCED 1
@@ -426,9 +413,9 @@ private:
 };
 
 extern CSerial* serialports[];
-const uint8_t serial_defaultirq[] = { 4, 3, 4, 3 };
-const uint16_t serial_baseaddr[] = {0x3f8,0x2f8,0x3e8,0x2e8};
-const char* const serial_comname[]={"COM1","COM2","COM3","COM4"};
+const uint8_t serial_defaultirq[] = { 4, 3, 4, 3, 0, 0, 0, 0, 0 };
+const uint16_t serial_defaultaddr[] = {0x3f8,0x2f8,0x3e8,0x2e8,0,0,0,0,0};
+const char* const serial_comname[]={"COM1","COM2","COM3","COM4","COM5","COM6","COM7","COM8","COM9"};
 
 // the COM devices
 
