@@ -25,9 +25,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-#include "dosbox.h"
 #include "logging.h"
-#include "SDL.h"
 #include "support.h"
 #include "cdrom.h"
 
