@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,7 @@
 #endif
 
 #include "drives.h"
+#include "logging.h"
 #include "support.h"
 #include "setup.h"
 #include "src/libs/decoders/audio_convert.c"
@@ -53,6 +54,27 @@
 #include "src/libs/decoders/wav.c"
 #include "src/libs/decoders/mp3_seek_table.cpp"
 #include "src/libs/decoders/mp3.cpp"
+#include "src/libs/libchdr/chd.h"
+#include "src/libs/libchdr/libchdr_chd.c"
+#include "src/libs/libchdr/libchdr_cdrom.c"
+#include "src/libs/libchdr/libchdr_flac.c"
+#include "src/libs/libchdr/libchdr_huffman.c"
+#include "src/libs/libchdr/libchdr_bitstream.c"
+#include "src/libs/libchdr/FLAC/stream_decoder.c"
+#include "src/libs/libchdr/FLAC/bitreader.c"
+#include "src/libs/libchdr/FLAC/format.c"
+#include "src/libs/libchdr/FLAC/cpu.c"
+#include "src/libs/libchdr/FLAC/crc.c"
+#include "src/libs/libchdr/FLAC/fixed.c"
+#include "src/libs/libchdr/FLAC/lpc.c"
+#include "src/libs/libchdr/FLAC/md5.c"
+#include "src/libs/libchdr/FLAC/memory.c"
+#if defined(WIN32) && !defined(HX_DOS)
+#include "src/libs/libchdr/FLAC/windows_unicode_filenames.c"
+#endif
+#include "src/libs/libchdr/lzma/LzmaDec.c"
+#include "src/libs/libchdr/lzma/LzmaEnc.c"
+#include "src/libs/libchdr/lzma/LzFind.c"
 
 using namespace std;
 
@@ -90,6 +112,17 @@ CDROM_Interface_Image::BinaryFile::BinaryFile(const char *filename, bool &error)
 	file = new ifstream(filename, ios::in | ios::binary);
 	// If new fails, an exception is generated and scope leaves this constructor
 	error = file->fail();
+#if defined(WIN32) && !defined(__MINGW32__) // Wengier: disable for MinGW for now but it appears to work on my MinGW version
+    if (error) {
+        typedef wchar_t host_cnv_char_t;
+        host_cnv_char_t *CodePageGuestToHost(const char *s);
+        const host_cnv_char_t* host_name = CodePageGuestToHost(filename);
+        if (host_name != NULL) {
+            file = new ifstream(host_name, ios::in | ios::binary);
+            error = file->fail();
+        }
+    }
+#endif
 }
 
 CDROM_Interface_Image::BinaryFile::~BinaryFile()
@@ -102,17 +135,18 @@ CDROM_Interface_Image::BinaryFile::~BinaryFile()
 	file = nullptr;
 }
 
-bool CDROM_Interface_Image::BinaryFile::read(uint8_t *buffer, int seek, int count)
+bool CDROM_Interface_Image::BinaryFile::read(uint8_t *buffer,int64_t offset, int count)
 {
-	file->seekg(seek, ios::beg);
+    if (!seek(offset)) return false;
+	file->seekg((streampos)offset, ios::beg);
 	file->read((char*)buffer, count);
 	return !(file->fail());
 }
 
-int CDROM_Interface_Image::BinaryFile::getLength()
+int64_t CDROM_Interface_Image::BinaryFile::getLength()
 {
 	file->seekg(0, ios::end);
-	int length = (int)file->tellg();
+	int64_t length = (int64_t)file->tellg();
 	if (file->fail()) return -1;
 	return length;
 }
@@ -127,16 +161,32 @@ uint16_t CDROM_Interface_Image::BinaryFile::getEndian()
 	#endif
 }
 
-bool CDROM_Interface_Image::BinaryFile::seek(uint32_t offset)
+bool CDROM_Interface_Image::BinaryFile::seek(int64_t offset)
 {
-	file->seekg(offset, ios::beg);
+	const auto pos = static_cast<std::streamoff>(offset);
+	if (file->tellg() == pos)
+		return true;
+
+	file->seekg(pos, std::ios::beg);
+
+	// If the first seek attempt failed, then try harder
+	if (file->fail()) {
+		file->clear();                   // clear fail and eof bits
+		file->seekg(0, std::ios::beg);   // "I have returned."
+		file->seekg(pos, std::ios::beg); // "It will be done."
+	}
 	return !file->fail();
 }
 
 uint16_t CDROM_Interface_Image::BinaryFile::decode(uint8_t *buffer)
 {
+    if (static_cast<uint32_t>(file->tellg()) != audio_pos)
+		if (!seek(audio_pos)) return 0;
+
 	file->read((char*)buffer, chunkSize);
-	return (uint16_t)file->gcount();
+    const uint16_t bytes_read = (uint16_t)file->gcount();
+    audio_pos += bytes_read;
+	return bytes_read;
 }
 
 CDROM_Interface_Image::AudioFile::AudioFile(const char *filename, bool &error)
@@ -176,14 +226,22 @@ CDROM_Interface_Image::AudioFile::~AudioFile()
  *  or number of channels.  To do this, we convert the byte offset to a
  *  time-offset, and use the Sound_Seek() function to move the read position.
  */
-bool CDROM_Interface_Image::AudioFile::seek(uint32_t offset)
+bool CDROM_Interface_Image::AudioFile::seek(int64_t offset)
 {
 	#ifdef DEBUG
 	const auto begin = std::chrono::steady_clock::now();
 	#endif
 
+	if (audio_pos == (uint32_t)offset) {
+#ifdef DEBUG
+		LOG_MSG("CDROM: seek to %u avoided with position-tracking", offset);
+#endif
+		return true;
+    }
+
 	// Convert the byte-offset to a time offset (milliseconds)
 	const bool result = Sound_Seek(sample, lround(offset/176.4f));
+	audio_pos = result ? (uint32_t)offset : UINT32_MAX;
 
 	#ifdef DEBUG
 	const auto end = std::chrono::steady_clock::now();
@@ -196,6 +254,7 @@ bool CDROM_Interface_Image::AudioFile::seek(uint32_t offset)
 uint16_t CDROM_Interface_Image::AudioFile::decode(uint8_t *buffer)
 {
 	const uint16_t bytes = Sound_Decode(sample);
+    audio_pos += bytes;
 	memcpy(buffer, sample->buffer, bytes);
 	return bytes;
 }
@@ -223,9 +282,9 @@ uint8_t CDROM_Interface_Image::AudioFile::getChannels()
 	return channels;
 }
 
-int CDROM_Interface_Image::AudioFile::getLength()
+int64_t CDROM_Interface_Image::AudioFile::getLength()
 {
-	int length(-1);
+	int64_t length(-1);
 
 	// GetDuration returns milliseconds ... but getLength needs Red Book bytes.
 	const int duration_ms = Sound_GetDuration(sample);
@@ -233,15 +292,176 @@ int CDROM_Interface_Image::AudioFile::getLength()
 		// ... so convert ms to "Red Book bytes" by multiplying with 176.4f,
 		// which is 44,100 samples/second * 2-channels * 2 bytes/sample
 		// / 1000 milliseconds/second
-		length = (int)round(duration_ms * 176.4f);
+		length = (int64_t)round(duration_ms * 176.4f);
 	}
     #ifdef DEBUG
-    LOG_MSG("%s CDROM: AudioFile::getLength is %d bytes", get_time(), length);
+    LOG_MSG("%s CDROM: AudioFile::getLength is %ld bytes", get_time(), length);
     #endif
 
 	return length;
 }
 
+CDROM_Interface_Image::CHDFile::CHDFile(const char* filename, bool& error)
+    :TrackFile(RAW_SECTOR_SIZE) // CDAudioCallBack needs 2352
+{
+    error = chd_open(filename, CHD_OPEN_READ, NULL, &this->chd) != CHDERR_NONE;
+    if (!error) {
+        this->header           = chd_get_header(this->chd);
+        this->hunk_buffer      = new uint8_t[this->header->hunkbytes];
+        this->hunk_buffer_next = new uint8_t[this->header->hunkbytes];
+    }
+}
+
+CDROM_Interface_Image::CHDFile::~CHDFile()
+{
+    // Guard: only cleanup if needed
+    if (this->chd) {
+        chd_close(this->chd);
+        this->chd = nullptr;
+    }
+
+    if (this->hunk_buffer) {
+        delete[] this->hunk_buffer;
+        this->hunk_buffer = nullptr;
+    }
+
+#if !defined(HX_DOS) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
+    if (this->hunk_thread) {
+        this->hunk_thread->join();
+        delete hunk_thread;
+        this->hunk_thread = nullptr;
+    }
+#endif
+}
+
+#if !defined(HX_DOS) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
+void hunk_thread_func(chd_file* chd, int hunk_index, uint8_t* buffer, bool* error)
+{
+    // loads one hunk into buffer
+    *error = chd_read(chd, hunk_index, buffer) != CHDERR_NONE;
+}
+#endif
+
+bool CDROM_Interface_Image::CHDFile::read(uint8_t* buffer,int64_t offset, int count)
+{
+    // we can not read more than a single sector currently
+    if (count > RAW_SECTOR_SIZE) {
+        return false;
+    }
+
+    uint64_t needed_hunk = (uint64_t)offset / (uint64_t)this->header->hunkbytes;
+
+    // EOF
+    if (needed_hunk > this->header->totalhunks) {
+        return false;
+    }
+
+    // read new hunk if needed
+    if ((int)needed_hunk != this->hunk_buffer_index) {
+#if defined(HX_DOS) || defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR)
+        if (chd_read(this->chd, needed_hunk, this->hunk_buffer) != CHDERR_NONE)
+            return false;
+#else
+        // make sure our thread is done
+        if (this->hunk_thread) this->hunk_thread->join();
+
+        // can we use our prefetched hunk
+        if (((int)needed_hunk == (this->hunk_buffer_index + 1)) && (!this->hunk_thread_error)) {
+            // swap pointers and we're good :)
+            std::swap(this->hunk_buffer, this->hunk_buffer_next);
+
+        } else {
+            // read new hunk
+            bool error = true;
+            hunk_thread_func(this->chd, (int)needed_hunk, this->hunk_buffer, &error);
+            if (error) {
+                return false;
+            }
+        }
+#endif
+
+        this->hunk_buffer_index = (int)needed_hunk;
+
+#if !defined(HX_DOS) && !(defined(__MINGW32__) && !defined(__MINGW64_VERSION_MAJOR))
+        // prefetch: let our thread decode the next hunk
+        if (hunk_thread) delete this->hunk_thread;
+        this->hunk_thread = new std::thread(
+            [this, needed_hunk]() {
+                hunk_thread_func(this->chd, (int)(needed_hunk + 1), this->hunk_buffer_next, &this->hunk_thread_error);
+            }
+        );
+#endif
+    }
+
+    // copy data
+    // the overlying read code thinks there is a sync header
+    // so for 2048 sector size images we need to subtract 16 from the offset to account for the missing sync header
+    uint8_t* source = this->hunk_buffer + ((uint64_t)offset - (uint64_t)needed_hunk * this->header->hunkbytes) - ((uint64_t)16 * this->skip_sync);
+    memcpy(buffer, source, min(count, RAW_SECTOR_SIZE));
+
+    return true;
+}
+
+int64_t CDROM_Interface_Image::CHDFile::getLength()
+{
+    return this->header->logicalbytes;
+}
+
+uint16_t CDROM_Interface_Image::CHDFile::getEndian()
+{
+    // CHD: no idea about this, chaning this did not fix the cd audio noise
+    // Image files are read into native-endian byte-order
+#if defined(WORDS_BIGENDIAN)
+    return AUDIO_S16MSB;
+#else
+    return AUDIO_S16LSB;
+#endif
+}
+
+bool CDROM_Interface_Image::CHDFile::seek(int64_t offset)
+{
+    // only checks if seek range is valid ? only used for audio ?
+    // only used by PlayAudioSector ?
+    if ((uint32_t)((uint64_t)offset / this->header->hunkbytes) < this->header->hunkcount) {
+        return true;
+    } else {
+        return false;
+    }
+}
+
+static void Endian_A16_Swap(void* src, uint32_t nelements)
+{
+    uint32_t i;
+    uint8_t* nsrc = (uint8_t*)src;
+
+    for (i = 0; i < nelements; i++)
+    {
+        uint8_t tmp = nsrc[i * 2];
+
+        nsrc[i * 2] = nsrc[i * 2 + 1];
+        nsrc[i * 2 + 1] = tmp;
+    }
+}
+
+uint16_t CDROM_Interface_Image::CHDFile::decode(uint8_t* buffer)
+{
+    // reads one sector of CD audio ?
+
+    assert(this->audio_pos % 2448 == 0);
+
+    if (this->read(buffer, this->audio_pos, RAW_SECTOR_SIZE)) {
+        // chd uses 2448
+        this->audio_pos += 2448;
+
+        // no idea if other platforms need this but on windows this is needed or there is only noise
+        Endian_A16_Swap(buffer, 588 * 2);
+
+        // we only read the raw audio nothing else
+        return RAW_SECTOR_SIZE;
+    }
+    return 0;
+}
+
 // initialize static members
 int CDROM_Interface_Image::refCount = 0;
 CDROM_Interface_Image* CDROM_Interface_Image::images[26] = {};
@@ -276,12 +496,13 @@ CDROM_Interface_Image::~CDROM_Interface_Image()
 	}
 }
 
+extern bool qmount;
 bool CDROM_Interface_Image::SetDevice(char* path, int forceCD)
 {
 	(void)forceCD;//UNUSED
-	const bool result = LoadCueSheet(path) || LoadIsoFile(path);
-	if (!result) {
-		// print error message on dosbox console
+	const bool result = LoadCueSheet(path) || LoadIsoFile(path) || LoadChdFile(path);
+	if (!result && !qmount) {
+		// print error message on dosbox-x console
 		char buf[MAX_LINE_LENGTH];
 		snprintf(buf, MAX_LINE_LENGTH, "Could not load image file: %s\r\n", path);
 		uint16_t size = (uint16_t)strlen(buf);
@@ -398,20 +619,20 @@ bool CDROM_Interface_Image::PlayAudioSector(unsigned long start, unsigned long l
 
 	// The CDROM Red Book standard allows up to 99 tracks, which includes the data track
 	if ( track < 0 || track > 99 )
-		LOG(LOG_MISC, LOG_WARN)("Game tried to load track #%d, which is invalid", track);
+		LOG_MSG("Game tried to load track #%d, which is invalid", track);
 
 	// Attempting to play zero sectors is a no-op
 	else if (len == 0)
-		LOG(LOG_MISC, LOG_WARN)("Game tried to play zero sectors, skipping");
+		LOG_MSG("Game tried to play zero sectors, skipping");
 
 	// The maximum storage achieved on a CDROM was ~900MB or just under 100 minutes
 	// with overburning, so use this threshold to sanity-check the start sector.
 	else if (start > 450000)
-		LOG(LOG_MISC, LOG_WARN)("Game tried to read sector %lu, which is beyond the 100-minute maximum of a CDROM", start);
+		LOG_MSG("Game tried to read sector %lu, which is beyond the 100-minute maximum of a CDROM", start);
 
 	// We can't play audio from a data track (as it would result in garbage/static)
 	else if(track >= 0 && tracks[track].attr == 0x40)
-		LOG(LOG_MISC,LOG_WARN)("Game tries to play the data track. Not doing this");
+		LOG_MSG("Game tries to play the data track. Not doing this");
 
 	// Checks passed, setup the audio stream
 	else {
@@ -423,6 +644,7 @@ bool CDROM_Interface_Image::PlayAudioSector(unsigned long start, unsigned long l
 
 		// only initialize the player elements if our track is playable
 		if (is_playable) {
+            trackFile->setAudioPosition(offset);
 			const uint8_t channels = trackFile->getChannels();
 			const uint32_t rate = trackFile->getRate();
 
@@ -560,7 +782,7 @@ bool CDROM_Interface_Image::ReadSector(uint8_t *buffer, bool raw, unsigned long
 	int track = GetTrack(sector) - 1;
 	if (track < 0) return false;
 
-	int seek = tracks[track].skip + (sector - tracks[track].start) * tracks[track].sectorSize;
+	int64_t seek = (int64_t)tracks[track].skip + ((int64_t)(sector - tracks[track].start)) * (int64_t)tracks[track].sectorSize;
 	int length = (raw ? RAW_SECTOR_SIZE : COOKED_SECTOR_SIZE);
 	if (tracks[track].sectorSize != RAW_SECTOR_SIZE && raw) return false;
 	if ((tracks[track].sectorSize == RAW_SECTOR_SIZE || tracks[track].sectorSize == 2448) && !tracks[track].mode2 && !raw) seek += 16;
@@ -635,7 +857,10 @@ void CDROM_Interface_Image::CDAudioCallBack(Bitu len)
 				// uses either the stereo or mono and native or nonnative AddSamples call assigned during construction
 				(player.channel->*player.addSamples)(requested / bytes_per_request, (int16_t*)(player.buffer + player.bufferConsumed) );
 				player.bufferConsumed += requested;
+
 				player.playbackRemaining -= requested;
+				if (player.playbackRemaining < 0)
+					player.playbackRemaining = 0;
 
 				// Games can query the current Red Book MSF frame-position, so we keep that up-to-date here.
 				// We scale the final number of frames by the percent complete, which
@@ -730,7 +955,8 @@ bool CDROM_Interface_Image::LoadIsoFile(char* filename)
         track.file = NULL;
 		return false;
 	}
-	track.length = track.file->getLength() / track.sectorSize;
+    int64_t len=track.file->getLength();
+	track.length = (int)(len / track.sectorSize);
 	// LOG_MSG("LoadIsoFile: %s, track 1, 0x40, sectorSize=%d, mode2=%s", filename, track.sectorSize, track.mode2 ? "true":"false");
 
 	tracks.push_back(track);
@@ -759,7 +985,7 @@ bool CDROM_Interface_Image::CanReadPVD(TrackFile *file, int sectorSize, bool mod
 
 #if defined(WIN32)
 static string dirname(char * file) {
-	char * sep = strrchr(file, '\\');
+	char * sep = strrchr_dbcs(file, '\\');
 	if (sep == nullptr)
 		sep = strrchr(file, '/');
 	if (sep == nullptr)
@@ -906,7 +1132,148 @@ bool CDROM_Interface_Image::LoadCueSheet(char *cuefile)
 	return true;
 }
 
+std::vector<string> split_string_to_list(const std::string& str, const std::string& delim)
+{
+    std::vector<string> tokens;
+    size_t prev = 0, pos = 0;
+    do
+    {
+        pos = str.find(delim, prev);
+        if (pos == string::npos) pos = str.length();
+        string token = str.substr(prev, pos - prev);
+        if (!token.empty()) tokens.push_back(token);
+        prev = pos + delim.length();
+    } while (pos < str.length() && prev < str.length());
+    return tokens;
+}
 
+bool CDROM_Interface_Image::LoadChdFile(char* chdfile)
+{
+    /*
+        ToDo:
+            - check if this is a CD and not an HDD CHD
+            - check for CDROM_TRACK_METADATA_TAG or CDROM_OLD_METADATA_TAG (?)
+    */
+    tracks.clear();
+    bool     error       = true;
+    CHDFile* file        = new CHDFile(chdfile, error);
+    Track    track       = { 0, 0, 0, 0, 0, 0, false, file };
+    int      shift       = 0;
+    int      currPregap  = 0;
+    int      totalPregap = 0;
+    int      prestart    = -1;
+    if (!error) {
+        // iterate over all track entries
+        char   metadata[256];
+        int    index        = 0;
+        UINT32 count        = 0;
+        UINT32 tag          = 0;
+        UINT8  flags        = 0;
+        int    total_frames = 0;
+        while (chd_get_metadata(file->getChd(), CDROM_TRACK_METADATA2_TAG, index, &metadata, 256, &count, &tag, &flags) == CHDERR_NONE) {
+            // parse track
+            // TRACK:1 TYPE:MODE1_RAW SUBTYPE:NONE FRAMES:206931 PREGAP:0 P
+            std::vector<string> tokens = split_string_to_list(metadata, " ");
+            track.start                = 0;
+            track.skip                 = 0;
+            currPregap                 = 0;
+            prestart                   = -1;
+            for (unsigned int i = 0; i < tokens.size(); i++) {
+                // "TRACK:1" > "TRACK" "1"
+                std::vector<string> track_meta = split_string_to_list(tokens[i], ":");
+                std::string         key        = track_meta[0];
+                std::string         value      = track_meta[1];
+
+                if (key == "TRACK") {
+                    // track index
+                    track.number = std::stoi(value);
+                } else if (key == "TYPE") {
+                    // track type
+                    if (value == "AUDIO") {
+                        track.sectorSize = RAW_SECTOR_SIZE;
+                        track.attr       = 0;
+                        track.mode2      = false;
+                    } else if (value == "MODE1") {
+                        track.sectorSize = COOKED_SECTOR_SIZE;
+                        track.attr       = 0x40;
+                        track.mode2      = false;
+                        file->skip_sync  = true;
+                    } else if (value == "MODE1_RAW") {
+                        track.sectorSize = RAW_SECTOR_SIZE;
+                        track.attr       = 0x40;
+                        track.mode2      = false;
+                    } else if (value == "MODE2") {
+                        track.sectorSize = 2336;
+                        track.attr       = 0x40;
+                        track.mode2      = true;
+                    } else if (value == "MODE2_FORM1") {
+                        track.sectorSize = COOKED_SECTOR_SIZE;
+                        track.attr       = 0x40;
+                        track.mode2      = true;
+                        file->skip_sync  = true;
+                    } else if (value == "MODE2_FORM2") {
+                        track.sectorSize = 2336;
+                        track.attr       = 0x40;
+                        track.mode2      = true;
+                    } else if (value == "MODE2_RAW") {
+                        track.sectorSize = RAW_SECTOR_SIZE;
+                        track.attr       = 0x40;
+                        track.mode2      = true;
+                    } else {
+                        // unknown track mode
+                        return false;
+                    };
+                } else if (key == "SUBTYPE") {
+                    if (value == "RAW") {
+                        //track.sectorSize += 96;
+                        // is subchannel data even supported?
+                    }
+                } else if (key == "FRAMES") {
+                    int frames    = std::stoi(value);
+                    track.start   = total_frames;
+                    total_frames += frames;
+                    track.length  = frames;
+
+                } else if (key == "PREGAP") {
+                    currPregap = std::stoi(value);
+                }
+
+            }
+
+            // chd has the pregap added to FRAMES
+            // this is needed or track.start and track.length do not match with LoadCueSheet for the same image
+            // creates the same track list as LoadCueSheet
+            if (currPregap) {
+                prestart      = track.start;
+                track.start  += currPregap;
+                track.length -= currPregap;
+            }
+
+            track.sectorSize = 2448; // chd always uses 2448
+            if (!AddTrack(track, shift, prestart, totalPregap, 0)) {
+                return false;
+            }
+
+            index += 1;
+        }
+
+        // no tracks found
+        if (tracks.empty()) return false;
+
+        // add leadout track
+        track.number++;
+        track.attr   = 0; // sync with load iso
+        track.start  = 0;
+        track.length = 0;
+        track.file   = NULL;
+        if (!AddTrack(track, shift, -1, totalPregap, 0)) return false;
+
+        return true;
+
+    } else {
+        return false;
+    }
+}
 
 bool CDROM_Interface_Image::AddTrack(Track &curr, int &shift, int prestart, int &totalPregap, int currPregap)
 {
@@ -941,8 +1308,8 @@ bool CDROM_Interface_Image::AddTrack(Track &curr, int &shift, int prestart, int
 	// current track uses a different file as the previous track
 	} else {
 		if (!prev.length) {
-			int tmp = prev.file->getLength() - prev.skip;
-			prev.length = tmp / prev.sectorSize;
+			int64_t tmp = prev.file->getLength() - prev.skip;
+			prev.length = (int)(tmp / prev.sectorSize);
 			if (tmp % prev.sectorSize != 0) prev.length++; // padding
 		}
 		curr.start += prev.start + prev.length + currPregap;
