@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,19 +19,25 @@
 
 #include "dosbox.h"
 #include "callback.h"
+#include "logging.h"
 #include "mem.h"
 #include "regs.h"
 #include "dos_inc.h"
 #include "control.h"
+#include "support.h"
 #include <list>
+#include <SDL.h>
 
 uint32_t DOS_HMA_LIMIT();
 uint32_t DOS_HMA_FREE_START();
 uint32_t DOS_HMA_GET_FREE_SPACE();
 void DOS_HMA_CLAIMED(uint16_t bytes);
 bool ANSI_SYS_installed();
+#if defined(MACOSX)
+bool SetClipboard(std::string value);
+#endif
 
-extern bool enable_share_exe_fake;
+extern bool enable_share_exe, enable_network_redirector;
 
 extern Bitu XMS_EnableA20(bool enable);
 
@@ -42,8 +48,6 @@ static Bitu call_int2f,call_int2a;
 static std::list<MultiplexHandler*> Multiplex;
 typedef std::list<MultiplexHandler*>::iterator Multiplex_it;
 
-extern RealPt ipx_callback;
-
 const char *Win_NameThatVXD(uint16_t devid) {
 	switch (devid) {
 		case 0x0006:	return "V86MMGR";
@@ -92,8 +96,12 @@ static Bitu INT2A_Handler(void) {
 	return CBRET_NONE;
 }
 
-extern bool i4dos, clipboard_dosapi;
+extern const char* RunningProgram;
+extern std::string strPasteBuffer;
+extern bool i4dos, shellrun, clipboard_dosapi, swapad;
 extern RealPt DOS_DriveDataListHead;       // INT 2Fh AX=0803h DRIVER.SYS drive data table list
+extern uint16_t seg_win_startup_info;
+void PasteClipboard(bool bPressed);
 
 // INT 2F
 char regpath[CROSS_LEN+1]="C:\\WINDOWS\\SYSTEM.DAT";
@@ -119,8 +127,8 @@ static bool DOS_MultiplexFunctions(void) {
         break;
 	/* ert, 20100711: Locking extensions */
     case 0x1000:    /* SHARE.EXE installation check */
-        if (enable_share_exe_fake) {
-            reg_al = 0xff; /* Pretend that share.exe is installed.. Of course it's a bloody LIE! */
+        if (enable_share_exe) {
+            reg_al = 0xff; /* Report that share.exe is installed. Of course not all SHARE functions are emulated. */
         }
         return true;
 	case 0x1216:	/* GET ADDRESS OF SYSTEM FILE TABLE ENTRY */
@@ -170,7 +178,7 @@ static bool DOS_MultiplexFunctions(void) {
 			// fill in filename in fcb style
 			// (space-padded name (8 chars)+space-padded extension (3 chars))
 			const char* filename=(const char*)Files[reg_bx]->GetName();
-			if (strrchr(filename,'\\')) filename=strrchr(filename,'\\')+1;
+			if (strrchr_dbcs((char *)filename,'\\')) filename=strrchr_dbcs((char *)filename,'\\')+1;
 			if (strrchr(filename,'/')) filename=strrchr(filename,'/')+1;
 			if (!filename) return true;
 			const char* dotpos=strrchr(filename,'.');
@@ -225,7 +233,7 @@ static bool DOS_MultiplexFunctions(void) {
 				/* Hack for Windows 98 SETUP.EXE (Wengier) */
 				return false;
 			}
-			strcpy(name,i4dos?"4DOS.COM":"COMMAND.COM");
+			strcpy(name,i4dos&&!shellrun?"4DOS.COM":"COMMAND.COM");
 			MEM_BlockWrite(SegPhys(ds)+reg_dx,name,(Bitu)(strlen(name)+1));
 			strcpy(name+1,"/P /D /K AUTOEXEC");
 			name[0]=(char)strlen(name+1);
@@ -256,10 +264,8 @@ static bool DOS_MultiplexFunctions(void) {
     case 0x1600:    /* Windows enhanced mode installation check */
         // Leave AX as 0x1600, indicating that neither Windows 3.x enhanced mode, Windows/386 2.x
         // nor Windows 95 are running, nor is XMS version 1 driver installed
-#ifdef WIN32
-		if (!control->SecureMode() && (reg_sp == 0xFFF6 && mem_readw(SegPhys(ss)+reg_sp) == 0x142A || reg_sp >= 0xFF7A && reg_sp <= 0xFF8F && mem_readw(SegPhys(ss)+reg_sp) == reg_sp + 21))
+		if (!control->SecureMode() && ((reg_sp == 0xFFF6 && mem_readw(SegPhys(ss)+reg_sp) == 0x142A) || !strcmp(RunningProgram, "DOSCLIP"))) // Hack for DOSCLIP
 			reg_ax = 0x301;
-#endif
         return true;
 	case 0x1605:	/* Windows init broadcast */
 		if (enable_a20_on_windows_init) {
@@ -275,7 +281,7 @@ static bool DOS_MultiplexFunctions(void) {
 		}
 
 		/* TODO: Maybe future parts of DOSBox-X will do something with this */
-		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox.conf or b) running a builtin command in Z:\ */
+		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox-x.conf or b) running a builtin command in Z:\ */
 		LOG_MSG("DEBUG: INT 2Fh Windows 286/386 DOSX init broadcast issued (ES:BX=%04x:%04x DS:SI=%04x:%04x CX=%04x DX=%04x DI=%04x(aka version %u.%u))",
 			SegValue(es),reg_bx,
 			SegValue(ds),reg_si,
@@ -302,10 +308,18 @@ static bool DOS_MultiplexFunctions(void) {
 			LOG_MSG("         unmodified, and only modify registers on the way back up the chain!\n");
 		}
 
+		// Dummy data is required for the Microsoft version of Japanese Windows 3.1 in enhanced mode.
+		if(IS_DOSV && (reg_dx & 0x0001) == 0) {
+			real_writew(seg_win_startup_info, 0x02, reg_bx);
+			real_writew(seg_win_startup_info, 0x04, SegValue(es));
+			SegSet16(es, seg_win_startup_info);
+			reg_bx = 0;
+		}
+
 		return false; /* pass it on to other INT 2F handlers */
 	case 0x1606:	/* Windows exit broadcast */
 		/* TODO: Maybe future parts of DOSBox-X will do something with this */
-		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox.conf or b) running a builtin command in Z:\ */
+		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox-x.conf or b) running a builtin command in Z:\ */
 		LOG_MSG("DEBUG: INT 2Fh Windows 286/386 DOSX exit broadcast issued (DX=0x%04x)",reg_dx);
 		if (reg_dx & 0x0001)
 			LOG_MSG(" [286 DOS extender]");
@@ -314,7 +328,7 @@ static bool DOS_MultiplexFunctions(void) {
 		LOG_MSG("\n");
 		return false; /* pass it on to other INT 2F handlers */
 	case 0x1607:
-		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox.conf or b) running a builtin command in Z:\
+		/* TODO: Don't show this by default. Show if the user wants it by a) setting something to "true" in dosbox-x.conf or b) running a builtin command in Z:\
 		 *       Additionally, if the user WANTS to see every invocation of the IDLE call, then allow them to enable that too */
 		if (reg_bx != 0x18) { /* don't show the idle call. it's used too often */
 			const char *str = Win_NameThatVXD(reg_bx);
@@ -381,7 +395,6 @@ static bool DOS_MultiplexFunctions(void) {
 	case 0x168f:	/*  Close awareness crap */
 	   /* Removing warning */
 		return true;
-#ifdef WIN32
 	case 0x1700:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_al = 1;
@@ -389,26 +402,37 @@ static bool DOS_MultiplexFunctions(void) {
 		return true;
 	case 0x1701:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
+#if defined(WIN32)
 		reg_ax=0;
 		if (OpenClipboard(NULL)) {
 			reg_ax=1;
 			CloseClipboard();
 		}
+#else
+		reg_ax=1;
+#endif
 		return true;
 	case 0x1702:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_ax=0;
-		if (OpenClipboard(NULL))
-			{
+#if defined(WIN32)
+		if (OpenClipboard(NULL)) {
 			reg_ax=EmptyClipboard()?1:0;
 			CloseClipboard();
-			}
+		}
+#elif defined(C_SDL2)
+        SDL_SetClipboardText("");
+        if (!SDL_HasClipboardText()) reg_ax=1;
+#elif defined(MACOSX)
+        if (SetClipboard("")) reg_ax=1;
+#endif
 		return true;
 	case 0x1703:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_ax=0;
-		if ((reg_dx==1||reg_dx==7)&&OpenClipboard(NULL))
-			{
+		if ((reg_dx==1||reg_dx==7)
+#if defined(WIN32)
+        &&OpenClipboard(NULL)) {
 			char *text, *buffer;
 			text = new char[reg_cx];
 			MEM_StrCopy(SegPhys(es)+reg_bx,text,reg_cx);
@@ -423,42 +447,112 @@ static bool DOS_MultiplexFunctions(void) {
 			SetClipboardData(reg_dx==1?CF_TEXT:CF_OEMTEXT,clipbuffer);
 			reg_ax++;
 			CloseClipboard();
-			}
+#elif defined(C_SDL2) || defined(MACOSX)
+        ) {
+			char *text = new char[reg_cx];
+			MEM_StrCopy(SegPhys(es)+reg_bx,text,reg_cx);
+            std::istringstream iss(text);
+            std::string result="";
+            for (std::string token; std::getline(iss, token); ) {
+                typedef char host_cnv_char_t;
+                host_cnv_char_t *CodePageGuestToHost(const char *s);
+                char* uname = CodePageGuestToHost(token.c_str());
+                result+=(uname!=NULL?std::string(uname):token)+std::string(1, 10);
+            }
+            if (result.size()&&result.back()==10) result.pop_back();
+#if defined(C_SDL2)
+            if (SDL_SetClipboardText(result.c_str()) == 0) reg_ax++;
+#else
+            if (SetClipboard(result)) reg_ax++;
+#endif
+#else
+        ) {
+#endif
+		}
 		return true;
 	case 0x1704:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_ax=0;
-		if ((reg_dx==1||reg_dx==7)&&OpenClipboard(NULL))
-			{
-			if (HANDLE text = GetClipboardData(reg_dx==1?CF_TEXT:CF_OEMTEXT))
-				{
+		if ((reg_dx==1||reg_dx==7)
+#if defined(WIN32)
+        &&OpenClipboard(NULL)) {
+			if (HANDLE text = GetClipboardData(reg_dx==1?CF_TEXT:CF_OEMTEXT)) {
 				reg_ax=(uint16_t)strlen((char *)text)+1;
 				reg_dx=(uint16_t)((strlen((char *)text)+1)/65536);
-				}
-			else
+			} else
 				reg_dx=0;
 			CloseClipboard();
-			}
+#else
+        ) {
+            swapad=false;
+            PasteClipboard(true);
+            swapad=true;
+            uint32_t size = 0, extra = 0;
+            unsigned char head, last=13;
+            uint8_t *text;
+            for (size_t i=0; i<strPasteBuffer.length(); i++) if (strPasteBuffer[i]==10||strPasteBuffer[i]==13) extra++;
+            if (strPasteBuffer.length() && (text = (uint8_t *)malloc(strPasteBuffer.length()+extra))) {
+                while (strPasteBuffer.length()) {
+                    head = strPasteBuffer[0];
+                    if (head == 10 && last != 13) text[size++] = 13;
+                    if (head > 31 || head == 9 || head == 10 || head == 13) text[size++] = head;
+                    if (head == 13 && (strPasteBuffer.length() < 2 || strPasteBuffer[1] != 10)) text[size++] = 10;
+                    strPasteBuffer = strPasteBuffer.substr(1, strPasteBuffer.length());
+                    last = head;
+                }
+				text[size]=0;
+				reg_ax=(uint16_t)size;
+				reg_dx=(uint16_t)(size/65536);
+				free(text);
+			} else
+				reg_dx=0;
+#endif
+		}
 		return true;
 	case 0x1705:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_ax=0;
-		if ((reg_dx==1||reg_dx==7)&&OpenClipboard(NULL))
-			{
-			if (HANDLE text = GetClipboardData(reg_dx==1?CF_TEXT:CF_OEMTEXT))
-				{
+		if ((reg_dx==1||reg_dx==7)
+#if defined(WIN32)
+        &&OpenClipboard(NULL)) {
+			if (HANDLE text = GetClipboardData(reg_dx==1?CF_TEXT:CF_OEMTEXT)) {
 				MEM_BlockWrite(SegPhys(es)+reg_bx,text,(Bitu)(strlen((char *)text)+1));
 				reg_ax++;
-				}
-			CloseClipboard();
 			}
+			CloseClipboard();
+#else
+        ) {
+            swapad=false;
+            PasteClipboard(true);
+            swapad=true;
+            uint32_t size = 0, extra = 0;
+            unsigned char head, last=13;
+            uint8_t *text;
+            for (size_t i=0; i<strPasteBuffer.length(); i++) if (strPasteBuffer[i]==10||strPasteBuffer[i]==13) extra++;
+            if (strPasteBuffer.length() && (text = (uint8_t *)malloc(strPasteBuffer.length()+extra))) {
+                while (strPasteBuffer.length()) {
+                    head = strPasteBuffer[0];
+                    if (head == 10 && last != 13) text[size++] = 13;
+                    if (head > 31 || head == 9 || head == 10 || head == 13) text[size++] = head;
+                    if (head == 13 && (strPasteBuffer.length() < 2 || strPasteBuffer[1] != 10)) text[size++] = 10;
+                    strPasteBuffer = strPasteBuffer.substr(1, strPasteBuffer.length());
+                    last = head;
+                }
+				text[size]=0;
+				MEM_BlockWrite(SegPhys(es)+reg_bx,text,(Bitu)(size+1));
+				reg_ax++;
+				free(text);
+            }
+#endif
+		}
 		return true;
 	case 0x1708:
 		if(control->SecureMode()||!clipboard_dosapi) return false;
 		reg_ax=1;
+#if defined(WIN32)
 		CloseClipboard();
-		return true;
 #endif
+		return true;
     case 0x1a00:    /* ANSI.SYS installation check (MS-DOS 4.0 or higher) */
         if (IS_PC98_ARCH) {
             /* NTS: PC-98 MS-DOS has ANSI handling directly within the kernel HOWEVER it does NOT
@@ -475,6 +569,15 @@ static bool DOS_MultiplexFunctions(void) {
             /* MS-DOS without ANSI.SYS loaded doesn't modify any registers in response to this call. */
             return true;
         }
+	case 0xb800:																	// Network - installation check
+        if (!enable_network_redirector) return false;
+		reg_al = 1;																	// Installed
+		reg_bx = 8;																	// Bit 3 - redirector
+		break;
+	case 0xb809:																	// Network - get version
+        if (!enable_network_redirector) return false;
+		reg_ax = 0x0201;															// Major-minor version as returned by NTVDM-Windows XP
+		break;
     case 0x4680:    /* Windows v3.0 check */
         // Leave AX as 0x4680, indicating that Windows 3.0 is not running in real (/R) or standard (/S) mode,
         // nor is DOS 5 DOSSHELL active
@@ -553,10 +656,6 @@ static bool DOS_MultiplexFunctions(void) {
 			reg_ax=0;
 			return true;
 		}
-    case 0x7a00:
-        LOG_MSG("call 2f 7a\n");
-        reg_al=0xff;
-        return true;
     }
 
 	return false;
@@ -574,7 +673,6 @@ void DOS_SetupMisc(void) {
 	RealSetVec(0x2A,CALLBACK_RealPointer(call_int2a));
 }
 
-extern const char* RunningProgram;
 void CALLBACK_DeAllocate(Bitu in);
 
 void DOS_UninstallMisc(void) {
