@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,13 +19,14 @@
 
 #include "dosbox.h"
 #include "dos_system.h"
+#include "bios_disk.h"
 #include "drives.h"
 #include "setup.h"
 #include "mapper.h"
 #include "support.h"
 #include "control.h"
 
-bool wild_match(char *haystack, char *needle) {
+bool wild_match(const char *haystack, char *needle) {
 	size_t max, i;
     for (; *needle != '\0'; ++needle) {
         switch (*needle) {
@@ -99,7 +100,7 @@ bool WildFileCmp(const char * file, const char * wild)
 			if (!wild_match(file_name, wild_name)) return false;
 			goto checkext;
 		} else
-			return wild_match((char *)file, wild_name);
+			return wild_match(file, wild_name);
 	} else {
 		r=0;
 		while (r<8) {
@@ -163,6 +164,7 @@ bool LWildFileCmp(const char * file, const char * wild)
     if (strrchr(nwild,'*')&&strrchr(nwild,'.')==NULL) strcat(nwild,".*");
     find_ext=strrchr(nwild,'.');
     if (find_ext) {
+            if (wild_match(file, nwild)) return true;
             Bitu size=(Bitu)(find_ext-nwild);
             if (size>255) size=255;
             memcpy(wild_name,nwild,size);
@@ -178,7 +180,7 @@ bool LWildFileCmp(const char * file, const char * wild)
 			if (!wild_match(file_name, wild_name)) return false;
 			goto checkext;
 		} else
-			return wild_match((char *)file, wild_name);
+			return wild_match(file, wild_name);
 	} else {
 		r=0;
 		while (r<size) {
@@ -264,6 +266,9 @@ const char * DOS_Drive::GetInfo(void) {
 }
 
 // static members variables
+extern int32_t swapPosition;
+extern int swapInDisksSpecificDrive;
+extern bool dos_kernel_disabled;
 int DriveManager::currentDrive;
 DriveManager::DriveInfo DriveManager::driveInfos[26];
 
@@ -271,6 +276,48 @@ void DriveManager::AppendDisk(int drive, DOS_Drive* disk) {
 	driveInfos[drive].disks.push_back(disk);
 }
 
+void DriveManager::ChangeDisk(int drive, DOS_Drive* disk) {
+    DriveInfo& driveInfo = driveInfos[drive];
+    if (Drives[drive]==NULL||disk==NULL||!driveInfo.disks.size()) return;
+    strcpy(disk->curdir,driveInfo.disks[driveInfo.currentDisk]->curdir);
+    disk->Activate();
+    disk->UpdateDPB(currentDrive);
+    driveInfo.disks[driveInfo.currentDisk] = disk;
+    fatDrive *old = dynamic_cast<fatDrive*>(Drives[drive]);
+    Drives[drive] = disk;
+    Drives[drive]->EmptyCache();
+    Drives[drive]->MediaChange();
+    fatDrive *fdp = dynamic_cast<fatDrive*>(Drives[drive]);
+    if (drive<2 && fdp && fdp->loadedDisk) {
+        if (imageDiskList[drive]) {
+            imageDiskList[drive]->Release();
+            imageDiskList[drive] = fdp->loadedDisk;
+            imageDiskList[drive]->Addref();
+            imageDiskChange[drive] = true;
+        }
+        if (swapInDisksSpecificDrive == drive && diskSwap[swapPosition]) {
+            diskSwap[swapPosition]->Release();
+            diskSwap[swapPosition] = fdp->loadedDisk;
+            diskSwap[swapPosition]->Addref();
+        }
+        if (!dos_kernel_disabled) {
+            char name[DOS_NAMELENGTH_ASCII],lname[LFN_NAMELENGTH];
+            uint32_t size;uint16_t date;uint16_t time;uint8_t attr;
+            RealPt save_dta = dos.dta();
+            dos.dta(dos.tables.tempdta);
+            DOS_DTA dta(dos.dta());
+            char root[7] = {(char)('A'+drive),':','\\','*','.','*',0};
+            bool ret = DOS_FindFirst(root,DOS_ATTR_VOLUME);
+            if (ret) {
+                dta.GetResult(name,lname,size,date,time,attr);
+                DOS_FindNext();
+            } else name[0] = 0;
+            dos.dta(save_dta);
+        }
+    }
+    if (old) old->UnMount();
+}
+
 void DriveManager::InitializeDrive(int drive) {
 	currentDrive = drive;
 	DriveInfo& driveInfo = driveInfos[currentDrive];
