@@ -379,6 +379,7 @@ static const char *def_menu_video_ttf[] =
 {
     "mapper_incsize",
     "mapper_decsize",
+    "ttf_resetcolor",
     "--",
     "ttf_showbold",
     "ttf_showital",
@@ -392,9 +393,11 @@ static const char *def_menu_video_ttf[] =
     "ttf_wpfe",
     "--",
     "ttf_blinkc",
+    "ttf_right_left",
 #if C_PRINTER
     "ttf_printfont",
 #endif
+    "--",
     "ttf_dbcs_sbcs",
     "ttf_autoboxdraw",
     "ttf_halfwidthkana",
@@ -1206,7 +1209,9 @@ LPWSTR getWString(std::string str, wchar_t *def, wchar_t*& buffer) {
     LPWSTR ret = def;
     int reqsize = 0, cp = dos.loaded_codepage;
     Section_prop *section = static_cast<Section_prop *>(control->GetSection("config"));
-    if ((!dos.loaded_codepage || dos_kernel_disabled || force_conversion) && section!=NULL && !control->opt_noconfig) {
+    if (control->opt_langcp && msgcodepage>0 && isSupportedCP(msgcodepage) && msgcodepage != dos.loaded_codepage)
+        cp = msgcodepage;
+    else if ((!dos.loaded_codepage || dos_kernel_disabled || force_conversion) && section!=NULL && !control->opt_noconfig) {
         char *countrystr = (char *)section->Get_string("country"), *r=strchr(countrystr, ',');
         if (r!=NULL && *(r+1)) {
             cp = atoi(trim(r+1));
