@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <sstream>
 
 #include "dosbox.h"
+#include "logging.h"
 #include "video.h"
 #include "render.h"
 #include "setup.h"
@@ -52,6 +53,13 @@ uint32_t                                GFX_palette32bpp[256] = {0};
 unsigned int                            GFX_GetBShift();
 void                                    RENDER_CallBack( GFX_CallBackFunctions_t function );
 
+#if defined(USE_TTF)
+bool resetreq=false;
+ttf_cell curAttrChar[txtMaxLins*txtMaxCols];					// currently displayed textpage
+ttf_cell newAttrChar[txtMaxLins*txtMaxCols];					// to be replaced by
+void resetFontSize();
+#endif
+
 static void Check_Palette(void) {
     /* Clean up any previous changed palette data */
     if (render.pal.changed) {
@@ -309,7 +317,7 @@ bool RENDER_StartUpdate(void) {
         return false;
     }
     render.frameskip.count=0;
-    if (render.scale.inMode == scalerMode8) {
+    if (render.scale.inMode == scalerMode8 && sdl.desktop.want_type != SCREEN_TTF) {
         Check_Palette();
     }
     render.scale.inLine = 0;
@@ -319,7 +327,6 @@ bool RENDER_StartUpdate(void) {
     render.scale.outPitch = 0;
     Scaler_ChangedLines[0] = 0;
     Scaler_ChangedLineIndex = 0;
-    /* Clearing the cache will first process the line to make sure it's never the same */
     if (GCC_UNLIKELY( render.scale.clearCache) ) {
 //      LOG_MSG("Clearing cache");
         //Will always have to update the screen with this one anyway, so let's update already
@@ -463,6 +470,9 @@ void RENDER_Reset( void ) {
         }
     }
 
+    if( sdl.desktop.isperfect ) /* Handle scaling if no pixel-perfect mode */
+        goto forcenormal;
+
     if ((dblh && dblw) || (render.scale.forced && dblh == dblw/*this branch works best with equal scaling in both directions*/)) {
         /* Initialize always working defaults */
         if (render.scale.size == 2)
@@ -570,7 +580,8 @@ void RENDER_Reset( void ) {
             else
                 simpleBlock = &ScaleNormalDh;
         }
-    } else  {
+    }
+    if( simpleBlock == NULL && complexBlock == NULL ) {
 forcenormal:
         complexBlock = 0;
         if(scalerOpGray==render.scale.op){
@@ -632,8 +643,12 @@ forcenormal:
 #if !defined(C_SDL2)
     gfx_flags=GFX_GetBestMode(gfx_flags);
 #else
-    gfx_flags &= ~GFX_SCALING;
-    gfx_flags |= GFX_RGBONLY | GFX_CAN_RANDOM;
+    if (sdl.desktop.want_type == SCREEN_TTF)
+        gfx_flags = GFX_CAN_32 | GFX_SCALING;
+    else {
+        gfx_flags &= ~GFX_SCALING;
+        gfx_flags |= GFX_RGBONLY | GFX_CAN_RANDOM;
+    }
 #endif
     if (!gfx_flags) {
         if (!complexBlock && simpleBlock == &ScaleNormal1x) 
@@ -785,6 +800,9 @@ forcenormal:
     render.active=true;
 
     last_gfx_flags = gfx_flags;
+#if defined(USE_TTF)
+    if (sdl.desktop.want_type == SCREEN_TTF && resetreq) resetFontSize();
+#endif
 }
 
 void RENDER_CallBack( GFX_CallBackFunctions_t function ) {
@@ -809,6 +827,13 @@ void RENDER_SetSize(Bitu width,Bitu height,Bitu bpp,float fps,double scrn_ratio)
         return; 
     }
 
+#if defined(USE_TTF)
+    if (sdl.desktop.want_type == SCREEN_TTF) {
+        render.cache.width	= width;
+        render.cache.height	= height;
+    }
+#endif
+
     // figure out doublewidth/height values
     bool dblw = false;
     bool dblh = false;
@@ -894,10 +919,17 @@ static bool RENDER_GetShader(std::string& shader_path, char *old_src) {
 	std::stringstream buf;
 	std::ifstream fshader(shader_path.c_str(), std::ios_base::binary);
 	if (!fshader.is_open()) fshader.open((shader_path + ".glsl").c_str(), std::ios_base::binary);
-	if (fshader.is_open()) {
-		buf << fshader.rdbuf();
-		fshader.close();
-	}
+    bool empty=true;
+    if (fshader.is_open()) {
+        buf << fshader.rdbuf();
+        empty=buf.str().empty();
+        fshader.close();
+        if (empty) {
+            buf.clear();
+            buf.str("");
+        }
+    }
+	if (!empty) ;
 	else if (shader_path == "advinterp2x") buf << advinterp2x_glsl;
 	else if (shader_path == "advinterp3x") buf << advinterp3x_glsl;
 	else if (shader_path == "advmame2x")   buf << advmame2x_glsl;
@@ -1090,6 +1122,52 @@ void RENDER_UpdateFromScalerSetting(void) {
     if (reset) RENDER_CallBack(GFX_CallBackReset);
 }
 
+#if C_OPENGL
+extern int initgl;
+void ResolvePath(std::string& in);
+std::string shader_src="", GetDOSBoxXPath(bool withexe=false);
+std::string LoadGLShader(Section_prop * section) {
+	shader_src = render.shader_src!=NULL?std::string(render.shader_src):"";
+    render.shader_def = false;
+	Prop_path *sh = section->Get_path("glshader");
+	std::string f = (std::string)sh->GetValue();
+    ResolvePath(f);
+    const char *ssrc=shader_src.c_str();
+	if (f.empty() || f=="none" || f=="default") {
+        render.shader_src = NULL;
+        render.shader_def = f=="default";
+        if (initgl==2) sdl_opengl.use_shader=true;
+    } else if (!RENDER_GetShader(sh->realpath,(char *)shader_src.c_str())) {
+        std::string path = std::string("glshaders") + CROSS_FILESPLIT + f;
+        if (!RENDER_GetShader(path,(char *)shader_src.c_str())) {
+            std::string exePath = GetDOSBoxXPath();
+            if (exePath.size()) path = exePath + std::string("glshaders") + CROSS_FILESPLIT + f;
+            else path = "";
+        } else {
+            if (initgl==2) sdl_opengl.use_shader=true;
+            LOG_MSG("Loaded GLSL shader: %s\n", path.c_str());
+            path = "";
+        }
+        if (path.size() && !RENDER_GetShader(path,(char *)shader_src.c_str())) {
+            Cross::GetPlatformConfigDir(path);
+            path = path + "glshaders" + CROSS_FILESPLIT + f;
+            if (!RENDER_GetShader(path,(char *)shader_src.c_str()) && (sh->realpath==f || !RENDER_GetShader(f,(char *)shader_src.c_str()))) {
+                sh->SetValue("none");
+                LOG_MSG("Shader file \"%s\" not found", f.c_str());
+            } else {
+                if (initgl==2) sdl_opengl.use_shader=true;
+                LOG_MSG("Loaded GLSL shader: %s\n", f.c_str());
+            }
+        }
+	} else {
+        if (initgl==2) sdl_opengl.use_shader=true;
+        LOG_MSG("Loaded GLSL shader: %s\n", f.c_str());
+    }
+	if (shader_src.size()&&shader_src.c_str()!=render.shader_src) shader_src="";
+    return shader_src;
+}
+#endif
+
 void RENDER_Init() {
     Section_prop * section=static_cast<Section_prop *>(control->GetSection("render"));
 
@@ -1106,7 +1184,7 @@ void RENDER_Init() {
 	}
 
 	//Set monochrome mode color and brightness
-	vga.draw.monochrome_pal=0;
+	vga.draw.monochrome_pal=MonochromeColor::Green;
 	vga.draw.monochrome_bright=1;
   Prop_multival* prop = section->Get_multival("monochrome_pal");
   std::string s_bright = prop->GetSection()->Get_string("bright");
@@ -1116,13 +1194,13 @@ void RENDER_Init() {
 		vga.draw.monochrome_bright=0;
 	}
 	if("green"==s_color){
-		vga.draw.monochrome_pal=0;
+		vga.draw.monochrome_pal=MonochromeColor::Green;
 	}else if("amber"==s_color){
-		vga.draw.monochrome_pal=1;
-	}else if("gray"==s_color){
-		vga.draw.monochrome_pal=2;
+		vga.draw.monochrome_pal=MonochromeColor::Amber;
+	}else if("gray"==s_color || "grey"==s_color){
+		vga.draw.monochrome_pal=MonochromeColor::Gray;
 	}else if("white"==s_color){
-		vga.draw.monochrome_pal=3;
+		vga.draw.monochrome_pal=MonochromeColor::White;
 	}
 
     //For restarting the renderer.
@@ -1148,21 +1226,41 @@ void RENDER_Init() {
 
     render.frameskip.max=(Bitu)section->Get_int("frameskip");
 
-    MAPPER_AddHandler(DecreaseFrameSkip,MK_nothing,0,"decfskip","Dec Fskip");
-    MAPPER_AddHandler(IncreaseFrameSkip,MK_nothing,0,"incfskip","Inc Fskip");
+    MAPPER_AddHandler(DecreaseFrameSkip,MK_nothing,0,"decfskip","Decrease frameskip");
+    MAPPER_AddHandler(IncreaseFrameSkip,MK_nothing,0,"incfskip","Increase frameskip");
 
 	DOSBoxMenu::item *item;
 
-	MAPPER_AddHandler(&AspectRatio_mapper_shortcut, MK_nothing, 0, "aspratio", "AspRatio", &item);
+	MAPPER_AddHandler(&AspectRatio_mapper_shortcut, MK_nothing, 0, "aspratio", "Fit to aspect ratio", &item);
 	item->set_text("Fit to aspect ratio");
 
+	if (machine==MCH_HERC || machine==MCH_MDA) {
+		void HercBlend(bool pressed), CycleHercPal(bool pressed);
+		MAPPER_AddHandler(CycleHercPal,MK_f7,MMOD1,"hercpal","Hercules Palette");
+		MAPPER_AddHandler(HercBlend,MK_f8,MMOD1,"hercblend","Hercules Blending");
+	}
+
+	if (machine==MCH_CGA || machine==MCH_MCGA || machine==MCH_PCJR || machine==MCH_AMSTRAD) {
+		if(!mono_cga) {
+            void IncreaseHue(bool pressed), DecreaseHue(bool pressed), CGAModel(bool pressed), Composite(bool pressed);
+            MAPPER_AddHandler(DecreaseHue,MK_f7,MMOD1|MMOD3,"dechue","Decrease Hue");
+            MAPPER_AddHandler(IncreaseHue,MK_f8,MMOD1|MMOD3,"inchue","Increase Hue");
+            MAPPER_AddHandler(CGAModel,MK_f7,MMOD1,"cgamodel","Early/Late CGA");
+            MAPPER_AddHandler(Composite,MK_f8,MMOD1,"cgacomp","CGA Composite");
+        } else {
+            void CycleMonoCGAPal(bool pressed), CycleMonoCGABright(bool pressed);
+            MAPPER_AddHandler(CycleMonoCGAPal,MK_f7,MMOD1,"monocgapal","Mono CGA Palette");
+            MAPPER_AddHandler(CycleMonoCGABright,MK_f8,MMOD1,"monocgabri","Mono CGA Brightness");
+        }
+	}
+
     mainMenu.get_item("vga_9widetext").check(vga.draw.char9_set).refresh_item(mainMenu);
     mainMenu.get_item("doublescan").check(vga.draw.doublescan_set).refresh_item(mainMenu);
     mainMenu.get_item("mapper_aspratio").check(render.aspect).refresh_item(mainMenu);
 
     RENDER_UpdateFrameskipMenu();
 
-    /* BUG FIX: Some people's dosbox.conf files have frameskip=-1 WTF?? */
+    /* BUG FIX: Some people's dosbox-x.conf files have frameskip=-1 WTF?? */
     /* without this fix, nothing displays, EVER */
     if ((int)render.frameskip.max < 0) render.frameskip.max = 0;
                                 
@@ -1185,26 +1283,7 @@ void RENDER_Init() {
     render.autofit=section->Get_bool("autofit");
 
 #if C_OPENGL
-	char* shader_src = render.shader_src;
-    render.shader_def = false;
-	Prop_path *sh = section->Get_path("glshader");
-	std::string f = (std::string)sh->GetValue();
-	if (f.empty() || f=="none" || f=="default") {
-        render.shader_src = NULL;
-        render.shader_def = f=="default";
-	} else if (!RENDER_GetShader(sh->realpath,shader_src)) {
-		std::string path = std::string("glshaders") + CROSS_FILESPLIT + f;
-		if (!RENDER_GetShader(path,shader_src)) {
-            Cross::GetPlatformConfigDir(path);
-            path = path + "glshaders" + CROSS_FILESPLIT + f;
-            if (!RENDER_GetShader(path,shader_src) && (sh->realpath==f || !RENDER_GetShader(f,shader_src))) {
-                sh->SetValue("none");
-                LOG_MSG("Shader file \"%s\" not found", f.c_str());
-            }
-        }
-	} else
-        LOG_MSG("Loaded GLSL shader: %s\n", f.c_str());
-	if (shader_src!=render.shader_src) free(shader_src);
+    std::string ssrc=LoadGLShader(section);
 #endif
 
     //If something changed that needs a ReInit
@@ -1212,7 +1291,7 @@ void RENDER_Init() {
     if(running && render.src.bpp && ((render.aspect != aspect) || (render.scale.op != scaleOp) || 
                   (render.scale.size != scalersize) || (render.scale.forced != scalerforced) ||
 #if C_OPENGL
-				  (render.shader_src != shader_src) ||
+				  (render.shader_src != ssrc.c_str()) ||
 #endif
                    render.scale.forced))
         RENDER_CallBack( GFX_CallBackReset );
@@ -1264,7 +1343,13 @@ private:
 		READ_POD( &render.fullFrame, render.fullFrame );
 		READ_POD( &render.frameskip, render.frameskip );
 		READ_POD( &render.aspect, render.aspect );
+		scalerOperation_t op = render.scale.op;
+		Bitu size = render.scale.size;
+		bool hardware = render.scale.hardware;
 		READ_POD( &render.scale, render.scale );
+		render.scale.op = op;
+		render.scale.size = size;
+		render.scale.hardware = hardware;
 
 		//***************************************
 		//***************************************
