@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include "dosbox.h"
 #include "inout.h"
+#include "logging.h"
 #include "mem.h"
 #include "paging.h"
 #include "glide.h"
@@ -57,7 +58,7 @@ extern void GFX_Stop(void);
 extern void GFX_ResetScreen(void);
 extern const char* RunningProgram;
 extern bool dpi_aware_enable;
-extern bool dos_kernel_disabled;
+extern bool DOS_GetMemory_unmapped;
 
 static float int_to_float(const uint32_t i)
 {
@@ -75,7 +76,7 @@ static float int_to_float(const uint32_t i)
 // Print debug messages
 #define LOG_GLIDE 0
 
-void VFILE_Remove(const char *name);
+void VFILE_Remove(const char *name,const char *dir = "");
 static void process_msg(Bitu);
 
 /** Global Variables **/
@@ -143,7 +144,7 @@ static void write_gl(Bitu port,Bitu val,Bitu iolen)
 
     // Allocate shared memory (80 bytes)
     if(val > GLIDE_MAX) {
-	if(glsegment==0 && !dos_kernel_disabled) {
+	if(glsegment==0 && !DOS_GetMemory_unmapped) {
 	    glsegment=DOS_GetMemory(5);
 #if LOG_GLIDE
 	    LOG_MSG("Glide:Memory allocated at 0x%x (segment: %hu)", glsegment<<4, glsegment);
@@ -450,7 +451,7 @@ public:
 	    hdll = NULL;
 	}
 
-	if (addovl) VFILE_Remove("GLIDE2X.OVL");
+	if (addovl) VFILE_Remove("GLIDE2X.OVL","SYSTEM");
     }
 };
 
@@ -460,6 +461,10 @@ void GLIDE_ShutDown(Section* sec) {
     delete test;
 }
 
+void GLIDE_PowerOn(Section* sec) {
+    test = new GLIDE(sec);
+}
+
 void GLIDE_Init() {
     test = new GLIDE(control->GetSection("voodoo"));
     AddExitFunction(AddExitFunctionFuncPair(GLIDE_ShutDown),true);
