@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,12 +16,13 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include <assert.h>
 #include <string.h>
 #include <iomanip>
 #include <sstream>
 #include "dosbox.h"
 #include "inout.h"
+#include "logging.h"
 #include "mixer.h"
 #include "dma.h"
 #include "pic.h"
@@ -2113,7 +2114,7 @@ public:
         gus_enable = false;
         if(!IS_EGAVGA_ARCH) return;
         Section_prop * section=static_cast<Section_prop *>(configuration);
-        if(!section->Get_bool("gus")) return;
+        if(!section->Get_bool("gus")||control->opt_silent) return;
 
         gus_enable = true;
         memset(&myGUS,0,sizeof(myGUS));
@@ -2184,6 +2185,8 @@ public:
 		myGUS.rate=(unsigned int)section->Get_int("gusrate");
 
         ultradir = section->Get_string("ultradir");
+        void ResolvePath(std::string& in);
+        ResolvePath(ultradir);
 
 		x = section->Get_int("gusmemsize");
 		if (x >= 0) myGUS.memsize = (unsigned int)x*1024u;
