@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,13 +16,17 @@
  *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
-
+#include <assert.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <limits.h>
+
+#include "bitmapinfoheader.h"
 #include "dosbox.h"
 #include "control.h"
 #include "hardware.h"
+#include "logging.h"
 #include "setup.h"
 #include "support.h"
 #include "mem.h"
@@ -31,6 +35,7 @@
 #include "mixer.h"
 #include "render.h"
 #include "cross.h"
+#include "wave_mmreg.h"
 
 #if (C_SSHOT)
 #include <zlib.h>
@@ -61,6 +66,8 @@ extern "C" {
 #endif
 
 bool            skip_encoding_unchanged_frames = false;
+std::string pathvid = "", pathwav = "", pathmtw = "";
+bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 
 #if (C_AVCODEC)
 bool ffmpeg_init = false;
@@ -313,8 +320,8 @@ bool export_ffmpeg = false;
 
 std::string capturedir;
 extern std::string savefilename;
-extern bool use_save_file;
-extern unsigned int sendkeymap;
+extern bool use_save_file, noremark_save_state, force_load_state;
+extern unsigned int hostkeyalt, sendkeymap;
 extern const char* RunningProgram;
 Bitu CaptureState = 0;
 
@@ -594,6 +601,10 @@ static void CAPTURE_AddAviChunk(const char * tag, uint32_t size, void * data, ui
 }
 #endif
 
+#if defined(USE_TTF)
+void ttf_switch_on(bool ss=true), ttf_switch_off(bool ss=true);
+#endif
+
 #if (C_SSHOT)
 void CAPTURE_VideoEvent(bool pressed) {
 	if (!pressed)
@@ -603,6 +614,10 @@ void CAPTURE_VideoEvent(bool pressed) {
 		CaptureState &= ~((unsigned int)CAPTURE_VIDEO);
 		LOG_MSG("Stopped capturing video.");	
 
+#if defined(USE_TTF)
+        if (!(CaptureState & CAPTURE_IMAGE) && !(CaptureState & CAPTURE_VIDEO))
+            ttf_switch_on();
+#endif
 		if (capture.video.writer != NULL) {
 			if ( capture.video.audioused ) {
 				CAPTURE_AddAviChunk( "01wb", (uint32_t)(capture.video.audioused * 4), capture.video.audiobuf, 0x10, 1);
@@ -614,6 +629,7 @@ void CAPTURE_VideoEvent(bool pressed) {
 			avi_writer_finish(capture.video.writer);
 			avi_writer_close_file(capture.video.writer);
 			capture.video.writer = avi_writer_destroy(capture.video.writer);
+			if (pathvid.size()) systemmessagebox("Recording completed",("Saved recording output to the file:\n\n"+pathvid).c_str(),"ok", "info", 1);
 		}
 #if (C_AVCODEC)
 		if (ffmpeg_fmt_ctx != NULL) {
@@ -633,7 +649,11 @@ void CAPTURE_VideoEvent(bool pressed) {
 		}
 	} else {
 		CaptureState |= CAPTURE_VIDEO;
+#if defined(USE_TTF)
+        ttf_switch_off();
+#endif
 	}
+	pathvid = "";
 
 	mainMenu.get_item("mapper_video").check(!!(CaptureState & CAPTURE_VIDEO)).refresh_item(mainMenu);
 }
@@ -695,6 +715,26 @@ extern uint32_t GFX_palette32bpp[256];
 
 unsigned int GFX_GetBShift();
 
+void CAPTURE_VideoStart() {
+#if (C_SSHOT)
+	if (CaptureState & CAPTURE_VIDEO) {
+		LOG_MSG("Already capturing video.");
+	} else {
+		CAPTURE_VideoEvent(true);
+	}
+#endif
+}
+
+void CAPTURE_VideoStop() {
+#if (C_SSHOT)
+	if (CaptureState & CAPTURE_VIDEO) {
+		CAPTURE_VideoEvent(true);
+	} else {
+		LOG_MSG("Not capturing video.");
+	}
+#endif
+}
+
 void CAPTURE_AddImage(Bitu width, Bitu height, Bitu bpp, Bitu pitch, Bitu flags, float fps, uint8_t * data, uint8_t * pal) {
 #if (C_SSHOT)
 	Bitu i;
@@ -759,7 +799,7 @@ void CAPTURE_AddImage(Bitu width, Bitu height, Bitu bpp, Bitu pitch, Bitu flags,
 #ifdef PNG_TEXT_SUPPORTED
 		int fields = 1;
 		png_text text[1] = {};
-		const char* text_s = "DOSBox " VERSION;
+		const char* text_s = "DOSBox-X " VERSION;
 		size_t strl = strlen(text_s);
 		char* ptext_s = new char[strl + 1];
 		strcpy(ptext_s, text_s);
@@ -895,6 +935,7 @@ skip_shot:
 			std::string path = GetCaptureFilePath("Video",".avi");
 			if (path == "")
 				goto skip_video;
+			pathvid = path;
 
 			capture.video.writer = avi_writer_create();
 			if (capture.video.writer == NULL)
@@ -1028,13 +1069,21 @@ skip_shot:
 			if (!avi_writer_begin_header(capture.video.writer) || !avi_writer_begin_data(capture.video.writer))
 				goto skip_video;
 
-			LOG_MSG("Started capturing video.");
+#if defined(WIN32)
+            char fullpath[MAX_PATH];
+            if (GetFullPathName(path.c_str(), MAX_PATH, fullpath, NULL)) path = fullpath;
+#elif defined(HAVE_REALPATH)
+            char fullpath[PATH_MAX];
+            if (realpath(path.c_str(), fullpath) != NULL) path = fullpath;
+#endif
+			LOG_MSG("Started capturing video to: %s", path.c_str());
 		}
 #if (C_AVCODEC)
 		else if (export_ffmpeg && ffmpeg_fmt_ctx == NULL) {
 			std::string path = GetCaptureFilePath("Video",".mts"); // Use widely recognized .MTS extension
 			if (path == "")
 				goto skip_video;
+			pathvid = path;
 
 			capture.video.width = width;
 			capture.video.height = height;
@@ -1206,7 +1255,14 @@ skip_shot:
 				goto skip_video;
 			}
 
-			LOG_MSG("Started capturing video (FFMPEG)");
+#if defined(WIN32)
+            char fullpath[MAX_PATH];
+            if (GetFullPathName(path.c_str(), MAX_PATH, fullpath, NULL)) path = fullpath;
+#elif defined(HAVE_REALPATH)
+            char fullpath[PATH_MAX];
+            if (realpath(path.c_str(), fullpath) != NULL) path = fullpath;
+#endif
+			LOG_MSG("Started capturing video (FFMPEG) to: %s", path.c_str());
 		}
 #endif
 
@@ -1416,18 +1472,25 @@ skip_shot:
         mainMenu.get_item("mapper_video").check(!!(CaptureState & CAPTURE_VIDEO)).refresh_item(mainMenu);
     }
 
-	return;
+#if defined(USE_TTF)
+    if (!(CaptureState & CAPTURE_IMAGE) && !(CaptureState & CAPTURE_VIDEO))
+        ttf_switch_on();
+#endif
+    return;
 skip_video:
 	capture.video.writer = avi_writer_destroy(capture.video.writer);
 # if (C_AVCODEC)
 	ffmpeg_flushout();
 	ffmpeg_closeall();
 # endif
+#endif
+#if defined(USE_TTF)
+    if (!(CaptureState & CAPTURE_IMAGE) && !(CaptureState & CAPTURE_VIDEO))
+        ttf_switch_on();
 #endif
 	return;
 }
 
-
 #if (C_SSHOT)
 void CAPTURE_ScreenShotEvent(bool pressed) {
 	if (!pressed)
@@ -1435,6 +1498,9 @@ void CAPTURE_ScreenShotEvent(bool pressed) {
 #if !defined(C_EMSCRIPTEN)
 	CaptureState |= CAPTURE_IMAGE;
 #endif
+#if defined(USE_TTF)
+    ttf_switch_off();
+#endif
 }
 #endif
 
@@ -1464,6 +1530,7 @@ void CAPTURE_MultiTrackAddWave(uint32_t freq, uint32_t len, int16_t * data,const
                 LOG_MSG("Cannot determine capture path");
 				goto skip_mt_wav;
             }
+			pathmtw = path;
 
 		    capture.multitrack_wave.audiorate = freq;
 
@@ -1551,7 +1618,14 @@ void CAPTURE_MultiTrackAddWave(uint32_t freq, uint32_t len, int16_t * data,const
 			if (!avi_writer_begin_header(capture.multitrack_wave.writer) || !avi_writer_begin_data(capture.multitrack_wave.writer))
 				goto skip_mt_wav;
 
-			LOG_MSG("Started capturing multitrack audio (%u channels).",streams);
+#if defined(WIN32)
+            char fullpath[MAX_PATH];
+            if (GetFullPathName(path.c_str(), MAX_PATH, fullpath, NULL)) path = fullpath;
+#elif defined(HAVE_REALPATH)
+            char fullpath[PATH_MAX];
+            if (realpath(path.c_str(), fullpath) != NULL) path = fullpath;
+#endif
+			LOG_MSG("Started capturing multitrack audio (%u channels) to: %s",streams, path.c_str());
 		}
 
         if (capture.multitrack_wave.writer != NULL) {
@@ -1598,6 +1672,7 @@ void CAPTURE_AddWave(uint32_t freq, uint32_t len, int16_t * data) {
 				CaptureState &= ~((unsigned int)CAPTURE_WAVE);
 				return;
 			}
+			pathwav = path;
 
 			capture.wave.writer = riff_wav_writer_create();
 			if (capture.wave.writer == NULL) {
@@ -1632,7 +1707,14 @@ void CAPTURE_AddWave(uint32_t freq, uint32_t len, int16_t * data) {
 			capture.wave.length = 0;
 			capture.wave.used = 0;
 			capture.wave.freq = freq;
-			LOG_MSG("Started capturing wave output.");
+#if defined(WIN32)
+            char fullpath[MAX_PATH];
+            if (GetFullPathName(path.c_str(), MAX_PATH, fullpath, NULL)) path = fullpath;
+#elif defined(HAVE_REALPATH)
+            char fullpath[PATH_MAX];
+            if (realpath(path.c_str(), fullpath) != NULL) path = fullpath;
+#endif
+			LOG_MSG("Started capturing wave output to: %s", path.c_str());
 		}
 		int16_t * read = data;
 		while (len > 0 ) {
@@ -1668,11 +1750,13 @@ void CAPTURE_MTWaveEvent(bool pressed) {
             avi_writer_close_file(capture.multitrack_wave.writer);
             capture.multitrack_wave.writer = avi_writer_destroy(capture.multitrack_wave.writer);
             CaptureState &= ~((unsigned int)CAPTURE_MULTITRACK_WAVE);
+            if (pathmtw.size()) systemmessagebox("Recording completed",("Saved recording output to the file:\n\n"+pathmtw).c_str(),"ok", "info", 1);
         }
     }
     else {
         CaptureState |= CAPTURE_MULTITRACK_WAVE;
     }
+    pathmtw = "";
 
 	mainMenu.get_item("mapper_recmtwave").check(!!(CaptureState & CAPTURE_MULTITRACK_WAVE)).refresh_item(mainMenu);
 #endif
@@ -1693,11 +1777,13 @@ void CAPTURE_WaveEvent(bool pressed) {
             riff_wav_writer_end_data(capture.wave.writer);
             capture.wave.writer = riff_wav_writer_destroy(capture.wave.writer);
             CaptureState &= ~((unsigned int)CAPTURE_WAVE);
+            if (pathwav.size()) systemmessagebox("Recording completed",("Saved recording output to the file:\n\n"+pathwav).c_str(),"ok", "info", 1);
         }
     }
     else {
         CaptureState |= CAPTURE_WAVE;
     }
+    pathwav = "";
 
 	mainMenu.get_item("mapper_recwave").check(!!(CaptureState & CAPTURE_WAVE)).refresh_item(mainMenu);
 #endif
@@ -1804,7 +1890,49 @@ void CAPTURE_Destroy(Section *sec) {
 	if (capture.midi.handle) CAPTURE_MidiEvent(true);
 }
 
-void OPL_SaveRawEvent(bool pressed), SetGameState_Run(int value);
+bool enable_autosave = false;
+int autosave_second = 0, autosave_count = 0, autosave_start[10], autosave_end[10], autosave_last[10];
+std::string autosave_name[10];
+void OPL_SaveRawEvent(bool pressed), SetGameState_Run(int value), ResolvePath(std::string& in);
+
+void ParseAutoSaveArg(std::string arg) {
+    if (arg.size()) {
+        size_t found=arg.find_last_of(":");
+        int start, end;
+        if (found==std::string::npos||found==0) {
+            found=arg.find_last_of("-");
+            if (found==std::string::npos||found==0) {
+                start=atoi(arg.c_str());
+                if (start>0) autosave_start[0]=start;
+                else if (start<0) autosave_start[0]=-1;
+            } else {
+                start=atoi(arg.substr(0, found).c_str());
+                end=atoi(arg.substr(found+1).c_str());
+                if (start>0) {
+                    autosave_start[0]=start;
+                    if (end>=start) autosave_end[0]=end;
+                } else if (start<0) autosave_start[0]=-1;
+            }
+        } else if (autosave_count<9) {
+            autosave_name[++autosave_count]=arg.substr(0, found);
+            std::string remain=arg.substr(found+1);
+            found=remain.find_last_of("-");
+            if (found==std::string::npos||found==0) {
+                start=atoi(remain.c_str());
+                if (start>0) autosave_start[autosave_count]=start;
+                else if (start<0) autosave_start[autosave_count]=-1;
+            } else {
+                start=atoi(remain.substr(0, found).c_str());
+                end=atoi(remain.substr(found+1).c_str());
+                if (start>0) {
+                    autosave_start[autosave_count]=start;
+                    if (end>=start) autosave_end[autosave_count]=end;
+                } else if (start<0) autosave_start[autosave_count]=-1;
+            }
+        }
+    }
+}
+
 void CAPTURE_Init() {
 	DOSBoxMenu::item *item;
 
@@ -1818,10 +1946,15 @@ void CAPTURE_Init() {
 	assert(proppath != NULL);
 	capturedir = proppath->realpath;
     SetGameState_Run(section->Get_int("saveslot")-1);
+    noremark_save_state = !section->Get_bool("saveremark");
+    mainMenu.get_item("noremark_savestate").check(noremark_save_state).refresh_item(mainMenu);
+    force_load_state = section->Get_bool("forceloadstate");
+    mainMenu.get_item("force_loadstate").check(force_load_state).refresh_item(mainMenu);
     savefilename = section->Get_string("savefile");
     trim(savefilename);
     if (savefilename.size()) {
         use_save_file=true;
+        ResolvePath(savefilename);
         mainMenu.get_item("usesavefile").set_text("Use save file ("+savefilename+")").check(use_save_file);
         mainMenu.get_item("browsesavefile").enable(use_save_file);
         std::string slot="";
@@ -1830,6 +1963,23 @@ void CAPTURE_Init() {
             mainMenu.get_item(slot).enable(!use_save_file).refresh_item(mainMenu);
         }
     }
+    Prop_multival* prop = section->Get_multival("autosave");
+    autosave_second = atoi(prop->GetSection()->Get_string("second"));
+    for (int i=0; i<10; i++) {
+        autosave_name[i] = "";
+        autosave_start[i] = autosave_end[i] = 0;
+        autosave_last[i] = -1;
+        ParseAutoSaveArg(prop->GetSection()->Get_string("arg"+std::to_string(i)));
+    }
+    enable_autosave = autosave_second>0;
+    if (autosave_second<0) autosave_second=-autosave_second;
+    mainMenu.get_item("enable_autosave").check(enable_autosave).enable(autosave_second>0).refresh_item(mainMenu);
+    mainMenu.get_item("lastautosaveslot").enable(autosave_second>0).refresh_item(mainMenu);
+    std::string hostkey = section->Get_string("hostkey");
+    if (hostkey=="ctrlalt") hostkeyalt=1;
+    else if (hostkey=="ctrlshift") hostkeyalt=2;
+    else if (hostkey=="altshift") hostkeyalt=3;
+    else hostkeyalt=0;
     std::string mapsendkey = section->Get_string("mapper send key");
     if (mapsendkey=="winlogo") sendkeymap=1;
     else if (mapsendkey=="winmenu") sendkeymap=2;
@@ -1880,23 +2030,23 @@ void CAPTURE_Init() {
 
 #if !defined(C_EMSCRIPTEN)
 	// mapper shortcuts for capture
-	MAPPER_AddHandler(CAPTURE_WaveEvent,MK_w,MMOD3|MMODHOST,"recwave","Rec Wave", &item);
+	MAPPER_AddHandler(CAPTURE_WaveEvent,MK_w,MMODHOST,"recwave","Record audio to WAV", &item);
 	item->set_text("Record audio to WAV");
 
-	MAPPER_AddHandler(CAPTURE_MTWaveEvent,MK_nothing,0,"recmtwave","Rec MTWav", &item);
+	MAPPER_AddHandler(CAPTURE_MTWaveEvent,MK_nothing,0,"recmtwave","Record to M.T. AVI", &item);
 	item->set_text("Record audio to multi-track AVI");
 
-	MAPPER_AddHandler(CAPTURE_MidiEvent,MK_nothing,0,"caprawmidi","Cap MIDI", &item);
+	MAPPER_AddHandler(CAPTURE_MidiEvent,MK_nothing,0,"caprawmidi","Record MIDI output", &item);
 	item->set_text("Record MIDI output");
 
-	MAPPER_AddHandler(OPL_SaveRawEvent,MK_nothing,0,"caprawopl","Cap OPL",&item);
+	MAPPER_AddHandler(OPL_SaveRawEvent,MK_nothing,0,"caprawopl","Record FM/OPL output",&item);
 	item->set_text("Record FM (OPL) output");
 #if (C_SSHOT)
-	MAPPER_AddHandler(CAPTURE_ScreenShotEvent,MK_s,MMOD3|MMODHOST,"scrshot","Screenshot", &item);
-	item->set_text("Take screenshot");
-
-	MAPPER_AddHandler(CAPTURE_VideoEvent,MK_v,MMOD3|MMODHOST,"video","Video", &item);
+	MAPPER_AddHandler(CAPTURE_VideoEvent,MK_i,MMODHOST,"video","Record video to AVI", &item);
 	item->set_text("Record video to AVI");
+
+	MAPPER_AddHandler(CAPTURE_ScreenShotEvent,MK_p,MMODHOST,"scrshot","Take screenshot", &item);
+	item->set_text("Take screenshot");
 #endif
 #endif
 
