@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,9 @@
 #include <assert.h>
 #include "dosbox.h"
 #include "dos_inc.h"
+#include "logging.h"
 #include "mem.h"
+#include "menu.h"
 #include "inout.h"
 #include "setup.h"
 #include "paging.h"
@@ -65,6 +67,7 @@ bool has_Init_MemHandles = false;
 bool has_Init_MemoryAccessArray = false;
 
 extern Bitu rombios_minimum_location;
+extern bool force_conversion;
 extern bool VIDEO_BIOS_always_carry_14_high_font;
 extern bool VIDEO_BIOS_always_carry_16_high_font;
 
@@ -543,9 +546,9 @@ PageHandler* lfb_memio_cb(MEM_CalloutObject &co,Bitu phys_page) {
     (void)co;//UNUSED
     if (memory.lfb.start_page == 0 || memory.lfb.pages == 0)
         return NULL;
-    if (phys_page >= memory.lfb.start_page || phys_page < memory.lfb.end_page)
+    if (phys_page >= memory.lfb.start_page && phys_page < memory.lfb.end_page)
         return memory.lfb.handler;
-    if (phys_page >= memory.lfb_mmio.start_page || phys_page < memory.lfb_mmio.end_page)
+    if (phys_page >= memory.lfb_mmio.start_page && phys_page < memory.lfb_mmio.end_page)
         return memory.lfb_mmio.handler;
 
     return NULL;
@@ -618,9 +621,20 @@ void MEM_SetLFB(Bitu page, Bitu pages, PageHandler *handler, PageHandler *mmioha
     memory.lfb_mmio.handler=mmiohandler;
     if (mmiohandler != NULL) {
         /* FIXME: Why is this hard-coded? There's more than just S3 emulation in this code's future! */
-        memory.lfb_mmio.start_page=page+(0x01000000/4096);
-        memory.lfb_mmio.end_page=page+(0x01000000/4096)+16;
-        memory.lfb_mmio.pages=16;
+        if (svgaCard == SVGA_S3Trio && (s3Card == S3_Trio64V || s3Card == S3_Vision868 || s3Card == S3_ViRGE || s3Card == S3_ViRGEVX)) {
+            /* 64MB BAR. According to the datasheet, this 64MB region is split into two 32MB halves,
+             * the lower half "little endian" and the upper half "big endian". Within the 32MB region,
+             * the low 16MB is video memory and the high 16MB is MMIO. */
+            memory.lfb_mmio.start_page=page+(0x01000000/4096);
+            memory.lfb_mmio.end_page=page+(0x01000000/4096)+16;
+            memory.lfb_mmio.pages=16;
+        }
+        else {
+            /* 8MB BAR, MMIO at +16MB */
+            memory.lfb_mmio.start_page=page+(0x01000000/4096);
+            memory.lfb_mmio.end_page=page+(0x01000000/4096)+16;
+            memory.lfb_mmio.pages=16;
+        }
     }
     else {
         memory.lfb_mmio.start_page=0;
@@ -651,14 +665,14 @@ void MEM_SetLFB(Bitu page, Bitu pages, PageHandler *handler, PageHandler *mmioha
 
 PageHandler * MEM_GetPageHandler(Bitu phys_page) {
     phys_page &= memory.mem_alias_pagemask_active;
-    if (phys_page<memory.handler_pages) {
+	if (glide.enabled && (phys_page>=(GLIDE_LFB>>12)) && (phys_page<(GLIDE_LFB>>12)+GLIDE_PAGES))
+		return (PageHandler*)glide.lfb_pagehandler;
+    else if (phys_page<memory.handler_pages) {
         if (memory.phandlers[phys_page] != NULL) /*likely*/
             return memory.phandlers[phys_page];
 
         return MEM_SlowPath(phys_page); /* will also fill in phandlers[] if zero or one matches, so the next access is very fast */
-    } else if (glide.enabled && (phys_page>=(GLIDE_LFB>>12)) && (phys_page<(GLIDE_LFB>>12)+GLIDE_PAGES)) {
-        return (PageHandler*)glide.lfb_pagehandler;
-    }
+	}
     return &illegal_page_handler;
 }
 
@@ -710,8 +724,8 @@ void MEM_BlockRead(PhysPt pt,void * data,Bitu size) {
     }
 }
 
-void MEM_BlockWrite(PhysPt pt,void const * const data,Bitu size) {
-    uint8_t const* read = reinterpret_cast<uint8_t const*>(data);
+void MEM_BlockWrite(PhysPt pt, const void *data, size_t size) {
+    const uint8_t* read = static_cast<const uint8_t *>(data);
     if (size==0)
         return;
 
@@ -1090,8 +1104,12 @@ void MEM_A20_Enable(bool enabled) {
     if (memory.a20.enabled != enabled)
         LOG(LOG_MISC,LOG_DEBUG)("MEM_A20_Enable(%u)",enabled?1:0);
 
-    if (a20_guest_changeable || a20_fake_changeable)
+    if (a20_guest_changeable || a20_fake_changeable) {
         memory.a20.enabled = enabled;
+        force_conversion = true;
+        mainMenu.get_item("enable_a20gate").check(enabled).refresh_item(mainMenu);
+        force_conversion = false;
+    }
 
     if (!a20_fake_changeable && (memory.mem_alias_pagemask & 0x100ul)) {
         if (memory.a20.enabled) memory.mem_alias_pagemask_active |= 0x100ul;
@@ -1134,7 +1152,7 @@ bool mem_unalignedreadw_checked(PhysPt address, uint16_t * val) {
     uint8_t rval1,rval2;
     if (mem_readb_checked(address+0, &rval1)) return true;
     if (mem_readb_checked(address+1, &rval2)) return true;
-    *val=(uint16_t)(((uint8_t)rval1) | (((uint8_t)rval2) << 8));
+    *val=(uint16_t)(rval1 | (rval2 << 8));
     return false;
 }
 
@@ -1144,7 +1162,7 @@ bool mem_unalignedreadd_checked(PhysPt address, uint32_t * val) {
     if (mem_readb_checked(address+1, &rval2)) return true;
     if (mem_readb_checked(address+2, &rval3)) return true;
     if (mem_readb_checked(address+3, &rval4)) return true;
-    *val=(uint32_t)(((uint8_t)rval1) | (((uint8_t)rval2) << 8) | (((uint8_t)rval3) << 16) | (((uint8_t)rval4) << 24));
+    *val=(uint32_t)(rval1 | (rval2 << 8) | (rval3 << 16) | (rval4 << 24));
     return false;
 }
 
@@ -1407,8 +1425,7 @@ void On_Software_CPU_Reset() {
 #if C_DYNAMIC_X86
     /* this technique is NOT reliable when running the dynamic core! */
     if (cpudecoder == &CPU_Core_Dyn_X86_Run) {
-        E_Exit("C++ exception method is not compatible with dynamic core when emulating reset, aborting");
-        return;
+        LOG_MSG("Warning: C++ exception method is not compatible with dynamic core when emulating reset");
     }
 #endif
 
@@ -1610,7 +1627,13 @@ class A20GATE : public Program {
 public:
     /*! \brief      Program entry point, when the command is run */
     void Run(void) {
-        if (cmd->FindString("SET",temp_line,false)) {
+        if (cmd->FindExist("-?", false) || cmd->FindExist("/?", false)) {
+            WriteOut("Turns on/off or changes the A20 gate mode.\n\n");
+            WriteOut("A20GATE [ON | OFF | SET [off | off_fake | on | on_fake | mask | fast]]\n\n"
+                    "  [ON | OFF | SET] Turns the A20 gate ON/OFF, or sets the A20 gate mode.\n\n"
+                    "Type A20GATE with no parameters to display the current A20 gate status.\n");
+        }
+        else if (cmd->FindString("SET",temp_line,false)) {
             char *x = (char*)temp_line.c_str();
 
             a20_fast_changeable = false;
@@ -1622,56 +1645,56 @@ public:
                 MEM_A20_Enable(false);
                 a20_guest_changeable = false;
                 a20_fake_changeable = true;
-                WriteOut("A20 gate now off_fake mode\n");
+                WriteOut("A20 gate is now in off_fake mode.\n");
             }
             else if (!strncasecmp(x,"off",3)) {
                 MEM_A20_Enable(false);
                 a20_guest_changeable = false;
                 a20_fake_changeable = false;
-                WriteOut("A20 gate now off mode\n");
+                WriteOut("A20 gate is now in off mode.\n");
             }
             else if (!strncasecmp(x,"on_fake",7)) {
                 MEM_A20_Enable(true);
                 a20_guest_changeable = false;
                 a20_fake_changeable = true;
-                WriteOut("A20 gate now on_fake mode\n");
+                WriteOut("A20 gate is now in on_fake mode.\n");
             }
             else if (!strncasecmp(x,"on",2)) {
                 MEM_A20_Enable(true);
                 a20_guest_changeable = false;
                 a20_fake_changeable = false;
-                WriteOut("A20 gate now on mode\n");
+                WriteOut("A20 gate is now in on mode.\n");
             }
             else if (!strncasecmp(x,"mask",4)) {
                 MEM_A20_Enable(false);
                 a20_guest_changeable = true;
                 a20_fake_changeable = false;
                 memory.a20.enabled = 0;
-                WriteOut("A20 gate now mask mode\n");
+                WriteOut("A20 gate is now in mask mode.\n");
             }
             else if (!strncasecmp(x,"fast",4)) {
                 MEM_A20_Enable(false);
                 a20_guest_changeable = true;
                 a20_fake_changeable = false;
                 a20_fast_changeable = true;
-                WriteOut("A20 gate now fast mode\n");
+                WriteOut("A20 gate is now in fast mode\n");
             }
             else {
-                WriteOut("Unknown setting\n");
+                WriteOut("Unknown setting - %s\n", x);
             }
         }
         else if (cmd->FindExist("ON")) {
+            WriteOut("Enabling A20 gate...\n");
             MEM_A20_Enable(true);
-            WriteOut("Enabling A20 gate\n");
+            if (!MEM_A20_Enabled()) WriteOut("Error: A20 gate cannot be enabled.\n");
         }
         else if (cmd->FindExist("OFF")) {
+            WriteOut("Disabling A20 gate...\n");
             MEM_A20_Enable(false);
-            WriteOut("Disabling A20 gate\n");
+            if (MEM_A20_Enabled()) WriteOut("Error: A20 gate cannot be disabled.\n");
         }
         else {
-			WriteOut("Turns on/off or changes the A20 gate mode.\n\n");
-            WriteOut("A20GATE SET [off | off_fake | on | on_fake | mask | fast]\n");
-            WriteOut("A20GATE [ON | OFF]\n");
+            WriteOut("A20 gate is currently %s.\n", MEM_A20_Enabled()?"ON":"OFF");
         }
     }
 };
@@ -1822,7 +1845,7 @@ void Init_RAM() {
 
     /* Allocate the RAM. We alloc as a large unsigned char array. new[] does not initialize the array,
      * so we then must zero the buffer. */
-    MemBase = new uint8_t[memory.pages*4096];
+    MemBase = new(std::nothrow) uint8_t[memory.pages*4096];
     if (!MemBase) E_Exit("Can't allocate main memory of %d KB",(int)memsizekb);
     /* Clear the memory, as new doesn't always give zeroed memory
      * (Visual C debug mode). We want zeroed memory though. */
@@ -1892,8 +1915,8 @@ void ShutDownMemHandles(Section * sec) {
 }
 
 /* this is called on hardware reset. the BIOS needs the A20 gate ON to boot properly on 386 or higher!
- * this temporarily switches on the A20 gate and lets it function as normal despite user settings.
- * BIOS will POST and then permit the A20 gate to go back to whatever emulation setting given in dosbox.conf */
+ * this temporarily switches on the A20 gate and lets it function as normal despite user settings. BIOS
+ * will POST and then permit the A20 gate to go back to whatever emulation setting given in dosbox-x.conf */
 void A20Gate_OnReset(Section *sec) {
     (void)sec;//UNUSED
     void A20Gate_OverrideOn(Section *sec);
@@ -1906,6 +1929,7 @@ void A20Gate_OnReset(Section *sec) {
 void A20Gate_OverrideOn(Section *sec) {
     (void)sec;//UNUSED
     memory.a20.enabled = 1;
+    mainMenu.get_item("enable_a20gate").check(true).refresh_item(mainMenu);
     a20_fake_changeable = false;
     a20_guest_changeable = true;
 }
@@ -1957,6 +1981,7 @@ void A20Gate_TakeUserSetting(Section *sec) {
         LOG(LOG_MISC,LOG_DEBUG)("A20: masking emulation");
         a20_guest_changeable = true;
     }
+    mainMenu.get_item("enable_a20gate").check(memory.a20.enabled).refresh_item(mainMenu);
 }
 
 void Init_A20_Gate() {
@@ -1974,7 +1999,7 @@ void PS2Port92_OnReset(Section *sec) {
     PS2_Port_92h_ReadHandler.Uninstall();
 
     if (IS_PC98_ARCH) {
-        // TODO: add separate dosbox.conf variable for A20 gate control on PC-98
+        // TODO: add separate dosbox-x.conf variable for A20 gate control on PC-98
         enable_port92 = true;
         if (enable_port92) {
             PS2_Port_92h_WriteHandler2.Install(0xF6,write_pc98_a20,IO_MB);
