@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "control.h"
 #include "dosbox.h"
 
+#include "logging.h"
 #include "support.h"
 #include "inout.h"
 #include "pic.h"
@@ -37,6 +39,8 @@
 #include "filelpt.h"
 #include "dos_inc.h"
 
+uint16_t parallel_baseaddr[9] = {0x378,0x278,0x3bc,0,0,0,0,0,0};
+
 bool device_LPT::Read(uint8_t * data,uint16_t * size) {
     (void)data;//UNUSED
 	*size=0;
@@ -67,7 +71,7 @@ uint16_t device_LPT::GetInformation(void) {
 	return 0x80A0;
 }
 
-const char* lptname[]={"LPT1","LPT2","LPT3"};
+const char* lptname[]={"LPT1","LPT2","LPT3","LPT4","LPT5","LPT6","LPT7","LPT8","LPT9"};
 device_LPT::device_LPT(uint8_t num, class CParallel* pp) {
 	pportclass = pp;
 	SetName(lptname[num]);
@@ -84,18 +88,18 @@ device_LPT::~device_LPT() {
 }
 
 static void Parallel_EventHandler(Bitu val) {
-	Bitu serclassid=val&0x3;
+	Bitu serclassid=val&0xf;
 	if(parallelPortObjects[serclassid]!=0)
-		parallelPortObjects[serclassid]->handleEvent((uint16_t)(val>>2ul));
+		parallelPortObjects[serclassid]->handleEvent((uint16_t)(val>>4ul));
 }
 
 void CParallel::setEvent(uint16_t type, float duration) {
-    PIC_AddEvent(Parallel_EventHandler,duration,((Bitu)type<<2u)|(Bitu)port_nr);
+    PIC_AddEvent(Parallel_EventHandler,duration,((Bitu)type<<4u)|(Bitu)port_nr);
 }
 
 void CParallel::removeEvent(uint16_t type) {
     // TODO
-	PIC_RemoveSpecificEvents(Parallel_EventHandler,((Bitu)type<<2u)|(Bitu)port_nr);
+	PIC_RemoveSpecificEvents(Parallel_EventHandler,((Bitu)type<<4u)|(Bitu)port_nr);
 }
 
 void CParallel::handleEvent(uint16_t type) {
@@ -104,10 +108,13 @@ void CParallel::handleEvent(uint16_t type) {
 
 static Bitu PARALLEL_Read (Bitu port, Bitu iolen) {
     (void)iolen;//UNUSED
-	for(Bitu i = 0; i < 3; i++) {
-		if(parallel_baseaddr[i]==(port&0xfffc) && (parallelPortObjects[i]!=0)) {
+	for(Bitu i = 0; i < 9; i++) {
+		/* NTS: The traditional parport range is assigned 8 ports by IBM, but only 0-2 are assigned.
+		 *      EPP ports assign ports 3-4 with possible vendor extensions in 5-6, while ECP assigns
+		 *      ports 0x400-0x402 relative to base and leave 3-7 undefined. */
+		if(((port-parallel_baseaddr[i])&0xfff8) == 0 && (parallelPortObjects[i]!=0)) {
 			Bitu retval=0xff;
-			switch (port & 0x7) {
+			switch ((port-parallel_baseaddr[i]) & 0x7) {
 				case 0:
 					retval = parallelPortObjects[i]->Read_PR();
 					break;
@@ -131,8 +138,11 @@ static Bitu PARALLEL_Read (Bitu port, Bitu iolen) {
 }
 
 static void PARALLEL_Write (Bitu port, Bitu val, Bitu) {
-	for(Bitu i = 0; i < 3; i++) {
-		if(parallel_baseaddr[i]==(port&0xfffc) && parallelPortObjects[i]) {
+	for(Bitu i = 0; i < 9; i++) {
+		/* NTS: The traditional parport range is assigned 8 ports by IBM, but only 0-2 are assigned.
+		 *      EPP ports assign ports 3-4 with possible vendor extensions in 5-6, while ECP assigns
+		 *      ports 0x400-0x402 relative to base and leave 3-7 undefined. */
+		if(((port-parallel_baseaddr[i])&0xfff8) == 0 && (parallelPortObjects[i]!=0)) {
 #if PARALLEL_DEBUG
 			const char* const dbgtext[]={"DAT","IOS","CON","???"};
 			parallelPortObjects[i]->log_par(parallelPortObjects[i]->dbg_cregs,
@@ -141,7 +151,7 @@ static void PARALLEL_Write (Bitu port, Bitu val, Bitu) {
 				fprintf(parallelPortObjects[i]->debugfp,"%c",val);
 			}
 #endif
-			switch (port & 0x3) {
+			switch ((port-parallel_baseaddr[i]) & 0x7) {
 				case 0:
 					parallelPortObjects[i]->Write_PR (val);
 					return;
@@ -181,9 +191,9 @@ void CParallel::log_par(bool active, char const* format,...) {
 // Initialisation
 CParallel::CParallel(CommandLine* cmd, Bitu portnr, uint8_t initirq) {
     (void)cmd;//UNUSED
-	base = parallel_baseaddr[portnr];
-	irq = initirq;
-	port_nr = portnr;
+    base = parallel_baseaddr[portnr];
+    irq = initirq;
+    port_nr = portnr;
 
 #if PARALLEL_DEBUG
 	dbg_data	= cmd->FindExist("dbgdata", false);
@@ -286,7 +296,7 @@ void CParallel::initialize()
 
 
 
-CParallel* parallelPortObjects[3]={NULL,NULL,NULL};
+CParallel* parallelPortObjects[]={NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL};
 
 bool DISNEY_HasInit();
 Bitu DISNEY_BasePort();
@@ -297,8 +307,8 @@ Bitu bios_post_parport_count() {
 	Bitu count = 0;
 	unsigned int i;
 
-	for (i=0;i < 3;i++) {
-		if (parallelPortObjects[i] != NULL)
+	for (i=0;i < 9;i++) {
+		if (parallelPortObjects[i] != NULL && parallel_baseaddr[i])
 			count++;
 		else if (DISNEY_HasInit() && parallel_baseaddr[i] == DISNEY_BasePort())
 			count++;
@@ -311,8 +321,8 @@ Bitu bios_post_parport_count() {
 void BIOS_Post_register_parports() {
 	unsigned int i;
 
-	for (i=0;i < 3;i++) {
-		if (parallelPortObjects[i] != NULL)
+	for (i=0;i < 9;i++) {
+		if (parallelPortObjects[i] != NULL && parallel_baseaddr[i])
 			BIOS_SetLPTPort(i,(uint16_t)parallelPortObjects[i]->base);
 		else if (DISNEY_HasInit() && parallel_baseaddr[i] == (uint16_t)DISNEY_BasePort())
 			BIOS_SetLPTPort(i,(uint16_t)DISNEY_BasePort());
@@ -333,16 +343,28 @@ public:
 #endif
 
 		// default ports & interrupts
-		uint8_t defaultirq[] = { 7, 5, 12};
+		uint8_t defaultirq[] = { 7, 5, 12, 0, 0, 0, 0, 0, 0};
 		Section_prop *section = static_cast <Section_prop*>(configuration);
 		
 		char pname[]="parallelx";
 		// iterate through all 3 lpt ports
-		for (Bitu i = 0; i < 3; i++) {
+		for (Bitu i = 0; i < 9; i++) {
 			pname[8] = '1' + (char)i;
 			CommandLine cmd(0,section->Get_string(pname));
+            CommandLine tmp(0,section->Get_string(pname), CommandLine::either, true);
 
 			std::string str;
+            bool squote = false;
+            // single quotes to quote string?
+            if(cmd.FindStringBegin("squote",str,false)) {
+                squote = true;
+                cmd=tmp;
+            }
+
+            if(cmd.FindStringBegin("base:",str,true))
+				parallel_baseaddr[i] = (uint16_t)strtol(str.c_str(), NULL, 16);
+            if(cmd.FindStringBegin("irq:",str,true))
+				defaultirq[i] = (uint8_t)strtol(str.c_str(), NULL, 10);
 			cmd.FindCommand(1,str);
 #if C_DIRECTLPT
 			if(str=="reallpt") {
@@ -353,19 +375,17 @@ public:
 					delete cdlpt;
 					parallelPortObjects[i]=0;
 				}
-			}
-			else
+			} else
 #endif
 			if(!str.compare("file")) {
-				CFileLPT* cflpt= new CFileLPT(i, defaultirq[i], &cmd);
+				CFileLPT* cflpt= new CFileLPT(i, defaultirq[i], &cmd, squote);
 				if(cflpt->InstallationSuccessful)
 					parallelPortObjects[i]=cflpt;
 				else {
 					delete cflpt;
 					parallelPortObjects[i]=0;
 				}
-			}
-			else 
+			} else
 #if C_PRINTER
             // allow printer redirection on a single port
             if (str == "printer" && !printer_used)
@@ -382,8 +402,7 @@ public:
                     delete cprd;
                     parallelPortObjects[i] = 0;
                 }
-            }
-            else
+            } else
 #endif				
             if(str=="disabled") {
 				parallelPortObjects[i] = 0;
@@ -404,7 +423,7 @@ public:
 
 	~PARPORTS () {
 //		LOG_MSG("Parports destructor\n");
-		for (Bitu i = 0; i < 3; i++) {
+		for (Bitu i = 0; i < 9; i++) {
 			if (parallelPortObjects[i]) {
 				delete parallelPortObjects[i];
 				parallelPortObjects[i] = 0;
@@ -433,7 +452,7 @@ void PARALLEL_OnPowerOn (Section * sec) {
 	/* Mainline DOSBox 0.74 compatible support for "disney=true" setting.
 	 * But, don't allocate the Disney Sound Source if LPT1 is already taken. */
 	if (!DISNEY_HasInit() && DISNEY_ShouldInit() && parallelPortObjects[0] == NULL) {
-		LOG_MSG("LPT: LPT1 not taken, and dosbox.conf says to emulate Disney Sound Source");
+		LOG_MSG("LPT: LPT1 not taken, and dosbox-x.conf says to emulate Disney Sound Source");
 		DISNEY_Init(parallel_baseaddr[0]);
 	}
 }
@@ -444,7 +463,7 @@ void PARALLEL_OnDOSKernelInit (Section * sec) {
 
 	LOG(LOG_MISC,LOG_DEBUG)("DOS kernel initializing, creating LPTx devices");
 
-	for (i=0;i < 3;i++) {
+	for (i=0;i < 9;i++) {
 		if (parallelPortObjects[i] != NULL)
 			parallelPortObjects[i]->registerDOSDevice();
 	}
@@ -454,7 +473,7 @@ void PARALLEL_OnDOSKernelExit (Section * sec) {
     (void)sec;//UNUSED
 	unsigned int i;
 
-	for (i=0;i < 3;i++) {
+	for (i=0;i < 9;i++) {
 		if (parallelPortObjects[i] != NULL)
 			parallelPortObjects[i]->unregisterDOSDevice();
 	}
@@ -465,7 +484,7 @@ void PARALLEL_OnReset (Section * sec) {
 	unsigned int i;
 
 	// FIXME: Unregister/destroy the DOS devices, but consider that the DOS kernel at reset is gone.
-	for (i=0;i < 3;i++) {
+	for (i=0;i < 9;i++) {
 		if (parallelPortObjects[i] != NULL)
 			parallelPortObjects[i]->unregisterDOSDevice();
 	}
