@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,9 @@
 #if C_PRINTER
 
 #include <math.h>
+#include <sys/stat.h>
+
+#include "logging.h"
 #include "setup.h"
 #include "mapper.h"
 #include "printer_if.h"
@@ -28,11 +31,25 @@
 #include "cross.h"
 #include "printer_charmaps.h"
 #include "control.h"
-
 #include "pic.h" // for timeout
+#include "jfont.h"
+#include "timer.h"
+#include "render.h"
+#include "dos_inc.h"
+#include "../../ints/int10.h"
+#include "sdlmain.h"
 
+#if defined(USE_TTF)
+extern unsigned char DOSBoxTTFbi[48868];
+extern bool printfont;
+extern void resetFontSize();
+extern FT_Face GetTTFFace();
+#endif
 extern void GFX_CaptureMouse(void);
-extern bool mouselocked;
+extern std::map<int, int> lowboxdrawmap;
+extern uint16_t cpMap[512], cpMap_PC98[256];
+extern bool dbcs_sbcs, autoboxdraw;
+extern bool halfwidthkana, mouselocked;
 
 static CPrinter* defaultPrinter = NULL;
 
@@ -43,10 +60,20 @@ static CPrinter* defaultPrinter = NULL;
 static uint16_t confdpi, confwidth, confheight;
 static Bitu printer_timout;
 static bool timeout_dirty;
-static const char* document_path;
-//static const char* font_path;
+static std::string document_path;
+static std::string font_path;
+static std::string device;
 static char confoutputDevice[50];
-static bool confmultipageOutput;
+static bool confmultipageOutput, shellhide;
+static int printdbcs;
+static std::string actstd, acterr;
+
+void UpdateDefaultPrinterFont() {
+    if (defaultPrinter!=NULL) {
+        defaultPrinter->curFont = NULL;
+        defaultPrinter->updateFont();
+    }
+}
 
 void CPrinter::FillPalette(uint8_t redmax, uint8_t greenmax, uint8_t bluemax, uint8_t colorID, SDL_Palette* pal)
 {
@@ -63,6 +90,41 @@ void CPrinter::FillPalette(uint8_t redmax, uint8_t greenmax, uint8_t bluemax, ui
 	}
 }
 
+extern std::string prtlist;
+
+void CPrinter::getPrinterContext() {
+#if defined (WIN32)
+    if (device.size()&&device!="-") {
+        printerDC = CreateDC("WINSPOOL", device.c_str(), NULL, NULL);
+        return;
+    }
+
+	// Show Print dialog to obtain a printer device context
+    PRINTDLG pd;
+    pd.lStructSize = sizeof(PRINTDLG);
+    pd.hDevMode = (HANDLE) NULL;
+    pd.hDevNames = (HANDLE) NULL;
+    pd.Flags = PD_RETURNDC;
+    pd.hwndOwner = NULL;
+    pd.hDC = (HDC) NULL;
+    pd.nFromPage = 1;
+    pd.nToPage = 1;
+    pd.nMinPage = 0;
+    pd.nMaxPage = 0;
+    pd.nCopies = 1;
+    pd.hInstance = NULL;
+    pd.lCustData = 0L;
+    pd.lpfnPrintHook = (LPPRINTHOOKPROC) NULL;
+    pd.lpfnSetupHook = (LPSETUPHOOKPROC) NULL;
+    pd.lpPrintTemplateName = (LPSTR) NULL;
+    pd.lpSetupTemplateName = (LPSTR) NULL;
+    pd.hPrintTemplate = (HANDLE) NULL;
+    pd.hSetupTemplate = (HANDLE) NULL;
+    if (PrintDlg(&pd)) printerDC = pd.hDC;
+    else printerDC = NULL;
+#endif
+}
+
 CPrinter::CPrinter(uint16_t dpi, uint16_t width, uint16_t height, char* output, bool multipageOutput) 
 {
 	if (FT_Init_FreeType(&FTlib))
@@ -123,35 +185,6 @@ CPrinter::CPrinter(uint16_t dpi, uint16_t width, uint16_t height, char* output,
 
 		resetPrinter();
 
-		if (strcasecmp(output, "printer") == 0)
-		{
-#if defined (WIN32)
-			// Show Print dialog to obtain a printer device context
-			PRINTDLG pd;
-			pd.lStructSize = sizeof(PRINTDLG); 
-			pd.hDevMode = (HANDLE) NULL; 
-			pd.hDevNames = (HANDLE) NULL; 
-			pd.Flags = PD_RETURNDC; 
-			pd.hwndOwner = NULL; 
-			pd.hDC = (HDC) NULL; 
-			pd.nFromPage = 1; 
-			pd.nToPage = 1; 
-			pd.nMinPage = 0; 
-			pd.nMaxPage = 0; 
-			pd.nCopies = 1; 
-			pd.hInstance = NULL; 
-			pd.lCustData = 0L; 
-			pd.lpfnPrintHook = (LPPRINTHOOKPROC) NULL; 
-			pd.lpfnSetupHook = (LPSETUPHOOKPROC) NULL; 
-			pd.lpPrintTemplateName = (LPSTR) NULL; 
-			pd.lpSetupTemplateName = (LPSTR)  NULL; 
-			pd.hPrintTemplate = (HANDLE) NULL; 
-			pd.hSetupTemplate = (HANDLE) NULL; 
-			PrintDlg(&pd);
-			// TODO: what if user presses cancel?
-			printerDC = pd.hDC;
-#endif
-		}
 		LOG(LOG_MISC,LOG_NORMAL)("PRINTER: Enabled");
 	}
 }
@@ -162,6 +195,10 @@ void CPrinter::resetPrinterHard()
 	resetPrinter();
 }
 
+static uint32_t lasttick = 0;
+static uint8_t lastchar = 0, last2 = 0, last3 = 0;
+static bool lastlead = false, lastbox = true;
+static bool box2 = false, box3 = false;
 void CPrinter::resetPrinter()
 {
 		color = COLOR_BLACK;
@@ -185,7 +222,7 @@ void CPrinter::resetPrinter()
 		densy = 2;
 		densz = 3;
 		charTables[0] = 0; // Italics
-		charTables[1] = charTables[2] = charTables[3] = 437;
+		charTables[1] = charTables[2] = charTables[3] = dos.loaded_codepage;
 		definedUnit = -1;
 		multipoint = false;
 		multiPointSize = 0.0;
@@ -194,6 +231,9 @@ void CPrinter::resetPrinter()
 		msb = 255;
 		numPrintAsChar = 0;
 		LQtypeFace = courier;
+		lasttick = 0;
+		lastchar = last2 = last3 = 0;
+		lastlead = box2 = box3 = false;
 
 		selectCodepage(charTables[curCharTable]);
 
@@ -219,7 +259,7 @@ CPrinter::~CPrinter(void)
 		FT_Done_FreeType(FTlib);
 	}
 #if defined (WIN32)
-	DeleteDC(printerDC);
+	if (printerDC) DeleteDC(printerDC);
 #endif
 }
 
@@ -228,16 +268,21 @@ void CPrinter::selectCodepage(uint16_t cp)
 	const uint16_t* mapToUse = NULL;
 
 	Bitu i = 0;
-	while(charmap[i].codepage!=0)
-    {
-		if(charmap[i].codepage==cp)
+#if defined(USE_TTF)
+    if (TTF_using() && dos.loaded_codepage == cp)
+        mapToUse = cpMap;
+    else
+#endif
+        while(charmap[i].codepage != 0)
         {
-			mapToUse = charmap[i].map;
-			break;
-		}
-		i++;
-	}
-	if (mapToUse == NULL)
+            if(charmap[i].codepage == cp)
+            {
+                mapToUse = charmap[i].map;
+                break;
+            }
+            i++;
+        }
+    if(mapToUse == NULL)
     {
 		LOG(LOG_MISC,LOG_WARN)("Unsupported codepage %i. Using CP437 instead.", cp);
 		selectCodepage(437);
@@ -252,14 +297,17 @@ void CPrinter::updateFont()
 	if (curFont != NULL)
 		FT_Done_Face(curFont);
 
-	std::string fontName, basedir;
-#if defined(WIN32)
-    basedir = ".\\FONTS\\";
-#else
-    basedir = "./FONTS/";
+#if defined(USE_TTF)
+    if (TTF_using()&&printfont) curFont = GetTTFFace();
+    if (curFont == NULL)
 #endif
+    {
 
-	switch (LQtypeFace)
+	std::string fontName, basedir = font_path;
+    if (basedir.back()!='\\' && basedir.back()!='/')
+        basedir += CROSS_FILESPLIT;
+
+    switch(LQtypeFace)
 	{
 	    case roman:
 		    fontName = basedir + "roman.ttf";
@@ -274,7 +322,7 @@ void CPrinter::updateFont()
 		    fontName = basedir + "script.ttf";
 		    break;
 	    case ocra:
-	    case ocrb:
+	    case ocrb:;
 		    fontName = basedir + "ocra.ttf";
 		    break;
 	    default:
@@ -310,7 +358,7 @@ void CPrinter::updateFont()
                 fontName = basedir + "times.ttf";
 #else
                 fontName = basedir + "liberation-serif/LiberationSerif-Regular.ttf";
-                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSerif-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "truetype/liberation/LiberationSerif-Regular.ttf";
 #endif
                 break;
             case sansserif:
@@ -318,7 +366,7 @@ void CPrinter::updateFont()
                 fontName = basedir + "arial.ttf";
 #else
                 fontName = basedir + "liberation-sans/LiberationSans-Regular.ttf";
-                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSans-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "truetype/liberation/LiberationSans-Regular.ttf";
 #endif
                 break;
             case courier:
@@ -326,7 +374,7 @@ void CPrinter::updateFont()
                 fontName = basedir + "cour.ttf";
 #else
                 fontName = basedir + "liberation-mono/LiberationMono-Regular.ttf";
-                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationMono-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "truetype/liberation/LiberationMono-Regular.ttf";
 #endif
                 break;
             case script:
@@ -341,15 +389,20 @@ void CPrinter::updateFont()
                 fontName = basedir + "times.ttf";
 #else
                 fontName = basedir + "liberation-serif/LiberationSerif-Regular.ttf";
-                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "liberation/LiberationSerif-Regular.ttf";
+                if(stat(fontName.c_str(),&wstat)) fontName = basedir + "truetype/liberation/LiberationSerif-Regular.ttf";
 #endif
         }
-        if (FT_New_Face(FTlib, fontName.c_str(), 0, &curFont)) {
+        if (FT_New_Face(FTlib, fontName.c_str(), 0, &curFont)
+#if defined(USE_TTF)
+        && FT_New_Memory_Face(FTlib, DOSBoxTTFbi, sizeof(DOSBoxTTFbi), 0, &curFont)
+#endif
+        ) {
             //LOG(LOG_MISC,LOG_ERROR)("Unable to load font %s", fontName);
             LOG_MSG("Unable to load font %s (or %s)", oldfont.c_str(), fontName.c_str());
             curFont = NULL;
         }
 	}
+    }
 
 	double horizPoints = 10.5;
 	double vertPoints = 10.5;
@@ -1281,8 +1334,147 @@ void CPrinter::newPage(bool save, bool resetx)
 	}*/
 }
 
-void CPrinter::printChar(uint8_t ch)
+extern bool CodePageGuestToHostUTF16(uint16_t *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
+extern bool isDBCSCP(), isKanji1(uint8_t chr), CheckBoxDrawing(uint8_t c1, uint8_t c2, uint8_t c3, uint8_t c4);
+void CPrinter::printChar(uint8_t ch, int box)
 {
+    bool dbcs=false;
+    uint8_t ll = 0;
+    uint16_t dbchar = 0;
+    if ((printdbcs==1 || (printdbcs==-1 && (isJEGAEnabled() || IS_DOSV || (TTF_using()
+#if defined(USE_TTF)
+    && dbcs_sbcs
+#endif
+    )))) && box!=1 && (IS_PC98_ARCH || isDBCSCP())) {
+        uint32_t tick = GetTicks();
+        if (box!=0 && last3 && last2 && lastchar && lasttick && (tick-lasttick<50||tick-lasttick<printer_timout)) {
+            if (ch>=176&&ch<=223
+#if defined(USE_TTF)
+            && autoboxdraw
+#endif
+            && CheckBoxDrawing(last3, last2, lastchar, ch)) {
+                ll=lastchar;
+                if (box3) {
+                    lastchar=0;
+                    printChar(last3, 1);
+                    lastchar=last3;
+                    printChar(last2, 1);
+                    lastchar=last2;
+                    printChar(lastchar, 1);
+                }
+                lastchar=ll;
+                printChar(ch, 1);
+                last3=last2;
+                last2=ll;
+                lastchar=ch;
+                box2=box3=false;
+                lastlead=false;
+                lastbox=true;
+                lasttick=GetTicks();
+                return;
+            } else {
+                if (box3) {
+                    lastlead=false;
+                    ll=lastchar;
+                    lastchar=0;
+                    printChar(last3, 0);
+                    lastchar=last3;
+                    printChar(last2, 0);
+                    lastchar=last2;
+                    printChar(ll, 0);
+                    lastchar=ll;
+                    printChar(ch, 0);
+                }
+                box2=box3=false;
+                last3=last2=0;
+                lasttick=0;
+                lastchar=0;
+                return;
+            }
+        } else if (box!=0 && box2 && last2 && lastchar && lasttick && (tick-lasttick<50||tick-lasttick<printer_timout)) {
+            if (ch>=176&&ch<=223
+#if defined(USE_TTF)
+            && autoboxdraw
+#endif
+                ) {
+                box2=false;
+                box3=true;
+                last3=last2;
+                last2=lastchar;
+                lastchar=ch;
+                lasttick=GetTicks();
+                return;
+            } else {
+                lastlead=false;
+                ll=lastchar;
+                lastchar=0;
+                printChar(last2, 0);
+                lastchar=last2;
+                printChar(ll, 0);
+                lastchar=ll;
+                box2=box3=false;
+                lastlead=isKanji1(ch);
+                if (lastlead) {
+                    lastchar=ch;
+                    lasttick=GetTicks();
+                    return;
+                } else
+                    lasttick=0;
+                last3=last2=0;
+            }
+        } else if (lastlead && ch>=0x40 && lastchar && lasttick && (tick-lasttick<50||tick-lasttick<printer_timout)) {
+            if (box!=0&&lastchar>=176&&lastchar<=223&&ch>=176&&ch<=223
+#if defined(USE_TTF)
+            && autoboxdraw
+#endif
+            ) {
+                box2=true;
+                box3=false;
+                last3=0;
+                last2=lastchar;
+                lastchar=ch;
+                lasttick=GetTicks();
+                return;
+            }
+            box2=box3=false;
+            dbcs=true;
+            lastlead=false;
+            lasttick=0;
+        } else if (lastlead && ch<0x40 && lastchar && lasttick) {
+            if (box!=0) {
+                box2=box3=false;
+                last3=last2=0;
+            }
+            ll=lastchar;
+            lastlead=false;
+            lastchar=lasttick=0;
+            printChar(ll, 1);
+        } else {
+            if (box!=0) box2=box3=false;
+            lastlead=box==1?0:isKanji1(ch);
+            if (lastlead) {
+                lastchar=ch;
+                lasttick=GetTicks();
+                return;
+            } else
+                lasttick=0;
+            if (box!=0) last3=last2=0;
+        }
+    }
+    if (dbcs) {
+        char text[3];
+        uint16_t uname[4];
+        text[0]=lastchar;
+        text[1]=ch;
+        text[2]=0;
+        uname[0]=0;
+        uname[1]=0;
+        CodePageGuestToHostUTF16(uname,text);
+        if (uname[0]!=0&&uname[1]==0)
+            dbchar=uname[0];
+    }
+    lastchar=0;
+
 	charRead = true;
 	if (page == NULL) return;
 
@@ -1308,7 +1500,15 @@ void CPrinter::printChar(uint8_t ch)
 	if(ch == 0x1) ch = 0x20;
 	
 	// Find the glyph for the char to render
-	FT_UInt index = FT_Get_Char_Index(curFont, curMap[ch]);
+    uint16_t printch = dbchar?dbchar:curMap[ch];
+    if (!dbchar && dos.loaded_codepage == 932 && (halfwidthkana || isJEGAEnabled())) {
+        if (ch>=0xA1&&ch<=0xDF) printch = cpMap_PC98[ch];
+        else {
+            std::map<int, int>::iterator it = lowboxdrawmap.find(ch);
+            if (lowboxdrawmap.find(ch)!=lowboxdrawmap.end()) printch = charmap[0].map[it->second];
+        }
+    }
+	FT_UInt index = FT_Get_Char_Index(curFont, printch);
 	
 	// Load the glyph 
 	FT_Load_Glyph(curFont, index, FT_LOAD_DEFAULT);
@@ -1349,7 +1549,7 @@ void CPrinter::printChar(uint8_t ch)
 
 	// advance the cursor to the right
 	double x_advance;
-	if (style &	STYLE_PROP)
+	if ((style & STYLE_PROP) || dbcs)
 		x_advance = (double)((double)(curFont->glyph->advance.x) / (double)(dpi * 64));
 	else
     {
@@ -1625,6 +1825,30 @@ void CPrinter::printBitGraph(uint8_t ch)
 
 void CPrinter::formFeed()
 {
+    if ((printdbcs==1 || (printdbcs==-1 && TTF_using()
+#if defined(USE_TTF)
+    && dbcs_sbcs
+#endif
+    )) && (IS_PC98_ARCH || isDBCSCP()) && lastchar>=0x80) {
+        uint8_t ll=lastchar;
+        lastchar=0;
+        if (last3 && last2) {
+            printChar(last3, last3>=176&&last3<=223&&last2>=176&&last2<=223?1:0);
+            lastchar=last3;
+            printChar(last2, last3>=176&&last3<=223&&last2>=176&&last2<=223?1:0);
+            lastchar=last2;
+            printChar(ll, 1);
+        } else if (last2) {
+            printChar(last2, 0);
+            lastchar=last2;
+            printChar(ll, 0);
+        } else
+            printChar(lastchar, 1);
+    }
+    lastchar=lasttick=0;
+    last2=last3=0;
+    box2=box3=false;
+
 	// Don't output blank pages
 	newPage(!isBlank(), true);
 	finishMultipage();
@@ -1633,7 +1857,7 @@ void CPrinter::formFeed()
 static void findNextName(const char* front, const char* ext, char* fname)
 {
 	int i = 1;
-	Bitu slen = (Bitu)strlen(document_path);
+	Bitu slen = (Bitu)document_path.size();
 	if(slen > (200 - 15))
     {
 		fname[0] = 0;
@@ -1643,7 +1867,7 @@ static void findNextName(const char* front, const char* ext, char* fname)
     FILE *test = NULL;
 	do
 	{
-		strcpy(fname, document_path);
+		strcpy(fname, document_path.c_str());
 #ifdef WIN32
 		const char* const pathstring = "\\%s%d%s";
 #else 
@@ -1656,6 +1880,57 @@ static void findNextName(const char* front, const char* ext, char* fname)
 	while (test != NULL);
 }
 
+void CPrinter::doAction(const char *fname) {
+    std::string action;
+    if (actstd.size()) {
+        action=actstd;
+        bool fail=false;
+#if defined(WIN32)
+        bool q=false;
+        int pos=-1;
+        std::string para=fname;
+        for (int i=0; i<action.size(); i++) {
+            if (action[i]=='"') q=!q;
+            else if (action[i]==' ' && !q) {
+                pos=i;
+                break;
+            }
+        }
+        if (pos>-1) {
+            para=action.substr(pos+1)+" "+fname;
+            action=action.substr(0, pos);
+        }
+        fail=(INT_PTR)ShellExecute(NULL, "open", action.c_str(), para.c_str(), NULL, shellhide?SW_HIDE:SW_NORMAL)<=32;
+#else
+        fail=system((action+" "+fname).c_str())!=0;
+#endif
+        if (acterr.size()&&fail) {
+            action=acterr;
+#if defined(WIN32)
+            q=false;
+            pos=-1;
+            para=fname;
+            for (int i=0; i<action.size(); i++) {
+                if (action[i]=='"') q=!q;
+                else if (action[i]==' ' && !q) {
+                    pos=i;
+                    break;
+                }
+            }
+            if (pos>-1) {
+                para=action.substr(pos+1)+" "+fname;
+                action=action.substr(0, pos);
+            }
+            fail=(INT_PTR)ShellExecute(NULL, "open", action.c_str(), para.c_str(), NULL, shellhide?SW_HIDE:SW_NORMAL)<=32;
+#else
+            fail=system((action+" "+fname).c_str())!=0;
+#endif
+        }
+        bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
+        if (fail) systemmessagebox("Error", "The requested file handler failed to complete.", "ok","error", 1);
+    }
+}
+
 void CPrinter::outputPage() 
 {
 	char fname[200];
@@ -1664,8 +1939,9 @@ void CPrinter::outputPage()
 	{
 #if defined (WIN32)
 		// You'll need the mouse for the print dialog
-		if(mouselocked)
+		if (mouselocked)
 			 GFX_CaptureMouse();
+        if (!device.size() || printerDC==NULL) getPrinterContext();
 
 		uint16_t physW = GetDeviceCaps(printerDC, PHYSICALWIDTH);
 		uint16_t physH = GetDeviceCaps(printerDC, PHYSICALHEIGHT);
@@ -1692,12 +1968,17 @@ void CPrinter::outputPage()
 			DOCINFO docinfo;
 			memset(&docinfo, 0, sizeof(docinfo));
 			docinfo.cbSize = sizeof(docinfo);
-			docinfo.lpszDocName = "DOSBOX Printer";
+			docinfo.lpszDocName = "DOSBox-X Printer";
 			docinfo.lpszOutput = NULL;
 			docinfo.lpszDatatype = NULL;
 			docinfo.fwType = 0;
 
-			StartDoc(printerDC, &docinfo);
+			if (StartDoc(printerDC, &docinfo)<=0) {
+                LOG_MSG("PRINTER: Cannot start print.");
+                DeleteObject(bitmap);
+                DeleteDC(memHDC);
+                return;
+            }
 			multiPageCounter = 1;
 		}
 
@@ -1822,6 +2103,7 @@ void CPrinter::outputPage()
 		
 		/*clean up dynamically allocated RAM.*/
 		free(row_pointers);
+		doAction(fname);
 	}
 #endif
 	else if (strcasecmp(output, "ps") == 0)
@@ -1851,7 +2133,7 @@ void CPrinter::outputPage()
 			fprintf(psfile, "%%!PS-Adobe-3.0\n");
 			fprintf(psfile, "%%%%Pages: (atend)\n");
 			fprintf(psfile, "%%%%BoundingBox: 0 0 %i %i\n", (uint16_t)(defaultPageWidth * 72), (uint16_t)(defaultPageHeight * 72));
-			fprintf(psfile, "%%%%Creator: DOSBOX Virtual Printer\n");
+			fprintf(psfile, "%%%%Creator: DOSBox-X Virtual Printer\n");
 			fprintf(psfile, "%%%%DocumentData: Clean7Bit\n");
 			fprintf(psfile, "%%%%LanguageLevel: 2\n");
 			fprintf(psfile, "%%%%EndComments\n");
@@ -1925,6 +2207,7 @@ void CPrinter::outputPage()
 			fprintf(psfile, "%%%%EOF\n");
 			fclose(psfile);
 			outputHandle = NULL;
+			doAction(fname);
 		}
 	}
 	else
@@ -1932,6 +2215,7 @@ void CPrinter::outputPage()
 		// Find a page that does not exists
 		findNextName("page", ".bmp", &fname[0]);
 		SDL_SaveBMP(page, fname);
+		doAction(fname);
 	}
 }
 
@@ -2015,7 +2299,7 @@ void CPrinter::finishMultipage()
 		else if (strcasecmp(output, "printer") == 0)
 		{
 #if defined (WIN32)
-			EndDoc(printerDC);
+			if (printerDC) EndDoc(printerDC);
 #endif
 		}
 		outputHandle = NULL;
@@ -2097,6 +2381,13 @@ static void FormFeed(bool pressed)
 		}
 }
 
+const char* Mouse_GetSelected(int x1, int y1, int x2, int y2, int w, int h, uint16_t *textlen);
+void PrintScreen(const char *text, uint16_t len)
+{
+    if (!defaultPrinter) defaultPrinter = new CPrinter(confdpi, confwidth, confheight, confoutputDevice, confmultipageOutput);
+    for (int i=0; i<len; i++) defaultPrinter->printChar(text[i]);
+    FormFeed(true);
+}
 
 static void PRINTER_EventHandler(Bitu param)
 {
@@ -2171,6 +2462,7 @@ bool PRINTER_isInited()
 	return inited;
 }
 
+void ResolvePath(std::string& in);
 void PRINTER_Init() 
 {
 	Section_prop * section = static_cast<Section_prop *>(control->GetSection("printer"));
@@ -2180,9 +2472,11 @@ void PRINTER_Init()
 	//real_writew(0x0040, 0x0008, LPTPORT);
 
 	if (!section->Get_bool("printer")) return;
-	inited = true;
 	document_path = section->Get_string("docpath");
-	//font_path = section->Get_string("fontpath");
+	ResolvePath(document_path);
+	font_path = section->Get_string("fontpath");
+	ResolvePath(font_path);
+	device = section->Get_string("device");
 	confdpi = section->Get_int("dpi");
 	confwidth = section->Get_int("width");
 	confheight = section->Get_int("height");
@@ -2191,6 +2485,13 @@ void PRINTER_Init()
 	else timeout_dirty = false;
 	strcpy(&confoutputDevice[0], section->Get_string("printoutput"));
 	confmultipageOutput = section->Get_bool("multipage");
+    const char *dbcsstr = section->Get_string("printdbcs");
+    printdbcs = !strcasecmp(dbcsstr, "true")||!strcasecmp(dbcsstr, "1")?1:(!strcasecmp(dbcsstr, "false")||!strcasecmp(dbcsstr, "0")?0:-1);
+	shellhide = section->Get_bool("shellhide");
+	actstd = section->Get_string("openwith");
+    ResolvePath(actstd);
+	acterr = section->Get_string("openerror");
+    ResolvePath(acterr);
 
 	//IO_RegisterWriteHandler(LPTPORT,PRINTER_writedata,IO_MB);
 	//IO_RegisterReadHandler(LPTPORT,PRINTER_readdata,IO_MB);
@@ -2200,8 +2501,34 @@ void PRINTER_Init()
 	//IO_RegisterReadHandler(LPTPORT+2,PRINTER_readcontrol,IO_MB);
 
     DOSBoxMenu::item *item;
-	MAPPER_AddHandler(FormFeed, MK_f2 , MMOD1, "ejectpage", "FormFeed", &item);
+	MAPPER_AddHandler(FormFeed, MK_f2 , MMOD1, "ejectpage", "Send form-feed", &item);
     item->set_text("Send form-feed");
+
+#if defined(WIN32)
+    if (!inited && !strcasecmp(confoutputDevice, "printer")) {
+        DWORD dwNeeded = 0, dwReturned = 0;
+        bool fnReturn = EnumPrinters(PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS, NULL, 1L, (LPBYTE)NULL, 0L, &dwNeeded, &dwReturned);        PRINTER_INFO_1* pInfo = NULL;
+        if (dwNeeded > 0) pInfo = (PRINTER_INFO_1 *)HeapAlloc(GetProcessHeap(), 0L, dwNeeded);
+        if (NULL != pInfo) {
+            dwReturned = 0;
+            fnReturn = EnumPrinters(PRINTER_ENUM_LOCAL | PRINTER_ENUM_CONNECTIONS, NULL, 1L, (LPBYTE)pInfo, dwNeeded, &dwNeeded, &dwReturned);
+        }
+        if (fnReturn) {
+            unsigned int devnum;
+            if (1!=sscanf(device.c_str(),"%u",&devnum)) devnum=-2;
+            prtlist = "Printer Device List\n-------------------------------------------------------------\n";
+            for (unsigned int i=1; i < dwReturned; i++) {
+                if(devnum>0&&i==devnum) device=pInfo[i].pName;
+                prtlist+=(i<10?"0":"")+std::to_string(i)+" "+pInfo[i].pName+"\n";
+            }
+            std::istringstream in(("\n"+prtlist+"\n").c_str());
+            if (in)	for (std::string line; std::getline(in, line); )
+                LOG_MSG("%s", line.c_str());
+        } else
+            pInfo = NULL;
+    }
+#endif
+	inited = true;
 }
 
 #endif
