@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -18,11 +18,15 @@
 
 
 #include "dosbox.h"
+#include "logging.h"
 #include "setup.h"
 #include "vga.h"
 #include "inout.h"
 #include "mem.h"
 
+extern unsigned int vbe_window_granularity;
+extern unsigned int vbe_window_size;
+
 typedef struct SVGA_PVGA1A_DATA_t {
 	Bitu PR0A;
 	Bitu PR0B;
@@ -53,7 +57,12 @@ static void bank_setup_pvga1a() {
 	} else {
 		// Single bank config is straightforward
 		vga.svga.bank_read = vga.svga.bank_write = (uint8_t)pvga1a.PR0A;
-		vga.svga.bank_size = 4*1024;
+
+		if (vbe_window_granularity > 0)
+			vga.svga.bank_size = vbe_window_granularity; /* allow different sizes for dev testing */
+		else
+			vga.svga.bank_size = 4*1024;
+
 		VGA_SetupHandlers();
 	}
 }
@@ -232,13 +241,6 @@ void SVGA_Setup_ParadisePVGA1A(void) {
 		pvga1a.PR1 = 2<<6;
 	}
 
-	// Paradise ROM signature
-	PhysPt rom_base=PhysMake(0xc000,0);
-	phys_writeb(rom_base+0x007d,'V');
-	phys_writeb(rom_base+0x007e,'G');
-	phys_writeb(rom_base+0x007f,'A');
-	phys_writeb(rom_base+0x0080,'=');
-
 	IO_Write(0x3cf, 0x05); // Enable!
 }
 
