@@ -29,6 +29,7 @@
 #include "SDL.h"
 #include "int10.h"
 #include "jfont.h"
+#include "render.h"
 
 #if defined(_MSC_VER)
 # pragma warning(disable:4244) /* const fmath::local::uint64_t to double possible loss of data */
@@ -297,6 +298,10 @@ static scancode_tbl scan_to_scanascii_pc98[0x80] = {
 #include <queue>
 std::queue <uint16_t>over_key_buffer;
 
+#if defined(USE_TTF)
+extern bool ttf_dosv;
+#endif
+
 bool BIOS_AddKeyToBuffer(uint16_t code) {
     if (!IS_PC98_ARCH) {
         if (mem_readb(BIOS_KEYBOARD_FLAGS2)&8) return true;
@@ -307,8 +312,8 @@ bool BIOS_AddKeyToBuffer(uint16_t code) {
         start=0x502;
         end=0x522;
     }
-    else if (machine==MCH_PCJR) {
-        /* should be done for cga and others as well, to be tested */
+    else if (machine==MCH_PCJR || machine==MCH_CGA) {
+        /* should be done for others as well, to be tested */
         start=0x1e;
         end=0x3e;
     } else {
@@ -332,7 +337,11 @@ bool BIOS_AddKeyToBuffer(uint16_t code) {
     /* Check for buffer Full */
     //TODO Maybe beeeeeeep or something although that should happend when internal buffer is full
     if (ttail==head) {
+#if defined(USE_TTF)
+        if(IS_DOSV || ttf_dosv) {
+#else
         if(IS_DOSV) {
+#endif
             over_key_buffer.push(code);
         }
         return false;
@@ -365,8 +374,8 @@ static bool get_key(uint16_t &code) {
         start=0x502;
         end=0x522;
     }
-    else if (machine==MCH_PCJR) {
-        /* should be done for cga and others as well, to be tested */
+    else if (machine==MCH_PCJR || machine==MCH_CGA) {
+        /* should be done for others as well, to be tested */
         start=0x1e;
         end=0x3e;
     } else {
@@ -400,7 +409,11 @@ static bool get_key(uint16_t &code) {
         mem_writew(BIOS_KEYBOARD_BUFFER_HEAD,thead);
         code = real_readw(0x40,head);
     }
+#if defined(USE_TTF)
+    if(IS_DOSV || ttf_dosv) {
+#else
     if(IS_DOSV) {
+#endif
         if (!over_key_buffer.empty()) {
             add_key(over_key_buffer.front());
             over_key_buffer.pop();
@@ -1255,7 +1268,11 @@ Bitu INT16_Handler(void) {
         break;
     case 0x13:
 #if defined(WIN32) && !defined(HX_DOS) && !defined(C_SDL2) && defined(SDL_DOSBOX_X_SPECIAL)
+#if defined(USE_TTF)
+        if((IS_DOSV || ttf_dosv) && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#else
         if(IS_DOSV && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#endif
             if(reg_al == 0x00) {
                 if(reg_dl & 0x81)
                     SDL_SetIMValues(SDL_IM_ONOFF, 1, NULL);
@@ -1271,7 +1288,11 @@ Bitu INT16_Handler(void) {
             }
         }
 #elif defined(WIN32) && !defined(HX_DOS) && defined(C_SDL2)
+#if defined(USE_TTF)
+        if((IS_DOSV || ttf_dosv) && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#else
         if(IS_DOSV && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#endif
             if(reg_al == 0x00) {
                 if(reg_dl & 0x81)
                     IME_SetEnable(TRUE);
@@ -1287,7 +1308,11 @@ Bitu INT16_Handler(void) {
 #endif
         break;
     case 0x14:
+#if defined(USE_TTF)
+        if((IS_DOSV || ttf_dosv) && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#else
         if(IS_DOSV && IS_DOS_CJK && (DOSV_GetFepCtrl() & DOSV_FEP_CTRL_IAS)) {
+#endif
             if(reg_al == 0x02) {
                 // get
                 reg_al = fep_line;
@@ -1439,14 +1464,19 @@ void BIOS_SetupKeyboard(void) {
 
         /* TODO: PCjr calls INT 48h to convert PCjr scan codes to IBM PC/XT compatible */
 
-        a = ((a >> 16) << 4) + (a & 0xFFFF);
-        /* a+0 = callback instruction (4 bytes)
-         * a+4 = iret (1 bytes) */
-        phys_writeb(a+5,0x50);          /* push ax */
-        phys_writew(a+6,0x60E4);        /* in al,60h */
-        phys_writew(a+8,0x09CD);        /* int 9h */
-        phys_writeb(a+10,0x58);         /* pop ax */
-        phys_writew(a+11,0x00EB + ((256-13)<<8));    /* jmp a+0 */
+	a = ((a >> 16) << 4) + (a & 0xFFFF);
+	/* a+0 = callback instruction (4 bytes)
+	 * a+4 = iret (1 bytes) */
+	phys_writeb(a+5,0x50);		/* push ax */
+	phys_writeb(a+6,0x1e);		/* push ds */
+	phys_writew(a+7,0xC0C7);	/* mov ax,0x0040    NTS: Do not use PUSH <imm>, that opcode does not exist on the 8086 */
+	phys_writew(a+9,0x0040);	/* <---------' */
+	phys_writew(a+11,0xD88E);	/* mov ds,ax */
+	phys_writew(a+13,0x60E4);	/* in al,60h */
+	phys_writew(a+15,0x09CD);	/* int 9h */
+	phys_writeb(a+17,0x1f);		/* pop ds */
+	phys_writeb(a+18,0x58);		/* pop ax */
+	phys_writew(a+19,0x00EB + ((256-21)<<8)); /* jmp a+0 */
     }
 
     if (IS_PC98_ARCH) {
@@ -1490,26 +1520,4 @@ void BIOS_SetupKeyboard(void) {
     //  out 0x20, al
     //  pop ax
     //  iret
-
-    if (machine==MCH_PCJR) {
-        call_irq6=CALLBACK_Allocate();
-        CALLBACK_Setup(call_irq6,NULL,CB_IRQ6_PCJR,"PCJr kb irq");
-        RealSetVec(0x0e,CALLBACK_RealPointer(call_irq6));
-        // pseudocode for CB_IRQ6_PCJR:
-        //  push ax
-        //  in al, 0x60
-        //  cmp al, 0xe0
-        //  je skip
-        //  push ds
-        //  push 0x40
-        //  pop ds
-        //  int 0x09
-        //  pop ds
-        //  label skip:
-        //  cli
-        //  mov al, 0x20
-        //  out 0x20, al
-        //  pop ax
-        //  iret
-    }
 }
