@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2002-2020  The DOSBox Team
+ *  Copyright (C) 2002-2021  The DOSBox Team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include "dosbox.h"
 #include "callback.h"
+#include "logging.h"
 #include "mem.h"
 #include "paging.h"
 #include "bios.h"
@@ -349,7 +350,7 @@ void EMS_ZeroAllocation(MemHandle mem,unsigned int pages) {
 extern bool dbg_zero_on_ems_allocmem;
 
 /* NTS: "page" in EMS refers to 16KB regions, not the 4KB memory pages we normally work with */
-static uint8_t EMM_AllocateMemory(uint16_t pages,uint16_t & dhandle,bool can_allocate_zpages) {
+uint8_t EMM_AllocateMemory(uint16_t pages,uint16_t & dhandle,bool can_allocate_zpages) {
 	/* Check for 0 page allocation */
 	if (!pages) {
 		if (!can_allocate_zpages) return EMM_ZERO_PAGES;
@@ -538,7 +539,7 @@ static uint8_t EMM_MapSegment(Bitu segment,uint16_t handle,uint16_t log_page) {
 	return EMM_ILL_PHYS;
 }
 
-static uint8_t EMM_ReleaseMemory(uint16_t handle) {
+uint8_t EMM_ReleaseMemory(uint16_t handle) {
 	/* Check for valid handle */
 	if (!ValidHandle(handle)) return EMM_INVALID_HANDLE;
 
@@ -715,7 +716,7 @@ static uint8_t GetSetHandleName(void) {
 		break;
 	case 0x01:	/* Set Handle Name */
 		if (handle>=EMM_MAX_HANDLES || emm_handles[handle].pages==NULL_HANDLE) return EMM_INVALID_HANDLE;
-		MEM_BlockRead(SegPhys(es)+reg_di,emm_handles[handle].name,8);
+		MEM_BlockRead(SegPhys(ds)+reg_si,emm_handles[handle].name,8);
 		break;
 	default:
 		LOG(LOG_MISC,LOG_ERROR)("EMS:Call %2X Subfunction %2X not supported",reg_ah,reg_al);
@@ -1689,7 +1690,7 @@ public:
 			ENABLE_V86_STARTUP=false;
 		}
 		if (ENABLE_V86_STARTUP && !ENABLE_VCPI) {
-			LOG_MSG("EMS: DOSBox does not support enabling virtual 8086 mode without VCPI.");
+			LOG_MSG("EMS: DOSBox-X does not support enabling virtual 8086 mode without VCPI.");
 			ENABLE_V86_STARTUP=false;
 		}
 
