@@ -33,11 +33,11 @@
 #include <string>
 using namespace std;
 
-int msgcodepage = 0;
 extern bool dos_kernel_disabled, force_conversion;
+int msgcodepage = 0, FileDirExistUTF8(std::string &localname, const char *name);
 bool morelen = false, loadlang = false, systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 bool isSupportedCP(int newCP), CodePageHostToGuestUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/), CodePageGuestToHostUTF8(char *d/*CROSS_LEN*/,const char *s/*CROSS_LEN*/);
-void menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname);
+void InitFontHandle(void), ShutFontHandle(void), menu_update_autocycle(void), update_bindbutton_text(void), set_eventbutton_text(const char *eventname, const char *buttonname);
 std::string langname = "", langnote = "", GetDOSBoxXPath(bool withexe=false);
 
 #define LINE_IN_MAXLEN 2048
@@ -78,6 +78,10 @@ void MSG_Replace(const char * _name, const char* _val) {
 
 bool InitCodePage() {
     if (!dos.loaded_codepage || dos_kernel_disabled || force_conversion) {
+        if (control->opt_langcp && msgcodepage>0 && isSupportedCP(msgcodepage) && msgcodepage != dos.loaded_codepage) {
+            dos.loaded_codepage = msgcodepage;
+            return true;
+        }
         Section_prop *section = static_cast<Section_prop *>(control->GetSection("config"));
         if (section!=NULL && !control->opt_noconfig) {
             char *countrystr = (char *)section->Get_string("country"), *r=strchr(countrystr, ',');
@@ -121,6 +125,11 @@ void LoadMessageFile(const char * fname) {
 	if (!mfile && exepath.size()) mfile=fopen((exepath + "languages/" + fname + ".lng").c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + "languages/" + fname).c_str(),"rt");
 	if (!mfile && config_path.size()) mfile=fopen((config_path + "languages/" + fname + ".lng").c_str(),"rt");
+#if defined(WIN32) && defined(C_SDL2)
+    std::string localname = fname;
+    if (!mfile && FileDirExistUTF8(localname, fname) == 1) mfile=fopen(localname.c_str(),"rt");
+#endif
+
 	/* This should never happen and since other modules depend on this use a normal printf */
 	if (!mfile) {
 		std::string message="Could not load language message file '"+std::string(fname)+"'. The default language will be used.";
@@ -163,7 +172,7 @@ void LoadMessageFile(const char * fname) {
                     *r=0;
                     if (!strcmp(p, "CODEPAGE")) {
                         int c = atoi(r+1);
-                        if (!res && c>0 && isSupportedCP(c)) {
+                        if ((!res || control->opt_langcp) && c>0 && isSupportedCP(c)) {
                             msgcodepage = c;
                             dos.loaded_codepage = c;
                         }
@@ -219,6 +228,15 @@ void LoadMessageFile(const char * fname) {
     menu_update_autocycle();
     update_bindbutton_text();
     dos.loaded_codepage=cp;
+    if (control->opt_langcp && msgcodepage>0 && isSupportedCP(msgcodepage) && msgcodepage != dos.loaded_codepage) {
+        ShutFontHandle();
+        if (msgcodepage == 932 || msgcodepage == 936 || msgcodepage == 949 || msgcodepage == 950) {
+            dos.loaded_codepage = msgcodepage;
+            InitFontHandle();
+            dos.loaded_codepage = cp;
+        }
+    }
+
     LOG_MSG("Loaded language file: %s",fname);
 	loadlang=true;
 }
@@ -284,6 +302,18 @@ void MSG_Init() {
 	if (control->opt_lang != "") {
 		LoadMessageFile(control->opt_lang.c_str());
 		SetVal("dosbox", "language", control->opt_lang.c_str());
+        if (control->opt_langcp && msgcodepage>0 && isSupportedCP(msgcodepage)) {
+            Section_prop *sec = static_cast<Section_prop *>(control->GetSection("config"));
+            char cstr[20];
+            cstr[0] = 0;
+            if (sec!=NULL) {
+                char *countrystr = (char *)sec->Get_string("country"), *r=strchr(countrystr, ',');
+                if (r!=NULL) *r=0;
+                if (strlen(countrystr)>10) countrystr[0] = 0;
+                sprintf(cstr, "%s,%d", countrystr, msgcodepage);
+                SetVal("config", "country", cstr);
+            }
+        }
 	}
 	else {
 		Prop_path* pathprop = section->Get_path("language");
