@@ -55,7 +55,7 @@ Bitu call_program;
 extern const char *modifier;
 extern std::string langname, configfile;
 extern int enablelfn, paste_speed, wheel_key, freesizecap, wpType, wpVersion, wpBG, wpFG, lastset, blinkCursor;
-extern bool dos_kernel_disabled, force_nocachedir, wpcolon, lockmount, enable_config_as_shell_commands, load, winrun, winautorun, startcmd, startwait, startquiet, starttranspath, mountwarning, wheel_guest, clipboard_dosapi, noremark_save_state, force_load_state, sync_time, manualtime, showbold, showital, showline, showsout, char512, printfont, dbcs_sbcs, autoboxdraw, halfwidthkana, ticksLocked;
+extern bool dos_kernel_disabled, force_nocachedir, wpcolon, lockmount, enable_config_as_shell_commands, load, winrun, winautorun, startcmd, startwait, startquiet, starttranspath, mountwarning, wheel_guest, clipboard_dosapi, noremark_save_state, force_load_state, sync_time, manualtime, showbold, showital, showline, showsout, char512, printfont, rtl, dbcs_sbcs, autoboxdraw, halfwidthkana, ticksLocked;
 
 /* This registers a file on the virtual drive and creates the correct structure for it*/
 
@@ -1299,6 +1299,11 @@ void CONFIG::Run(void) {
 								mainMenu.get_item("wheel_none").check(wheel_key==0).refresh_item(mainMenu);
 								mainMenu.get_item("wheel_guest").check(wheel_guest).refresh_item(mainMenu);
 							}
+							if (!strcasecmp(inputline.substr(0, 12).c_str(), "sensitivity=")) {
+                                Prop_multival* p3 = static_cast<Section_prop *>(section)->Get_multival("sensitivity");
+                                sdl.mouse.xsensitivity = p3->GetSection()->Get_int("xsens");
+                                sdl.mouse.ysensitivity = p3->GetSection()->Get_int("ysens");
+                            }
 							if (!strcasecmp(inputline.substr(0, 11).c_str(), "fullscreen=")) {
                                 if (section->Get_bool("fullscreen")) {
                                     if (!GFX_IsFullscreen()) {GFX_LosingFocus();GFX_SwitchFullScreen();}
@@ -1582,6 +1587,12 @@ void CONFIG::Run(void) {
 #if defined(USE_TTF)
                                 char512 = section->Get_bool("char512");
                                 if (TTF_using()) resetFontSize();
+#endif
+							} else if (!strcasecmp(inputline.substr(0, 12).c_str(), "righttoleft=")) {
+#if defined(USE_TTF)
+                                rtl = section->Get_bool("righttoleft");
+                                mainMenu.get_item("ttf_right_left").check(rtl).refresh_item(mainMenu);
+                                if (TTF_using()) resetFontSize();
 #endif
 							} else if (!strcasecmp(inputline.substr(0, 10).c_str(), "printfont=")) {
 #if defined(USE_TTF) && C_PRINTER
