@@ -18,8 +18,11 @@
 
 #include "dosbox.h"
 #include "control.h"
+#include "logging.h"
 #include "menu.h"
 
+bool informd3d = false;
+
 #if (HAVE_D3D9_H) && defined(WIN32)
 
 extern Bitu currentWindowWidth, currentWindowHeight;
@@ -57,11 +60,11 @@ std::string shader_translate_directory(const std::string& path) {
     /* DOSBox fork compatability: if only the name of a file is given, assume it
        exists in the shaders\ directory.
 
-       This fork's variation is to NOT prefix shaders\ to it if it looks like a
+       DOSBox-X's variation is to NOT prefix shaders\ to it if it looks like a
        full path, with or without a drive letter. */
     if (path.length() >= 2 && isalpha(path[0]) && path[1] == ':') /* drive letter ex. C:, D:, etc. */
         return path;
-    if (path.length() >= 1 && path[0] == '\\') /* perhaps a UNC path or an absolute path from the current drive */
+    if (path.length() >= 1 && path.find('\\') != std::string::npos) /* perhaps a path with "\" */
         return path;
 
     return std::string("shaders\\") + path;
@@ -121,7 +124,11 @@ HRESULT CDirect3D::InitializeDX(HWND wnd, bool triplebuf)
 
     thread_run = true;
     thread_command = D3D_IDLE;
+#if defined(C_SDL2)
+    thread = SDL_CreateThread(EntryPoint, "Direct3D", this);
+#else
     thread = SDL_CreateThread(EntryPoint, this);
+#endif
     SDL_SemWait(thread_ack);
 #endif
 
@@ -950,6 +957,7 @@ HRESULT CDirect3D::RestoreDeviceObjects(void)
 }
 
 extern void RENDER_SetForceUpdate(bool);
+extern bool systemmessagebox(char const * aTitle, char const * aMessage, char const * aDialogType, char const * aIconType, int aDefaultButton);
 HRESULT CDirect3D::LoadPixelShader(const char * shader, double scalex, double scaley, bool forced)
 {
     if(!psEnabled) {
@@ -990,12 +998,17 @@ HRESULT CDirect3D::LoadPixelShader(const char * shader, double scalex, double sc
 	// Compare optimal scaling factor
 	bool dblgfx=((scalex < scaley ? scalex : scaley) >= psEffect->getScale());
 
+    std::string message;
 	if(dblgfx || forced) {
+	    message = "Loaded pixel shader - "+std::string(shader);
+	    if (informd3d) systemmessagebox("Direct3D shader", message.c_str(), "ok","info", 1);
 	    LOG_MSG("D3D:Pixel shader %s active", shader);
 	    RENDER_SetForceUpdate(psEffect->getForceUpdate());
 	    psActive = true;
 	    return S_OK;
 	} else {
+	    message = "Pixel shader not needed - "+std::string(shader);
+	    if (informd3d) systemmessagebox("Direct3D shader", message.c_str(), "ok","info", 1);
 	    LOG_MSG("D3D:Pixel shader not needed");
 	    psActive = false;
 	    return E_FAIL;
@@ -1398,7 +1411,7 @@ HRESULT CDirect3D::CreateVertex(void)
     if (psActive) {
         vertices[0].position = D3DXVECTOR3(-0.5f, -0.5f, 0.0f);
         vertices[0].diffuse = 0xFFFFFFFF;
-        vertices[0].texcoord = D3DXVECTOR2(0.0f, (float)sizey);
+        vertices[0].texcoord = D3DXVECTOR2(0.0f, sizey);
 
         vertices[1].position = D3DXVECTOR3(-0.5f, 0.5f, 0.0f);
         vertices[1].diffuse = 0xFFFFFFFF;
@@ -1406,11 +1419,11 @@ HRESULT CDirect3D::CreateVertex(void)
 
         vertices[2].position = D3DXVECTOR3(0.5f, -0.5f, 0.0f);
         vertices[2].diffuse = 0xFFFFFFFF;
-        vertices[2].texcoord = D3DXVECTOR2((float)sizex, (float)sizey);
+        vertices[2].texcoord = D3DXVECTOR2(sizex, sizey);
 
         vertices[3].position = D3DXVECTOR3(0.5f, 0.5f, 0.0f);
         vertices[3].diffuse = 0xFFFFFFFF;
-        vertices[3].texcoord = D3DXVECTOR2((float)sizex, 0.0f);
+        vertices[3].texcoord = D3DXVECTOR2(sizex, 0.0f);
     }
     else {
         vertices[0].position = D3DXVECTOR3((float)dwX, (float)dwY, 0.0f);
@@ -1419,15 +1432,15 @@ HRESULT CDirect3D::CreateVertex(void)
 
         vertices[1].position = D3DXVECTOR3((float)dwX, (float)(dwY + dwScaledHeight), 0.0f);
         vertices[1].diffuse = 0xFFFFFFFF;
-        vertices[1].texcoord = D3DXVECTOR2(0.0f, (float)sizey);
+        vertices[1].texcoord = D3DXVECTOR2(0.0f, sizey);
 
         vertices[2].position = D3DXVECTOR3((float)(dwX + dwScaledWidth), (float)dwY, 0.0f);
         vertices[2].diffuse = 0xFFFFFFFF;
-        vertices[2].texcoord = D3DXVECTOR2((float)sizex, 0.0f);
+        vertices[2].texcoord = D3DXVECTOR2(sizex, 0.0f);
 
         vertices[3].position = D3DXVECTOR3((float)(dwX + dwScaledWidth), (float)(dwY + dwScaledHeight), 0.0f);
         vertices[3].diffuse = 0xFFFFFFFF;
-        vertices[3].texcoord = D3DXVECTOR2((float)sizex, (float)sizey);
+        vertices[3].texcoord = D3DXVECTOR2(sizex, sizey);
     }
 
     // Additional vertices required for some PS effects
