@@ -7,8 +7,14 @@
 #include <sys/types.h>
 #include <assert.h>
 #include <math.h>
-
+extern "C" {
+#include "ppscale.h"
+#include "ppscale.c"
+}
+#include "control.h"
 #include "dosbox.h"
+#include "logging.h"
+#include "menudef.h"
 #include <output/output_opengl.h>
 
 #include <algorithm>
@@ -79,19 +85,68 @@ PFNGLVERTEXATTRIBPOINTERPROC glVertexAttribPointer = NULL;
 #define glVertexAttribPointer     gl2::glVertexAttribPointer
 
 #if C_OPENGL && DOSBOXMENU_TYPE == DOSBOXMENU_SDLDRAW
-extern unsigned int SDLDrawGenFontTextureUnitPerRow;
-extern unsigned int SDLDrawGenFontTextureRows;
 extern unsigned int SDLDrawGenFontTextureWidth;
 extern unsigned int SDLDrawGenFontTextureHeight;
-extern GLuint SDLDrawGenFontTexture;
+extern GLuint SDLDrawGenFontTexture, SDLDrawGenDBCSFontTexture;
 extern bool SDLDrawGenFontTextureInit;
 #endif
+extern int initgl, lastcp;
+extern bool font_16_init;
+extern uint8_t int10_font_16[256 * 16], int10_font_16_init[256 * 16];
 
 SDL_OpenGL sdl_opengl;
 
 int Voodoo_OGL_GetWidth();
 int Voodoo_OGL_GetHeight();
 bool Voodoo_OGL_Active();
+bool InitCodePage();
+uint8_t *GetDbcsFont(Bitu code);
+
+// NTS: With high DPI displays (e.g. on Windows 7+ with DPI scaling enabled)
+//      this works better with maximized window or full-screen mode and the
+//      setting "dpi aware=true".
+
+static void PPScale (
+    uint16_t  fixed_w , uint16_t  fixed_h,
+    uint16_t* window_w, uint16_t* window_h )
+{
+    int    sx, sy, orig_w, orig_h, min_w, min_h;
+    double par, par_sq;
+
+    orig_w = min_w = (int)render.src.width;
+    orig_h = min_h = (int)render.src.height;
+
+    par = ( double) orig_w / orig_h * 3 / 4;
+    /* HACK: because RENDER_SetSize() does not set dblw and dblh correctly: */
+    /* E.g. in 360x360 mode DOSBox-X will wrongly allocate a 720x360 area. I  */
+    /* therefore calculate square-pixel proportions par_sq myself:          */
+         if( par < 0.707 ) { par_sq = 0.5; min_w *= 2; }
+    else if( par > 1.414 ) { par_sq = 2.0; min_h *= 2; }
+    else                     par_sq = 1.0;
+
+    if( !render.aspect ) par = par_sq;
+
+    *window_w = fixed_w; *window_h = fixed_h;
+    /* Handle non-fixed resolutions and ensure a sufficient window size: */
+    if( fixed_w < min_w ) fixed_w = *window_w = min_w;
+    if( fixed_h < min_h ) fixed_h = *window_h = min_h;
+
+    pp_getscale(
+        orig_w , orig_h , par ,
+        fixed_w, fixed_h, 1.14,
+        &sx    , &sy         );
+
+    sdl.clip.w = orig_w * sx;
+    sdl.clip.h = orig_h * sy;
+    sdl.clip.x = (*window_w - sdl.clip.w) / 2;
+    sdl.clip.y = (*window_h - sdl.clip.h) / 2;
+
+    LOG_MSG( "OpenGL PP: [%ix%i]: %ix%i (%3.2f) -> [%ix%i] -> %ix%i (%3.2f)",
+        fixed_w,    fixed_h,
+        orig_w,     orig_h, par,
+        sx,         sy,
+        sdl.clip.w, sdl.clip.h, (double)sy/sx );
+}
 
 static SDL_Surface* SetupSurfaceScaledOpenGL(uint32_t sdl_flags, uint32_t bpp) 
 {
@@ -160,7 +215,7 @@ retry:
         LOG_MSG("menuScale=%d", scale);
         mainMenu.setScale((unsigned int)scale);
 
-        if (mainMenu.isVisible() && !sdl.desktop.fullscreen)
+        if (mainMenu.isVisible() && !sdl.desktop.fullscreen && fixedHeight)
             fixedHeight -= mainMenu.menuBox.h;
     }
 #endif
@@ -171,20 +226,24 @@ retry:
         /* 3Dfx openGL do not allow resize */
         sdl.clip.w = windowWidth = (uint16_t)Voodoo_OGL_GetWidth();
         sdl.clip.h = windowHeight = (uint16_t)Voodoo_OGL_GetHeight();
-    }
-    else if (fixedWidth && fixedHeight) 
-    {
-        sdl.clip.w = windowWidth = fixedWidth;
-        sdl.clip.h = windowHeight = fixedHeight;
-        if (render.aspect) aspectCorrectFitClip(sdl.clip.w, sdl.clip.h, sdl.clip.x, sdl.clip.y, fixedWidth, fixedHeight);
-    }
-    else 
-    {
-        windowWidth = (uint16_t)(sdl.draw.width * sdl.draw.scalex);
-        windowHeight = (uint16_t)(sdl.draw.height * sdl.draw.scaley);
-        if (render.aspect) aspectCorrectExtend(windowWidth, windowHeight);
-        sdl.clip.w = windowWidth; sdl.clip.h = windowHeight;
-    }
+    } else if (sdl_opengl.kind == GLPerfect ) {
+        PPScale( fixedWidth, fixedHeight, &windowWidth, &windowHeight );
+    } else
+        if (fixedWidth && fixedHeight)
+        {
+            windowWidth  = fixedWidth;
+            windowHeight = fixedHeight;
+            sdl.clip.w = windowWidth;
+            sdl.clip.h = windowHeight;
+            if (render.aspect) aspectCorrectFitClip(sdl.clip.w, sdl.clip.h, sdl.clip.x, sdl.clip.y, fixedWidth, fixedHeight);
+        }
+        else
+        {
+            windowWidth = (uint16_t)(sdl.draw.width * sdl.draw.scalex);
+            windowHeight = (uint16_t)(sdl.draw.height * sdl.draw.scaley);
+            if (render.aspect) aspectCorrectExtend(windowWidth, windowHeight);
+            sdl.clip.w = windowWidth; sdl.clip.h = windowHeight;
+        }
 
     LOG(LOG_MISC, LOG_DEBUG)("GFX_SetSize OpenGL window=%ux%u clip=x,y,w,h=%d,%d,%d,%d",
         (unsigned int)windowWidth,
@@ -207,8 +266,10 @@ retry:
     sdl.surface = NULL;
     sdl.window = GFX_SetSDLWindowMode(windowWidth, windowHeight, (sdl_flags & SDL_WINDOW_OPENGL) ? SCREEN_OPENGL : SCREEN_SURFACE);
     if (sdl.window != NULL) sdl.surface = SDL_GetWindowSurface(sdl.window);
-#else
+#elif defined(SDL_DOSBOX_X_SPECIAL)
     sdl.surface = SDL_SetVideoMode(windowWidth, windowHeight, (int)bpp, (unsigned int)sdl_flags | (unsigned int)(setSizeButNotResize() ? SDL_HAX_NORESIZEWINDOW : 0));
+#else
+    sdl.surface = SDL_SetVideoMode(windowWidth, windowHeight, (int)bpp, (unsigned int)sdl_flags);
 #endif
     if (sdl.surface == NULL && sdl.desktop.fullscreen) {
         LOG_MSG("Fullscreen not supported: %s", SDL_GetError());
@@ -254,7 +315,7 @@ void OUTPUT_OPENGL_Initialize()
     memset(&sdl_opengl, 0, sizeof(sdl_opengl));
 }
 
-void OUTPUT_OPENGL_Select()
+void OUTPUT_OPENGL_Select( GLKind kind )
 {
     sdl.desktop.want_type = SCREEN_OPENGL;
     render.aspectOffload = true;
@@ -262,6 +323,80 @@ void OUTPUT_OPENGL_Select()
 #if defined(WIN32) && !defined(C_SDL2)
     SDL1_hax_inhibit_WM_PAINT = 0;
 #endif
+
+    sdl_opengl.use_shader = false;
+    initgl=0;
+#if defined(C_SDL2)
+    SDL_SetHint(SDL_HINT_FRAMEBUFFER_ACCELERATION, "opengl");
+    void GFX_SetResizeable(bool enable);
+    GFX_SetResizeable(true);
+    sdl.window = GFX_SetSDLWindowMode(640,400, SCREEN_OPENGL);
+    if (sdl.window) {
+        sdl_opengl.context = SDL_GL_CreateContext(sdl.window);
+        sdl.surface = SDL_GetWindowSurface(sdl.window);
+    }
+    if (!sdl.window || !sdl_opengl.context || sdl.surface == NULL) {
+#else
+    sdl.surface = SDL_SetVideoMode(640,400,0,SDL_OPENGL);
+    if (sdl.surface == NULL) {
+#endif
+        LOG_MSG("Could not initialize OpenGL, switching back to surface");
+        sdl.desktop.want_type = SCREEN_SURFACE;
+    } else if (initgl!=2) {
+        initgl = 1;
+        sdl_opengl.kind = kind;
+        sdl.desktop.isperfect = kind == GLPerfect;
+        sdl_opengl.program_object = 0;
+        glAttachShader = (PFNGLATTACHSHADERPROC)SDL_GL_GetProcAddress("glAttachShader");
+        glCompileShader = (PFNGLCOMPILESHADERPROC)SDL_GL_GetProcAddress("glCompileShader");
+        glCreateProgram = (PFNGLCREATEPROGRAMPROC)SDL_GL_GetProcAddress("glCreateProgram");
+        glCreateShader = (PFNGLCREATESHADERPROC)SDL_GL_GetProcAddress("glCreateShader");
+        glDeleteProgram = (PFNGLDELETEPROGRAMPROC)SDL_GL_GetProcAddress("glDeleteProgram");
+        glDeleteShader = (PFNGLDELETESHADERPROC)SDL_GL_GetProcAddress("glDeleteShader");
+        glEnableVertexAttribArray = (PFNGLENABLEVERTEXATTRIBARRAYPROC)SDL_GL_GetProcAddress("glEnableVertexAttribArray");
+        glGetAttribLocation = (PFNGLGETATTRIBLOCATIONPROC)SDL_GL_GetProcAddress("glGetAttribLocation");
+        glGetProgramiv = (PFNGLGETPROGRAMIVPROC)SDL_GL_GetProcAddress("glGetProgramiv");
+        glGetProgramInfoLog = (PFNGLGETPROGRAMINFOLOGPROC)SDL_GL_GetProcAddress("glGetProgramInfoLog");
+        glGetShaderiv = (PFNGLGETSHADERIVPROC)SDL_GL_GetProcAddress("glGetShaderiv");
+        glGetShaderInfoLog = (PFNGLGETSHADERINFOLOGPROC)SDL_GL_GetProcAddress("glGetShaderInfoLog");
+        glGetUniformLocation = (PFNGLGETUNIFORMLOCATIONPROC)SDL_GL_GetProcAddress("glGetUniformLocation");
+        glLinkProgram = (PFNGLLINKPROGRAMPROC)SDL_GL_GetProcAddress("glLinkProgram");
+        glShaderSource = (PFNGLSHADERSOURCEPROC_NP)SDL_GL_GetProcAddress("glShaderSource");
+        glUniform2f = (PFNGLUNIFORM2FPROC)SDL_GL_GetProcAddress("glUniform2f");
+        glUniform1i = (PFNGLUNIFORM1IPROC)SDL_GL_GetProcAddress("glUniform1i");
+        glUseProgram = (PFNGLUSEPROGRAMPROC)SDL_GL_GetProcAddress("glUseProgram");
+        glVertexAttribPointer = (PFNGLVERTEXATTRIBPOINTERPROC)SDL_GL_GetProcAddress("glVertexAttribPointer");
+        sdl_opengl.use_shader = (glAttachShader && glCompileShader && glCreateProgram && glDeleteProgram && glDeleteShader && \
+            glEnableVertexAttribArray && glGetAttribLocation && glGetProgramiv && glGetProgramInfoLog && \
+            glGetShaderiv && glGetShaderInfoLog && glGetUniformLocation && glLinkProgram && glShaderSource && \
+            glUniform2f && glUniform1i && glUseProgram && glVertexAttribPointer);
+        if (sdl_opengl.use_shader) initgl = 2;
+        sdl_opengl.buffer=0;
+        sdl_opengl.framebuf=0;
+        sdl_opengl.texture=0;
+        sdl_opengl.displaylist=0;
+        glGetIntegerv (GL_MAX_TEXTURE_SIZE, &sdl_opengl.max_texsize);
+        glGenBuffersARB = (PFNGLGENBUFFERSARBPROC)SDL_GL_GetProcAddress("glGenBuffersARB");
+        glBindBufferARB = (PFNGLBINDBUFFERARBPROC)SDL_GL_GetProcAddress("glBindBufferARB");
+        glDeleteBuffersARB = (PFNGLDELETEBUFFERSARBPROC)SDL_GL_GetProcAddress("glDeleteBuffersARB");
+        glBufferDataARB = (PFNGLBUFFERDATAARBPROC)SDL_GL_GetProcAddress("glBufferDataARB");
+        glMapBufferARB = (PFNGLMAPBUFFERARBPROC)SDL_GL_GetProcAddress("glMapBufferARB");
+        glUnmapBufferARB = (PFNGLUNMAPBUFFERARBPROC)SDL_GL_GetProcAddress("glUnmapBufferARB");
+        const char * gl_ext = (const char *)glGetString (GL_EXTENSIONS);
+        if(gl_ext && *gl_ext){
+            sdl_opengl.packed_pixel=(strstr(gl_ext,"EXT_packed_pixels") != NULL);
+            sdl_opengl.paletted_texture=(strstr(gl_ext,"EXT_paletted_texture") != NULL);
+            //sdl_opengl.pixel_buffer_object=(strstr(gl_ext,"GL_ARB_pixel_buffer_object") != NULL ) && glGenBuffersARB && glBindBufferARB && glDeleteBuffersARB && glBufferDataARB && glMapBufferARB && glUnmapBufferARB;
+        } else {
+            sdl_opengl.packed_pixel = false;
+            sdl_opengl.paletted_texture = false;
+            //sdl_opengl.pixel_buffer_object = false;
+        }
+#ifdef DB_DISABLE_DBO
+        sdl_opengl.pixel_buffer_object = false;
+#endif
+        //LOG_MSG("OpenGL extension: pixel_buffer_object %d",sdl_opengl.pixel_buffer_object);
+	} /* OPENGL is requested end */
 }
 
 Bitu OUTPUT_OPENGL_GetBestMode(Bitu flags)
@@ -290,7 +425,7 @@ static GLuint BuildShader ( GLenum type, const char *shaderSrc ) {
 	}
 
 	top += (type==GL_VERTEX_SHADER) ? "#define VERTEX 1\n":"#define FRAGMENT 1\n";
-	if (!sdl_opengl.bilinear)
+	if (sdl_opengl.kind == GLNearest || sdl_opengl.kind == GLPerfect)
 		top += "#define OPENGLNB 1\n";
 
 	src_strings[0] = top.c_str();
@@ -342,6 +477,88 @@ static bool LoadGLShaders(const char *src, GLuint *vertex, GLuint *fragment) {
 	return false;
 }
 
+#if DOSBOXMENU_TYPE == DOSBOXMENU_SDLDRAW
+void UpdateSDLDrawTexture() {
+    glBindTexture(GL_TEXTURE_2D, SDLDrawGenFontTexture);
+
+    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, 0);
+
+    // No borders
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, (int)SDLDrawGenFontTextureWidth, (int)SDLDrawGenFontTextureHeight, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, 0);
+
+    /* load the font */
+    {
+        int cp = dos.loaded_codepage;
+        if (!cp) InitCodePage();
+        uint32_t tmp[8 * 16];
+        unsigned int x, y, c;
+
+        for (c = 0; c < 256; c++)
+        {
+            unsigned char *bmp;
+            if (font_16_init&&dos.loaded_codepage&&dos.loaded_codepage!=437)
+                bmp = int10_font_16_init + (c * 16);
+            else
+                bmp = int10_font_16 + (c * 16);
+            for (y = 0; y < 16; y++)
+                for (x = 0; x < 8; x++)
+                    tmp[(y * 8) + x] = (bmp[y] & (0x80 >> x)) ? 0xFFFFFFFFUL : 0x00000000UL;
+
+            glTexSubImage2D(GL_TEXTURE_2D, /*level*/0, /*x*/(int)((c % 16) * 8), /*y*/(int)((c / 16) * 16),
+                8, 16, GL_BGRA_EXT, GL_UNSIGNED_INT_8_8_8_8_REV, (void*)tmp);
+        }
+        lastcp = dos.loaded_codepage;
+        dos.loaded_codepage = cp;
+    }
+
+    glBindTexture(GL_TEXTURE_2D, 0);
+}
+
+void UpdateSDLDrawDBCSTexture(Bitu code) {
+    glBindTexture(GL_TEXTURE_2D, SDLDrawGenDBCSFontTexture);
+
+    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
+    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, 0);
+
+    // No borders
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, (int)SDLDrawGenFontTextureWidth, (int)SDLDrawGenFontTextureHeight, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, 0);
+
+    /* load the font */
+    {
+        int cp = dos.loaded_codepage;
+        if (!cp) InitCodePage();
+        uint32_t tmp[8 * 16];
+        unsigned int x, y, c;
+
+        for (c = 0; c < (code?2:1); c++)
+        {
+            unsigned char *bmp = code?GetDbcsFont(code):(int10_font_16 + 0xFB * 16);
+            for (y = 0; y < 16; y++)
+                for (x = 0; x < 8; x++)
+                    tmp[(y * 8) + x] = (bmp[code?(y*2+c):y] & (0x80 >> x)) ? 0xFFFFFFFFUL : 0x00000000UL;
+
+            glTexSubImage2D(GL_TEXTURE_2D, /*level*/0, /*x*/(int)(c * 8), /*y*/0,
+                8, 16, GL_BGRA_EXT, GL_UNSIGNED_INT_8_8_8_8_REV, (void*)tmp);
+        }
+        lastcp = dos.loaded_codepage;
+        dos.loaded_codepage = cp;
+    }
+
+    glBindTexture(GL_TEXTURE_2D, 0);
+}
+#endif
+
 Bitu OUTPUT_OPENGL_SetSize()
 {
     Bitu retFlags = 0;
@@ -366,16 +583,14 @@ Bitu OUTPUT_OPENGL_SetSize()
     sdl_opengl.framebuf = 0;
 
     SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
-#if !defined(C_SDL2)
-# if SDL_VERSION_ATLEAST(1, 2, 11)
-    {
-        Section_prop* sec = static_cast<Section_prop*>(control->GetSection("vsync"));
-
-        if (sec)
-            SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, (!strcmp(sec->Get_string("vsyncmode"), "host")) ? 1 : 0);
-    }
-# endif
+    Section_prop* sec = static_cast<Section_prop*>(control->GetSection("vsync"));
+    if (sec) {
+#if defined(C_SDL2)
+        SDL_GL_SetSwapInterval((!strcmp(sec->Get_string("vsyncmode"), "host")) ? 1 : 0);
+#elif SDL_VERSION_ATLEAST(1, 2, 11)
+        SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, (!strcmp(sec->Get_string("vsyncmode"), "host")) ? 1 : 0);
 #endif
+    }
 
     // try 32 bits first then 16
 #if defined(C_SDL2)
@@ -489,15 +704,19 @@ Bitu OUTPUT_OPENGL_SetSize()
                 glUseProgram(sdl_opengl.program_object);
 
                 GLint u = glGetAttribLocation(sdl_opengl.program_object, "a_position");
+                // NTS: This is now a triangle strip (GL_TRIANGLE_STRIP)
                 // upper left
                 sdl_opengl.vertex_data[0] = -1.0f;
-                sdl_opengl.vertex_data[1] = 1.0f;
+                sdl_opengl.vertex_data[1] =  1.0f;
                 // lower left
                 sdl_opengl.vertex_data[2] = -1.0f;
-                sdl_opengl.vertex_data[3] = -3.0f;
+                sdl_opengl.vertex_data[3] = -1.0f;
                 // upper right
-                sdl_opengl.vertex_data[4] = 3.0f;
-                sdl_opengl.vertex_data[5] = 1.0f;
+                sdl_opengl.vertex_data[4] =  1.0f;
+                sdl_opengl.vertex_data[5] =  1.0f;
+                // lower right
+                sdl_opengl.vertex_data[6] =  1.0f;
+                sdl_opengl.vertex_data[7] = -1.0f;
                 // Load the vertex positions
                 glVertexAttribPointer(u, 2, GL_FLOAT, GL_FALSE, 0, sdl_opengl.vertex_data);
                 glEnableVertexAttribArray(u);
@@ -540,12 +759,10 @@ Bitu OUTPUT_OPENGL_SetSize()
     }
 #endif
 
-    if (!sdl.desktop.fullscreen)
-        glViewport(0, 0, sdl.surface->w, sdl.surface->h);
-    else if (sdl_opengl.use_shader)
+    if (sdl.desktop.fullscreen&&sdl_opengl.use_shader)
         glViewport((sdl.surface->w-sdl.clip.w)/2,(sdl.surface->h-sdl.clip.h)/2,sdl.clip.w,sdl.clip.h);
     else
-        glViewport(sdl.clip.x,sdl.clip.y,sdl.clip.w,sdl.clip.h);
+        glViewport(0, 0, sdl.surface->w, sdl.surface->h);
     if (sdl_opengl.texture > 0) glDeleteTextures(1, &sdl_opengl.texture);
     glGenTextures(1, &sdl_opengl.texture);
     glBindTexture(GL_TEXTURE_2D, sdl_opengl.texture);
@@ -556,8 +773,12 @@ Bitu OUTPUT_OPENGL_SetSize()
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
     glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
 
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, sdl_opengl.bilinear ? GL_LINEAR : GL_NEAREST);
-    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, sdl_opengl.bilinear ? GL_LINEAR : GL_NEAREST);
+    GLint interp;
+    if( sdl_opengl.kind == GLNearest || sdl_opengl.kind == GLPerfect )
+        interp = GL_NEAREST; else
+        interp = GL_LINEAR ;
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, interp );
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, interp );
 
     glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, texsize, texsize, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, 0);
 
@@ -650,45 +871,14 @@ Bitu OUTPUT_OPENGL_SetSize()
 
         SDLDrawGenFontTextureInit = 1;
 
-        glBindTexture(GL_TEXTURE_2D, SDLDrawGenFontTexture);
-
-        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, 0);
-
-        // No borders
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
-        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
-
-        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, (int)SDLDrawGenFontTextureWidth, (int)SDLDrawGenFontTextureHeight, 0, GL_BGRA_EXT, GL_UNSIGNED_BYTE, 0);
-
-        /* load the font */
-        {
-            extern uint8_t int10_font_16[256 * 16];
-
-            uint32_t tmp[8 * 16];
-            unsigned int x, y, c;
-
-            for (c = 0; c < 256; c++) 
-            {
-                unsigned char *bmp = int10_font_16 + (c * 16);
-                for (y = 0; y < 16; y++) 
-                {
-                    for (x = 0; x < 8; x++) 
-                    {
-                        tmp[(y * 8) + x] = (bmp[y] & (0x80 >> x)) ? 0xFFFFFFFFUL : 0x00000000UL;
-                    }
-                }
-
-                glTexSubImage2D(GL_TEXTURE_2D, /*level*/0, /*x*/(int)((c % 16) * 8), /*y*/(int)((c / 16) * 16),
-                    8, 16, GL_BGRA_EXT, GL_UNSIGNED_INT_8_8_8_8_REV, (void*)tmp);
-            }
-        }
-
-        glBindTexture(GL_TEXTURE_2D, 0);
+        UpdateSDLDrawTexture();
     }
-    // }
+
+    err = 0;
+    glGetError(); /* read and discard last error */
+
+    SDLDrawGenDBCSFontTexture = (GLuint)(~0UL);
+    glGenTextures(1, &SDLDrawGenDBCSFontTexture);
 #endif
 
     glFinish();
@@ -770,7 +960,7 @@ void OUTPUT_OPENGL_EndUpdate(const uint16_t *changedLines)
                 }
                 else
                 {
-                    trgTex = reinterpret_cast<uint32_t*>(static_cast<void*>(sdl_opengl.framebuf));
+                    trgTex = reinterpret_cast<uint32_t*>(sdl_opengl.framebuf);
                 }
 
                 if (trgTex)
@@ -868,7 +1058,7 @@ void OUTPUT_OPENGL_EndUpdate(const uint16_t *changedLines)
             return;
         if (sdl_opengl.program_object) {
             glUniform1i(sdl_opengl.ruby.frame_count, sdl_opengl.actual_frame_count++);
-            glDrawArrays(GL_TRIANGLES, 0, 3);
+            glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
         } else
             glCallList(sdl_opengl.displaylist);
 
