@@ -32,6 +32,7 @@
 #include "callback.h"
 #include "support.h"
 #include "inout.h"
+#include "render.h"
 #include "../ints/int10.h"
 #include "../dos/drives.h"
 
@@ -49,9 +50,13 @@
 bool clearline=false, inshell=false;
 int autofixwarn=3;
 extern int lfn_filefind_handle;
-extern bool ctrlbrk, gbk;
+extern bool ctrlbrk, gbk, rtl;
 extern bool DOS_BreakFlag;
 extern bool DOS_BreakConioFlag;
+extern uint16_t cmd_line_seg;
+#if defined(USE_TTF)
+extern bool ttf_dosv;
+#endif
 
 void DOS_Shell::ShowPrompt(void) {
 	char dir[DOS_PATHLENGTH];
@@ -324,13 +329,13 @@ static void RemoveAllChar(char *line, uint16_t str_index)
 	for ( ; str_index > 0 ; str_index--) {
 		// removes all characters
 		if(CheckHat(line[str_index])) {
-			outc(8); outc(8); outc(' '); outc(' '); outc(8); outc(8);
+			backone(); backone(); outc(' '); outc(' '); backone(); backone();
 		} else {
-			outc(8); outc(' '); outc(8);
+			backone(); outc(' '); backone();
 		}
 	}
 	if(CheckHat(line[0])) {
-		outc(8); outc(' '); outc(8);
+		backone(); outc(' '); backone();
 	}
 }
 
@@ -355,6 +360,8 @@ static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_inde
 		DOS_WriteFile(STDOUT, (uint8_t *)&line[pos], &len);
 		pos++;
 	}
+    if (delete_flag && str_index >= str_len)
+        return 0;
 	RemoveAllChar(line, str_len);
 	pos = delete_flag ? str_index : str_index - count;
 	while(pos < str_len - count) {
@@ -370,9 +377,9 @@ static uint16_t DeleteBackspace(bool delete_flag, char *line, uint16_t &str_inde
 	DOS_WriteFile(STDOUT, (uint8_t *)line, &len);
 	pos = str_len;
 	while(pos > str_index) {
-		outc(8);
+		backone();
 		if(CheckHat(line[pos - 1])) {
-			outc(8);
+			backone();
 		}
 		pos--;
 	}
@@ -389,6 +396,30 @@ void DOS_Shell::InputCommand(char * line) {
 	bool current_hist=false; // current command stored in history?
     uint16_t cr;
 
+#if defined(USE_TTF)
+	if(IS_DOSV || ttf_dosv) {
+#else
+	if(IS_DOSV) {
+#endif
+		uint16_t int21_seg = mem_readw(0x0086);
+		if(int21_seg != 0xf000) {
+			if(real_readw(int21_seg - 1, 8) == 0x5a56) {
+                // Vz editor resident
+                real_writeb(cmd_line_seg, 0, 250);
+				reg_dx = 0;
+				reg_ah = 0x0a;
+				SegSet16(ds, cmd_line_seg);
+				CALLBACK_RunRealInt(0x21);
+				str_len = real_readb(cmd_line_seg, 1);
+				for(len = 0 ; len < str_len ; len++) {
+					line[len] = real_readb(cmd_line_seg, 2 + len);
+				}
+				line[str_len] = '\0';
+				return;
+			}
+		}
+	}
+
     inshell = true;
     input_eof = false;
 	line[0] = '\0';
@@ -502,6 +533,14 @@ void DOS_Shell::InputCommand(char * line) {
             }
         }
 
+#if defined(USE_TTF)
+        if (ttf.inUse && rtl) {
+            if (cr == 0x4B00) cr = 0x4D00;
+            else if (cr == 0x4D00) cr = 0x4B00;
+            else if (cr == 0x7300) cr = 0x7400;
+            else if (cr == 0x7400) cr = 0x7300;
+        }
+#endif
         switch (cr) {
             case 0x3d00:	/* F3 */
                 if (!l_history.size()) break;
@@ -524,12 +563,12 @@ void DOS_Shell::InputCommand(char * line) {
                         uint16_t count = GetWideCount(line, str_index);
                         uint8_t ch = line[str_index - 1];
                         while(count > 0) {
-                            outc(8);
+                            backone();
                             str_index --;
                             count--;
                         }
                         if(CheckHat(ch)) {
-                            outc(8);
+                            backone();
                         }
                     }
                 } else {
@@ -697,7 +736,7 @@ void DOS_Shell::InputCommand(char * line) {
                 break;
             case 0x5300:/* DELETE */
                 if(IS_PC98_ARCH || (isDBCSCP() && IS_DOS_JAPANESE)) {
-                    if(str_index) {
+                    if(str_len) {
                         size += DeleteBackspace(true, line, str_index, str_len);
                     }
                 } else {
@@ -942,10 +981,10 @@ void DOS_Shell::InputCommand(char * line) {
                     }
                     pos = str_len;
                     while(pos > str_index) {
-                        outc(8);
+                        backone();
                         pos--;
                         if (CheckHat(line[pos])) {
-                            outc(8);
+                            backone();
                         }
                     }
                 } else {
