@@ -25,14 +25,6 @@ fi
 
 ac_cv_header_iconv_h=no ac_cv_func_iconv=no ac_cv_lib_iconv_libiconv_open=no ../configure "--srcdir=$srcdir" "--prefix=$instdir" --enable-static --disable-shared --disable-x11-shared --disable-video-x11-xrandr --disable-video-x11-vm --disable-video-x11-xv $opts || exit 1
 
-# MinGW mod
-cat >>include/SDL_config.h <<_EOF
-/* do NOT allow ddraw! */
-#ifdef SDL_VIDEO_DRIVER_DDRAW
-#undef SDL_VIDEO_DRIVER_DDRAW
-#endif
-_EOF
-
 if [ "$1" == "hx-dos" ]; then
 cat >>include/SDL_config.h <<_EOF
 /* For HX-DOS, no parent window */
