@@ -96,6 +96,9 @@ DEVMODE SDL_fullscreen_mode;
 #endif
 WORD *gamma_saved = NULL;
 
+#if !defined(__MINGW32__) || defined(__MINGW64_VERSION_MAJOR)
+_IM_Context IM_Context; /* patched */
+#endif
 
 /* Functions called by the message processing function */
 LONG (*HandleMessage)(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)=NULL;
@@ -790,7 +793,7 @@ int SDL_RegisterApp(char *name, Uint32 style, void *hInst)
 #define CS_BYTEALIGNCLIENT	0
 #endif
 	if ( ! name && ! SDL_Appname ) {
-		name = "SDL_app";
+		name = "DOSBox-X";
 		SDL_Appstyle = CS_BYTEALIGNCLIENT;
 		SDL_Instance = hInst ? hInst : SDL_GetModuleHandle();
 	}
