@@ -91,6 +91,16 @@ static SDLKey Arrows_keymap[4];
 #endif
 static WNDPROCTYPE userWindowProc = NULL;
 
+#ifdef ENABLE_IM_EVENT
+#include <imm.h>
+int DIB_HandleComposition(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
+/* data field required by DIB_HandleComposition */
+static COMPOSITIONFORM form;
+extern Uint32 end_ticks;
+extern wchar_t CompositionFontName[LF_FACESIZE];
+#define	IME_END_CR_WAIT		50
+#endif
+
 
 #ifdef SDL_VIDEO_DRIVER_GAPI
 
@@ -176,6 +186,17 @@ int Win32_ShouldPassMessageToSysWMEvent(UINT msg) {
 	return 0;
 }
 
+#ifdef ENABLE_IM_EVENT
+#define FLIP_BREAK \
+	if (IM_Context.bFlip) { \
+		break; \
+	} else { \
+		return 0; \
+	}
+
+SDL_Event event_keydown;
+#endif
+
 /* The main Win32 event handler */
 LRESULT DIB_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 {
@@ -229,6 +250,8 @@ LRESULT DIB_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 				wParam = rotateKey(wParam, this->hidden->gapiInfo->coordinateTransform);
 			}
 #endif 
+			/* Ignore repeated keys */
+			// if ( lParam&REPEATED_KEYMASK ) return(0);
 			switch (wParam) {
 				case VK_CONTROL:
 					if ( lParam&EXTENDED_KEYMASK )
@@ -293,9 +316,16 @@ LRESULT DIB_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 				}
 			}
 #endif /* NO_GETKEYBOARDSTATE */
-			posted = SDL_PrivateKeyboard(SDL_PRESSED,
-				TranslateKey(wParam,HIWORD(lParam),&keysym,1));
-		}
+            {
+                SDL_keysym *key =  TranslateKey(wParam,HIWORD(lParam),&keysym,1);
+#ifdef ENABLE_IM_EVENT
+                if (!IM_Context.bCompos && (GetTickCount() - end_ticks > IME_END_CR_WAIT || key->sym != 0x0d))
+#endif
+                {
+                    posted = SDL_PrivateKeyboard(SDL_PRESSED,key);
+                }
+            }
+        }
 		return(0);
 
 		case WM_INITMENU:
@@ -371,6 +401,61 @@ LRESULT DIB_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 				TranslateKey(wParam,HIWORD(lParam),&keysym,0));
 		}
 		return(0);
+#if defined(ENABLE_IM_EVENT)
+		case WM_INPUTLANGCHANGE:
+			SendMessage(hwnd, WM_IME_NOTIFY, wParam, lParam);
+			FLIP_BREAK;
+		case WM_IME_STARTCOMPOSITION:
+			//if (!IM_Context.SDL_IMC)
+			//	IM_Context.SDL_IMC = ImmGetContext(hwnd);
+			{
+				HIMC imc = ImmGetContext(hwnd);
+				IM_Context.bCompos = 1;
+				form.dwStyle = CFS_POINT;
+				ImmSetCompositionWindow(imc, &form);
+				ImmReleaseContext(hwnd, imc);
+			}
+			return DefWindowProc(hwnd, msg, wParam, lParam);
+			//FLIP_BREAK;
+		case WM_IME_ENDCOMPOSITION:
+			end_ticks = GetTickCount();
+			IM_Context.bCompos = 0;
+			FLIP_BREAK;
+		case WM_IME_NOTIFY:
+			if (IM_Context.notify_func)/* && IM_Context.bEnable && ImmIsIME(GetKeyboardLayout(0)))*/
+				IM_Context.notify_func(IM_Context.notify_data);
+
+			// To flip state window in fullscreen mode
+			if (IM_Context.bFlip && IM_Context.bEnable) {
+				DefWindowProc(hwnd, WM_IME_NOTIFY, IMN_OPENSTATUSWINDOW, lParam);
+			}
+			FLIP_BREAK;
+		case WM_IME_SETCONTEXT:
+			// To show the composition window in fullscreen mode
+			if (IM_Context.bFlip && IM_Context.bEnable)
+			{
+				lParam |= ISC_SHOWUICOMPOSITIONWINDOW;
+				return DefWindowProc(hwnd, WM_IME_SETCONTEXT, wParam, lParam);
+			}
+			else
+			{
+				lParam &= ~ISC_SHOWUICOMPOSITIONWINDOW;
+				//if (IM_Context.bFlip)
+				return DefWindowProc(hwnd, WM_IME_SETCONTEXT, wParam, lParam);
+			}
+			FLIP_BREAK;
+		case WM_IME_COMPOSITION:
+			DIB_HandleComposition(hwnd, msg, wParam, lParam);
+
+			if (lParam & GCS_RESULTSTR) {
+				/* Send KEYDOWN event to fix some IME that doesn't sent key down event
+					after composition. */
+				event_keydown.type = SDL_KEYDOWN;
+				SDL_PushEvent(&event_keydown);
+				FLIP_BREAK;
+			}
+			break;
+#endif
 #if defined(SC_SCREENSAVE) && defined(SC_MONITORPOWER)
 		case WM_SYSCOMMAND: {
 			const DWORD val = (DWORD) (wParam & 0xFFF0);
@@ -384,7 +469,6 @@ LRESULT DIB_HandleMessage(_THIS, HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 		}
 		/* Fall through to default processing */
 #endif /* SC_SCREENSAVE && SC_MONITORPOWER */
-
 		default: {
 			/* Only post the event if we're watching for it */
 			if (SDL_ProcessEvents[SDL_SYSWMEVENT] == SDL_ENABLE) {
@@ -485,6 +569,13 @@ void DIB_PumpEvents(_THIS)
 
 	while ( PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE) ) {
 		if ( GetMessage(&msg, NULL, 0, 0) > 0 ) {
+#ifdef ENABLE_IM_EVENT
+			if (IM_Context.bEnable) {
+				if(msg.message != WM_SYSKEYDOWN && msg.message != WM_SYSKEYUP) {
+					TranslateMessage( &msg ); /* for IME */
+				}
+			}
+#endif
 			DispatchMessage(&msg);
 		}
 	}
@@ -675,6 +766,8 @@ void DIB_InitOSKeymapPriv(void) {
 			VK_keymap[VK_OEM_4] = SDLK_LEFTBRACKET;
 			VK_keymap[VK_OEM_6] = SDLK_RIGHTBRACKET;
 			VK_keymap[VK_OEM_5] = SDLK_JP_YEN;
+			VK_keymap[VK_CONVERT] = SDLK_WORLD_14;  // Henkan
+			VK_keymap[VK_NONCONVERT] = SDLK_WORLD_13; // Muhenkan
 			break;
 	}
 
@@ -1159,3 +1252,248 @@ void DIB_DestroyWindow(_THIS)
 	*/
 	WIN_FlushMessageQueue();
 }
+
+
+#ifdef ENABLE_IM_EVENT
+
+int DIB_HandleComposition(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
+{
+	if (lParam & GCS_RESULTSTR) {
+		HIMC imc = ImmGetContext(hWnd);
+
+		if (!imc) {
+		    IM_Context.im_compose_sz = 0;
+		    SDL_SetError("ERROR_NULLCONTEXT");
+		}
+
+		/* Get the size of the result string. */
+		if (SDL_TranslateUNICODE) {
+			IM_Context.im_compose_sz = ImmGetCompositionStringW(imc, GCS_RESULTSTR, NULL, 0);
+		} else {
+			IM_Context.im_compose_sz = ImmGetCompositionStringA(imc, GCS_RESULTSTR, NULL, 0);
+		}
+		if (!IM_Context.im_compose_sz) {
+			return 0;
+		}
+		else {
+			if (IM_Context.im_compose_sz >= IM_Context.im_buffer_sz) {
+				IM_Context.im_buffer_sz = IM_Context.im_compose_sz + 1*sizeof(wchar_t);
+				IM_Context.string.im_multi_byte_buffer = 
+					(char*)realloc(IM_Context.string.im_multi_byte_buffer,
+								IM_Context.im_buffer_sz);
+			}
+			IM_Context.string.im_wide_char_buffer[0] = '\0';
+		}
+
+		if (IM_Context.string.im_multi_byte_buffer) {
+		    /* Get the result strings that is generated by IME into lpstr. */
+			if (SDL_TranslateUNICODE) {
+				ImmGetCompositionStringW(imc, GCS_RESULTSTR, 
+					IM_Context.string.im_wide_char_buffer, IM_Context.im_buffer_sz);
+				IM_Context.string.im_wide_char_buffer[IM_Context.im_compose_sz/2] = '\0';
+			}
+			else {
+				ImmGetCompositionStringA(imc, GCS_RESULTSTR, 
+					IM_Context.string.im_multi_byte_buffer, IM_Context.im_buffer_sz);
+				IM_Context.string.im_multi_byte_buffer[IM_Context.im_compose_sz] = '\0';
+			}
+			ImmReleaseContext(hWnd, imc);
+		}
+		else {
+		    IM_Context.im_compose_sz = 0;
+	    }
+	}
+/*
+	if (lParam & GCS_COMPSTR) {
+		HIMC imc = ImmGetContext(hWnd);
+		if (!ImmSetCompositionWindow(imc, &form)) {
+			SDL_SetError("ImmSetCompositionWindow: fail to set composition form.\n");
+		}
+		ImmReleaseContext(hWnd, imc);
+	}
+*/
+	return 1;
+}
+
+int DIB_SetIMPosition(_THIS, int x, int y)
+{
+	form.dwStyle = CFS_POINT;;
+	form.ptCurrentPos.x = x;
+	form.ptCurrentPos.y = y;
+
+	HIMC imc = ImmGetContext(SDL_Window);
+//	if (!IM_Context.SDL_IMC)
+//		IM_Context.SDL_IMC = ImmGetContext(SDL_Window);
+
+//	if (!IM_Context.SDL_IMC)
+//		return 0;
+
+//	if (ImmSetCompositionWindow(IM_Context.SDL_IMC, &form)) {
+	if (ImmSetCompositionWindow(imc, &form)) {
+		ImmReleaseContext(SDL_Window, imc);
+		return 1;
+	}
+	ImmReleaseContext(SDL_Window, imc);
+	return 0;
+}
+
+char *DIB_SetIMValues(_THIS, SDL_imvalue value, int alt)
+{
+	switch (value) {
+		case SDL_IM_ENABLE:
+			if (alt) {
+				if(!IM_Context.bEnable) {
+					ImmAssociateContext(SDL_Window, IM_Context.SDL_IMC);
+					IM_Context.bEnable = 1;
+				}
+				return NULL;
+			}
+			else {
+				if(IM_Context.bEnable) {
+					IM_Context.SDL_IMC = ImmAssociateContext(SDL_Window, 0);
+					IM_Context.bEnable = 0;
+				}
+				return NULL;
+			}
+		case SDL_IM_FLIP:
+			if (alt) {
+				IM_Context.bFlip = 1;
+				SendMessage(SDL_Window, WM_IME_SETCONTEXT, TRUE, ISC_SHOWUIALL);
+				return NULL;
+			}
+			else {
+				IM_Context.bFlip = 0;
+				SendMessage(SDL_Window, WM_IME_SETCONTEXT, TRUE, 0);
+				return NULL;
+			}
+		case SDL_IM_ONOFF:
+			{
+				HIMC imc = ImmGetContext(SDL_Window);
+				ImmSetOpenStatus(imc, alt ? TRUE : FALSE);
+				ImmReleaseContext(SDL_Window, imc);
+			}
+			return NULL;
+		case SDL_IM_FONT_SIZE:
+			{
+				LOGFONTW lf;
+				HIMC imc = ImmGetContext(SDL_Window);
+				HDC hc = GetDC(SDL_Window);
+				HFONT hf = (HFONT)GetCurrentObject(hc, OBJ_FONT);
+				GetObjectW(hf, sizeof(lf), &lf);
+				ReleaseDC(SDL_Window, hc);
+				if(CompositionFontName[0]) {
+					wcscpy(lf.lfFaceName, CompositionFontName);
+				}
+				lf.lfHeight = -alt;
+				lf.lfWidth = alt / 2;
+				ImmSetCompositionFontW(imc, &lf);
+				ImmReleaseContext(SDL_Window, imc);
+			}
+			return NULL;
+		default:
+			SDL_SetError("SDL_SetIMValues: unknow enum type: %d", value);
+			return "SDL_SetIMValues: unknow enum type";
+	}
+	return NULL;
+}
+
+char *DIB_GetIMValues(_THIS, SDL_imvalue value, int *alt)
+{
+	switch (value) {
+		case SDL_IM_ENABLE:
+			if (IM_Context.bEnable) {
+				*alt = 1;
+				return NULL;
+			}
+			else {
+				*alt = 0;
+				return NULL;
+			}
+		case SDL_IM_FLIP:
+			if (IM_Context.bFlip) {
+				*alt = 1;
+				return NULL;
+			}
+			else {
+				*alt = 0;
+				return NULL;
+			}
+		case SDL_IM_ONOFF:
+			{
+				HIMC imc = ImmGetContext(SDL_Window);
+				if(ImmGetOpenStatus(imc)) {
+					*alt = 1;
+				} else {
+					*alt = 0;
+				}
+				ImmReleaseContext(SDL_Window, imc);
+				return NULL;
+			}
+		case SDL_IM_FONT_SIZE:
+			{
+				LOGFONTW lf;
+				HDC hc = GetDC(SDL_Window);
+				HFONT hf = (HFONT)GetCurrentObject(hc, OBJ_FONT);
+				GetObjectW(hf, sizeof(lf), &lf);
+				ReleaseDC(SDL_Window, hc);
+				*alt = abs(lf.lfHeight);
+			}
+			return NULL;
+		default:
+			SDL_SetError("DIB_GetIMValues: unknown enum type %d", value);
+			return "DIB_GetIMValues: unknown enum type";
+	}
+	return NULL;
+}
+
+int DIB_FlushIMString(_THIS, void *buffer)
+{
+	int result;
+
+	if (buffer && IM_Context.im_compose_sz) {
+		int len;
+		if (SDL_TranslateUNICODE) {
+			int i;
+			Uint16 *b = (Uint16*)buffer;
+			len = IM_Context.im_compose_sz / 2;
+			for (i = 0; i < len; ++i) 
+				b[i] = IM_Context.string.im_wide_char_buffer[i];
+		}
+		else {
+			len = IM_Context.im_compose_sz;
+			memcpy(buffer, IM_Context.string.im_multi_byte_buffer, len);
+		}
+
+		result = len;
+		IM_Context.im_compose_sz = 0;
+	}
+	else {
+		if (SDL_TranslateUNICODE)
+			result = IM_Context.im_compose_sz/2;
+		else
+			result = IM_Context.im_compose_sz;
+	}
+
+	return result;
+}
+
+#else /* !ENABLE_IM_EVENT */
+/* Fill with null implementation */
+
+int DIB_SetIMPosition(_THIS, int x, int y)
+{
+	return 0;
+}
+char *DIB_SetIMValues(_THIS, SDL_imvalue value, int alt)
+{
+	return NULL;
+}
+char *DIB_GetIMValues(_THIS, SDL_imvalue value, int *alt)
+{
+	return NULL;
+}
+int DIB_FlushIMString(_THIS, void *buffer)
+{
+	return 0;
+}
+#endif
